@@ -30,6 +30,7 @@ static void done_callback(), guess_callback(),
 	    adjust_callback(), dst_callback();
 
 extern Display		*display;	/* everybody uses the same server */
+extern Widget		toplevel;	/* top-level shell for icon name */
 extern struct edit	edit;		/* info about entry being edited */
 extern struct config	config;		/* global configuration data */
 extern XtAppContext	app;		/* application handle */
@@ -168,6 +169,8 @@ void destroy_adjust_popup()
 		XTDESTROYWIDGET(shell);
 		have_shell = FALSE;
 		mainlist->modified = TRUE;
+		write_mainlist();
+		resynchronize_daemon();
 	}
 }
 
@@ -189,8 +192,8 @@ void create_adjust_popup()
 	n = 0;
 	XtSetArg(args[n], XmNdeleteResponse,	XmDO_NOTHING);		n++;
 	XtSetArg(args[n], XmNiconic,		False);			n++;
-	shell = XtAppCreateShell("Adjust Time", "plan",
-			applicationShellWidgetClass, display, args, n);
+	shell = XtCreatePopupShell("Adjust Time", applicationShellWidgetClass,
+				   toplevel, args, n);
 #	ifdef EDITRES
 	XtAddEventHandler(shell, (EventMask)0, TRUE, 
  			_XEditResCheckMessages, NULL);
