@@ -35,6 +35,7 @@ static void filemenu_callback(), configmenu_callback(), search_callback(),
 	    list_by_user();
 
 extern Display		*display;	/* everybody uses the same server */
+extern Widget		toplevel;	/* top-level shell for icon name */
 extern GC		gc;		/* everybody uses this context */
 extern struct config	config;		/* global configuration data */
 extern struct plist	*mainlist;	/* list of all schedule entries */
@@ -43,6 +44,7 @@ extern Pixel		color[NCOLS];	/* colors: COL_* */
 extern int		curr_month;	/* month being displayed, 0..11 */
 extern int		curr_year;	/* year being displayed, since 1900 */
 extern time_t		curr_week;	/* week being displayed, time in sec */
+extern time_t		curr_day;	/* day being displayed, time in sec */
 Widget			mainwindow;	/* popup menus hang off main window */
 Widget			menubar;	/* menu bar at the top of main window*/
 Widget			mainview;	/* area in main window below menubar */
@@ -102,19 +104,22 @@ void create_cal_widgets(toplevel)
 	s[3] = XmStringCreateSimple("Print...");
 	s[4] = XmStringCreateSimple("About...");
 	s[5] = XmStringCreateSimple("Quit");
+	s[6] = XmStringCreateSimple("Ctrl-L");
+	s[7] = XmStringCreateSimple("Ctrl-P");
+	s[8] = XmStringCreateSimple("Ctrl-Q");
 	fmenu = XmVaCreateSimplePulldownMenu(menubar, "file", 0,
 			filemenu_callback,
 			FIX_MENUBAR
-			XmVaPUSHBUTTON,    s[0], 'F', NULL, NULL,
+			XmVaPUSHBUTTON,    s[0], 'F', "Ctrl<Key>L", s[6],
 			XmVaPUSHBUTTON,    s[1], 'R', NULL, NULL,
 			XmVaCASCADEBUTTON, s[2], 'D',
-			XmVaPUSHBUTTON,    s[3], 'P', NULL, NULL,
+			XmVaPUSHBUTTON,    s[3], 'P', "Ctrl<Key>P", s[7],
 			XmVaPUSHBUTTON,    s[4], 'A', NULL, NULL,
-			XmVaPUSHBUTTON,    s[5], 'Q', NULL, NULL,
+			XmVaPUSHBUTTON,    s[5], 'Q', "Ctrl<Key>Q", s[8],
 			NULL);
 	XtAddCallback(fmenu, XmNhelpCallback,
 					help_callback, (XtPointer)"pd_file");
-	for (n=0; n < 6; n++)
+	for (n=0; n < 9; n++)
 		XmStringFree(s[n]);
 
 	s[0] = XmStringCreateSimple("older than one day");
@@ -140,70 +145,83 @@ void create_cal_widgets(toplevel)
 	s[1] = XmStringCreateSimple("Adjust time...");
 	s[2] = XmStringCreateSimple("Alarm options...");
 	s[3] = XmStringCreateSimple("Define holidays...");
+	s[4] = XmStringCreateSimple("Ctrl-C");
 	menu = XmVaCreateSimplePulldownMenu(menubar, "config", 1,
 			configmenu_callback,
 			FIX_MENUBAR
-			XmVaPUSHBUTTON, s[0],  'C', NULL, NULL,
+			XmVaPUSHBUTTON, s[0],  'C', "Ctrl<Key>C", s[4],
 			XmVaPUSHBUTTON, s[1],  't', NULL, NULL,
 			XmVaPUSHBUTTON, s[2],  'o', NULL, NULL,
 			XmVaPUSHBUTTON, s[3],  'D', NULL, NULL,
 			NULL);
 	XtAddCallback(menu, XmNhelpCallback,
 					help_callback, (XtPointer)"pd_config");
-	for (n=0; n < 4; n++)
+	for (n=0; n < 5; n++)
 		XmStringFree(s[n]);
 
-	s[0] = XmStringCreateSimple("Today");
-	s[1] = XmStringCreateSimple("Tomorrow");
-	s[2] = XmStringCreateSimple("This week");
-	s[3] = XmStringCreateSimple("Next week");
-	s[4] = XmStringCreateSimple("This month");
-	s[5] = XmStringCreateSimple("All");
-	s[6] = XmStringCreateSimple("Private");
-	s[7] = XmStringCreateSimple("One file...");
-	s[8] = XmStringCreateSimple("Search keywords...");
+	s[ 0] = XmStringCreateSimple("Today");
+	s[ 1] = XmStringCreateSimple("Tomorrow");
+	s[ 2] = XmStringCreateSimple("This week");
+	s[ 3] = XmStringCreateSimple("Next week");
+	s[ 4] = XmStringCreateSimple("This month");
+	s[ 5] = XmStringCreateSimple("All");
+	s[ 6] = XmStringCreateSimple("Private");
+	s[ 7] = XmStringCreateSimple("One file...");
+	s[ 8] = XmStringCreateSimple("Search keywords...");
+	s[ 9] = XmStringCreateSimple("Ctrl-E");
+	s[10] = XmStringCreateSimple("Ctrl-I");
+	s[11] = XmStringCreateSimple("Ctrl-J");
+	s[12] = XmStringCreateSimple("Ctrl-S");
 	menu = XmVaCreateSimplePulldownMenu(menubar, "search", 2,
 			search_callback,
 			FIX_MENUBAR
-			XmVaPUSHBUTTON, s[0], 'T', NULL, NULL,
-			XmVaPUSHBUTTON, s[1], 'o', NULL, NULL,
-			XmVaPUSHBUTTON, s[2], 'w', NULL, NULL,
+			XmVaPUSHBUTTON, s[0], 'T', "Ctrl<Key>E", s[9],
+			XmVaPUSHBUTTON, s[1], 'o', "Ctrl<Key>I", s[10],
+			XmVaPUSHBUTTON, s[2], 'w', "Ctrl<Key>J", s[11],
 			XmVaPUSHBUTTON, s[3], 'N', NULL, NULL,
 			XmVaPUSHBUTTON, s[4], 'm', NULL, NULL,
 			XmVaPUSHBUTTON, s[5], 'A', NULL, NULL,
 			XmVaPUSHBUTTON, s[6], 'P', NULL, NULL,
 			XmVaPUSHBUTTON, s[7], 'f', NULL, NULL,
-			XmVaPUSHBUTTON, s[8], 'S', NULL, NULL,
+			XmVaPUSHBUTTON, s[8], 'S', "Ctrl<Key>S", s[12],
 			NULL);
 	XtAddCallback(menu, XmNhelpCallback,
 					help_callback, (XtPointer)"pd_search");
-	for (n=0; n < 9; n++)
+	for (n=0; n < 13; n++)
 		XmStringFree(s[n]);
 
-	s[0] = XmStringCreateSimple("Day");
-	s[1] = XmStringCreateSimple("Week");
-	s[2] = XmStringCreateSimple("Small Month");
-	s[3] = XmStringCreateSimple("Large Month");
-	s[4] = XmStringCreateSimple("Year");
-	s[5] = XmStringCreateSimple("Year Overview");
-	s[6] = XmStringCreateSimple("Goto today");
-	s[7] = XmStringCreateSimple("Goto...");
+	s[ 0] = XmStringCreateSimple("Day");
+	s[ 1] = XmStringCreateSimple("Week");
+	s[ 2] = XmStringCreateSimple("Small Month");
+	s[ 3] = XmStringCreateSimple("Large Month");
+	s[ 4] = XmStringCreateSimple("Year");
+	s[ 5] = XmStringCreateSimple("Year Overview");
+	s[ 6] = XmStringCreateSimple("Goto today");
+	s[ 7] = XmStringCreateSimple("Goto...");
+	s[ 8] = XmStringCreateSimple("Ctrl-D");
+	s[ 9] = XmStringCreateSimple("Ctrl-W");
+	s[10] = XmStringCreateSimple("Ctrl-N");
+	s[11] = XmStringCreateSimple("Ctrl-M");
+	s[12] = XmStringCreateSimple("Ctrl-Y");
+	s[13] = XmStringCreateSimple("Ctrl-O");
+	s[14] = XmStringCreateSimple("Ctrl-T");
+	s[15] = XmStringCreateSimple("Ctrl-G");
 	menu = XmVaCreateSimplePulldownMenu(menubar, "view", 3,
 			view_callback,
 			FIX_MENUBAR
-			XmVaPUSHBUTTON, s[0], 'D', NULL, NULL,
-			XmVaPUSHBUTTON, s[1], 'W', NULL, NULL,
-			XmVaPUSHBUTTON, s[2], 'S', NULL, NULL,
-			XmVaPUSHBUTTON, s[3], 'M', NULL, NULL,
-			XmVaPUSHBUTTON, s[4], 'Y', NULL, NULL,
-			XmVaPUSHBUTTON, s[5], 'O', NULL, NULL,
+			XmVaPUSHBUTTON, s[0], 'D', "Ctrl<Key>D", s[8],
+			XmVaPUSHBUTTON, s[1], 'W', "Ctrl<Key>W", s[9],
+			XmVaPUSHBUTTON, s[2], 'S', "Ctrl<Key>N", s[10],
+			XmVaPUSHBUTTON, s[3], 'M', "Ctrl<Key>M", s[11],
+			XmVaPUSHBUTTON, s[4], 'Y', "Ctrl<Key>Y", s[12],
+			XmVaPUSHBUTTON, s[5], 'O', "Ctrl<Key>O", s[13],
 			XM_VA_SEPARATOR
-			XmVaPUSHBUTTON, s[6], 't', NULL, NULL,
-			XmVaPUSHBUTTON, s[7], 'G', NULL, NULL,
+			XmVaPUSHBUTTON, s[6], 't', "Ctrl<Key>T", s[14],
+			XmVaPUSHBUTTON, s[7], 'G', "Ctrl<Key>G", s[15],
 			NULL);
 	XtAddCallback(menu, XmNhelpCallback,
 					help_callback, (XtPointer)"pd_view");
-	for (n=0; n < 8; n++)
+	for (n=0; n < 16; n++)
 		XmStringFree(s[n]);
 
 	s[0] = XmStringCreateSimple("On context");
@@ -214,10 +232,11 @@ void create_cal_widgets(toplevel)
 	s[5] = XmStringCreateSimple("Networking");
 	s[6] = XmStringCreateSimple("Database access");
 	s[7] = XmStringCreateSimple("X resources");
+	s[8] = XmStringCreateSimple("Ctrl-H");
 	(void)XmVaCreateSimplePulldownMenu(menubar, "help", 4,
 			help_pulldown,
 			FIX_MENUBAR
-			XmVaPUSHBUTTON, s[0], 'c', NULL, NULL,
+			XmVaPUSHBUTTON, s[0], 'c', "Ctrl<Key>H", s[8],
 			XM_VA_SEPARATOR
 			XmVaPUSHBUTTON, s[1], 'I', NULL, NULL,
 			XmVaPUSHBUTTON, s[2], 'G', NULL, NULL,
@@ -228,7 +247,7 @@ void create_cal_widgets(toplevel)
 			XmVaPUSHBUTTON, s[7], 'X', NULL, NULL,
 			NULL);
 
-	for (n=0; n < 8; n++)
+	for (n=0; n < 9; n++)
 		XmStringFree(s[n]);
 
 	XtManageChild(menubar);
@@ -325,8 +344,8 @@ Widget create_view_form(shell, parent, view, xs, ys, name)
 	} else {
 		XtSetArg(args[0], XmNdeleteResponse,	XmDO_NOTHING);
 		XtSetArg(args[1], XmNiconic,		False);
-		*shell = XtAppCreateShell(name, "plan",
-				applicationShellWidgetClass, display, args, 2);
+		*shell = XtCreatePopupShell(name, applicationShellWidgetClass,
+					    toplevel, args, 2);
 #		ifdef EDITRES
 		XtAddEventHandler(*shell, (EventMask)0, TRUE, 
 				_XEditResCheckMessages, NULL);
@@ -684,7 +703,8 @@ static void view_callback(widget, item, data)
 	  case 6:						/* goto today*/
 		curr_month = tm->tm_mon;
 		curr_year  = tm->tm_year;
-		curr_week  = time - 86400 *
+		curr_week  =
+		curr_day   = time - 86400 *
 				(tm->tm_wday - !config.sunday_first);
 		draw_month_year();
 		redraw_all_views();
