@@ -31,6 +31,7 @@ static void done_callback(), toggle_callback(), mailer_callback(),
 	    alarm_prog_callback();
 
 extern Display		*display;	/* everybody uses the same server */
+extern Widget		toplevel;	/* top-level shell for icon name */
 extern struct edit	edit;		/* info about entry being edited */
 extern struct config	config;		/* global configuration data */
 extern Pixel		color[NCOLS];	/* colors: COL_* */
@@ -116,8 +117,8 @@ void create_config_popup()
 	n = 0;
 	XtSetArg(args[n], XmNdeleteResponse,	XmDO_NOTHING);		n++;
 	XtSetArg(args[n], XmNiconic,		False);			n++;
-	shell = XtAppCreateShell("Alarm Options", "plan",
-			applicationShellWidgetClass, display, args, n);
+	shell = XtCreatePopupShell("Alarm Options",
+			applicationShellWidgetClass, toplevel, args, n);
 #	ifdef EDITRES
 	XtAddEventHandler(shell, (EventMask)0, TRUE, 
  			_XEditResCheckMessages, NULL);
