@@ -45,6 +45,9 @@
 #ifndef PID_T
 #define PID_T		pid_t		/* from types.h, see -DPID_T */
 #endif
+#ifndef S_IFDIR				/* some Linuces hide this */
+#define S_IFDIR		040000		/* I'll hate myself for this */
+#endif					/* in the morning... */
 
 #define BOOL		int
 #define TRUE		1
@@ -69,13 +72,14 @@
 #define PR_DAY		1
 #define PR_WEEK_L	2
 #define PR_WEEK_P	3
-#define PR_YOV		4
-#define PR_YEAR		5
+#define PR_YEAR		4
+#define PR_YOV		5
 
 
 struct config {			/* global configuration data */
 	time_t		tzone;		/* timezone with dst figured in */
 	BOOL		showicontime;	/* print time into icontitle if nz */
+	BOOL		showicondate;	/* print date+time into icontitle */
 	BOOL		noiconclock;	/* use icon to display time in clock */
 	BOOL		noicon;		/* don't draw anything into the icon */
 	BOOL		frame_today;	/* draw black frame into today's box */
@@ -93,7 +97,6 @@ struct config {			/* global configuration data */
 	BOOL		weeknum;	/* TRUE prints week numbers */
 	BOOL		gpsweek;	/* TRUE prints GPS week numbers */
 	BOOL		nopast;		/* TRUE skips todays past daybox notes*/
-	BOOL		bigwarning;	/* TRUE shows wide adv-warning column*/
 	BOOL		weekwarn;	/* TRUE shows warnings in week view */
 	BOOL		weekuser;	/* TRUE shows user names in week bars*/
 	BOOL		fullweek;	/* TRUE if week #1 is the one w 7 days*/
@@ -196,6 +199,7 @@ struct entry {			/* one day-schedule entry */
 	time_t		length;		/* length in seconds */
 	time_t		early_warn;	/* warn this many seconds in advance */
 	time_t		late_warn;	/* and this many seconds, too */
+	time_t		days_warn;	/* and this many days/86400 before */
 	time_t		except[NEXC];	/* don't trigger on these dates */
 	time_t		rep_every;	/* repeat every N days (in seconds) */
 	time_t		rep_last;	/* stop repeating on this date (secs)*/
@@ -254,6 +258,7 @@ struct lookup {
 					/* the next two are return values */
 	long		index;		/* the entry found when looking up */
 	time_t		trigger;	/* ... that entry's trigger time */
+	BOOL		days_warn;	/* this is an n-days-ahead warning */
 					/* the rest is for the next lookup */
 	struct plist	*list;		/* in which list were we looking */
 	long		regindex;	/* last regular entry found */
