@@ -29,6 +29,7 @@
 static void		flag_callback(), done_callback();
 
 extern Display		*display;	/* everybody uses the same server */
+extern Widget		toplevel;	/* top-level shell for icon name */
 extern struct config	config;		/* global configuration data */
 extern Pixel		color[NCOLS];	/* colors: COL_* */
 extern struct plist	*mainlist;	/* list of all schedule entries */
@@ -100,8 +101,8 @@ void create_calconfig_popup()
 	n = 0;
 	XtSetArg(args[n], XmNdeleteResponse,	XmDO_NOTHING);		n++;
 	XtSetArg(args[n], XmNiconic,		False);			n++;
-	shell = XtAppCreateShell("Calendar view configuration", "plan",
-			applicationShellWidgetClass, display, args, n);
+	shell = XtCreatePopupShell("Calendar view configuration",
+			applicationShellWidgetClass, toplevel, args, n);
 #	ifdef EDITRES
 	XtAddEventHandler(shell, (EventMask)0, TRUE, 
  			_XEditResCheckMessages, NULL);
@@ -183,7 +184,7 @@ void create_calconfig_popup()
 	XtSetArg(args[n], XmNset,		config.share_mainwin);	n++;
 	w = XtCreateManagedWidget("Use main window for all views",
 			xmToggleButtonWidgetClass, form2, args, n);
-	XtAddCallback(w, XmNvalueChangedCallback, flag_callback,(XtPointer)15);
+	XtAddCallback(w, XmNvalueChangedCallback, flag_callback,(XtPointer)14);
 	XtAddCallback(w, XmNhelpCallback, help_callback,
 						(XtPointer)"global_flags");
 
@@ -200,7 +201,7 @@ void create_calconfig_popup()
 	XtSetArg(args[n], XmNset,		config.resize_mainwin);	n++;
 	w = XtCreateManagedWidget("Auto-resize windows",
 			xmToggleButtonWidgetClass, form2, args, n);
-	XtAddCallback(w, XmNvalueChangedCallback, flag_callback,(XtPointer)16);
+	XtAddCallback(w, XmNvalueChangedCallback, flag_callback,(XtPointer)15);
 	XtAddCallback(w, XmNhelpCallback, help_callback,
 						(XtPointer)"global_flags");
 #endif
@@ -266,22 +267,6 @@ void create_calconfig_popup()
 	w = XtCreateManagedWidget("Auto-delete past entries",
 			xmToggleButtonWidgetClass, form2, args, n);
 	XtAddCallback(w, XmNvalueChangedCallback, flag_callback, (XtPointer)3);
-	XtAddCallback(w, XmNhelpCallback, help_callback,
-						(XtPointer)"global_flags");
-
-							/*-- fastwarn --*/
-	n = 0;
-	XtSetArg(args[n], XmNselectColor,	color[COL_TOGGLE]);	n++;
-	XtSetArg(args[n], XmNtopAttachment,	XmATTACH_WIDGET);	n++;
-	XtSetArg(args[n], XmNtopWidget,		w);			n++;
-	XtSetArg(args[n], XmNtopOffset,		4);			n++;
-	XtSetArg(args[n], XmNleftAttachment,	XmATTACH_FORM);		n++;
-	XtSetArg(args[n], XmNleftOffset,	8);			n++;
-	XtSetArg(args[n], XmNhighlightThickness,0);			n++;
-	XtSetArg(args[n], XmNset,		config.bigwarning);	n++;
-	w = XtCreateManagedWidget("Fast warning entry, no popup",
-			xmToggleButtonWidgetClass, form2, args, n);
-	XtAddCallback(w, XmNvalueChangedCallback, flag_callback, (XtPointer)4);
 	XtAddCallback(w, XmNhelpCallback, help_callback,
 						(XtPointer)"global_flags");
 
@@ -298,7 +283,7 @@ void create_calconfig_popup()
 	XtSetArg(args[n], XmNsensitive,		!all_files_served());	n++;
 	w = XtCreateManagedWidget("Only owner can write appt files",
 			xmToggleButtonWidgetClass, form2, args, n);
-	XtAddCallback(w, XmNvalueChangedCallback, flag_callback,(XtPointer)12);
+	XtAddCallback(w, XmNvalueChangedCallback, flag_callback,(XtPointer)11);
 	XtAddCallback(w, XmNhelpCallback, help_callback,
 						(XtPointer)"global_flags");
 
@@ -337,7 +322,7 @@ void create_calconfig_popup()
 	XtSetArg(args[n], XmNset,		config.julian);		n++;
 	w = XtCreateManagedWidget("Show Julian dates",
 			xmToggleButtonWidgetClass, form2, args, n);
-	XtAddCallback(w, XmNvalueChangedCallback, flag_callback, (XtPointer)5);
+	XtAddCallback(w, XmNvalueChangedCallback, flag_callback, (XtPointer)4);
 	XtAddCallback(w, XmNhelpCallback, help_callback,
 						(XtPointer)"month_flags");
 
@@ -353,7 +338,7 @@ void create_calconfig_popup()
 	XtSetArg(args[n], XmNset,		config.weeknum);	n++;
 	w = XtCreateManagedWidget("Show week numbers",
 			xmToggleButtonWidgetClass, form2, args, n);
-	XtAddCallback(w, XmNvalueChangedCallback, flag_callback, (XtPointer)6);
+	XtAddCallback(w, XmNvalueChangedCallback, flag_callback, (XtPointer)5);
 	XtAddCallback(w, XmNhelpCallback, help_callback,
 						(XtPointer)"month_flags");
 
@@ -369,7 +354,7 @@ void create_calconfig_popup()
 	XtSetArg(args[n], XmNset,		config.gpsweek);	n++;
 	w = XtCreateManagedWidget("Use GPS week numbers",
 			xmToggleButtonWidgetClass, form2, args, n);
-	XtAddCallback(w, XmNvalueChangedCallback, flag_callback,(XtPointer)14);
+	XtAddCallback(w, XmNvalueChangedCallback, flag_callback,(XtPointer)13);
 	XtAddCallback(w, XmNhelpCallback, help_callback,
 						(XtPointer)"month_flags");
 
@@ -385,7 +370,7 @@ void create_calconfig_popup()
 	XtSetArg(args[n], XmNset,		config.fullweek);	n++;
 	w = XtCreateManagedWidget("First week is first full week",
 			xmToggleButtonWidgetClass, form2, args, n);
-	XtAddCallback(w, XmNvalueChangedCallback, flag_callback, (XtPointer)7);
+	XtAddCallback(w, XmNvalueChangedCallback, flag_callback, (XtPointer)6);
 	XtAddCallback(w, XmNhelpCallback, help_callback,
 						(XtPointer)"month_flags");
 
@@ -401,7 +386,7 @@ void create_calconfig_popup()
 	XtSetArg(args[n], XmNset,		config.colorcode_m);	n++;
 	w = XtCreateManagedWidget("Colored background for other files",
 			xmToggleButtonWidgetClass, form2, args, n);
-	XtAddCallback(w, XmNvalueChangedCallback, flag_callback, (XtPointer)8);
+	XtAddCallback(w, XmNvalueChangedCallback, flag_callback, (XtPointer)7);
 	XtAddCallback(w, XmNhelpCallback, help_callback,
 						(XtPointer)"month_flags");
 
@@ -417,7 +402,7 @@ void create_calconfig_popup()
 	XtSetArg(args[n], XmNset,		config.nopast);		n++;
 	w = XtCreateManagedWidget("Don't show past entries in today's box",
 			xmToggleButtonWidgetClass, form2, args, n);
-	XtAddCallback(w, XmNvalueChangedCallback, flag_callback, (XtPointer)9);
+	XtAddCallback(w, XmNvalueChangedCallback, flag_callback, (XtPointer)8);
 	XtAddCallback(w, XmNhelpCallback, help_callback,
 						(XtPointer)"month_flags");
 
@@ -580,7 +565,7 @@ void create_calconfig_popup()
 	w = XtCreateManagedWidget("Show advance-warning times",
 			xmToggleButtonWidgetClass,
 			form2, args, n);
-	XtAddCallback(w, XmNvalueChangedCallback, flag_callback,(XtPointer)10);
+	XtAddCallback(w, XmNvalueChangedCallback, flag_callback,(XtPointer)9);
 	XtAddCallback(w, XmNhelpCallback, help_callback,
 						(XtPointer)"range_warn");
 
@@ -597,7 +582,7 @@ void create_calconfig_popup()
 	w = XtCreateManagedWidget("Show file names in bar labels",
 			xmToggleButtonWidgetClass,
 			form2, args, n);
-	XtAddCallback(w, XmNvalueChangedCallback, flag_callback,(XtPointer)11);
+	XtAddCallback(w, XmNvalueChangedCallback, flag_callback,(XtPointer)10);
 	XtAddCallback(w, XmNhelpCallback, help_callback,
 						(XtPointer)"range_user");
 
@@ -614,7 +599,7 @@ void create_calconfig_popup()
 	w = XtCreateManagedWidget("Large bars for week entries without time",
 			xmToggleButtonWidgetClass,
 			form2, args, n);
-	XtAddCallback(w, XmNvalueChangedCallback, flag_callback,(XtPointer)13);
+	XtAddCallback(w, XmNvalueChangedCallback, flag_callback,(XtPointer)12);
 	XtAddCallback(w, XmNhelpCallback, help_callback,
 						(XtPointer)"range_user");
 	XtPopup(shell, XtGrabNone);
@@ -624,8 +609,8 @@ void create_calconfig_popup()
 	XtSetArg(args[0], XmNwidth,  &xs);
 	XtSetArg(args[1], XmNheight, &ys);
 	XtGetValues(form, args, 2);
-	XtSetArg(args[0], XmNwidth,  xs);
-	XtSetArg(args[1], XmNheight, ys);
+	XtSetArg(args[0], XmNwidth,  xs+8);
+	XtSetArg(args[1], XmNheight, ys+8);
 	XtSetValues(shell, args, 2);
 
 	print_text_button(ndays, "%d", config.week_ndays);
@@ -653,10 +638,10 @@ static void done_callback(widget, item, data)
 
 static BOOL *flagptr[] = {
 	&config.sunday_first, &config.mmddyy, &config.ampm, &config.autodel,
-	&config.bigwarning, &config.julian, &config.weeknum, &config.fullweek,
-	&config.colorcode_m, &config.nopast, &config.weekwarn,&config.weekuser,
-	&config.owner_only, &config.week_bignotime, &config.gpsweek,
-	&config.share_mainwin, &config.resize_mainwin
+	&config.julian, &config.weeknum, &config.fullweek, &config.colorcode_m,
+	&config.nopast, &config.weekwarn,&config.weekuser, &config.owner_only,
+	&config.week_bignotime, &config.gpsweek, &config.share_mainwin,
+	&config.resize_mainwin
 };
 
 /*ARGSUSED*/
@@ -666,8 +651,6 @@ static void flag_callback(widget, item, data)
 	XmToggleButtonCallbackStruct	*data;
 {
 	*flagptr[item] ^= TRUE;
-	if (flagptr[item] == &config.bigwarning)
-		destroy_all_listmenus();
 	if (flagptr[item] == &config.owner_only) {
 		read_mainlist();
 		if (!config.owner_only)
