@@ -30,6 +30,7 @@ static void		done_callback(), wkday_callback(), day_callback(),
 			yearly_callback(), last_callback(),last_text_callback();
 
 extern Display		*display;	/* everybody uses the same server */
+extern Widget		toplevel;	/* top-level shell for icon name */
 extern struct edit	edit;		/* info about entry being edited */
 extern struct config	config;		/* global configuration data */
 extern Pixel		color[NCOLS];	/* colors: COL_* */
@@ -71,10 +72,9 @@ void destroy_recycle_popup()
 		}
 		edit.changed = TRUE;
 		edit.entry.suspended = FALSE;
-		print_pixmap(edit.menu->entry[edit.y][SC_RECYCLE],
-				edit.entry.rep_yearly || edit.entry.rep_days ||
-				edit.entry.rep_every || edit.entry.rep_weekdays
-							? PIC_RECYCLE : -1);
+		print_pixmap(edit.menu->entry[edit.y][SC_RECYCLE], PIC_RECYCLE,
+			     edit.entry.rep_yearly || edit.entry.rep_days ||
+			     edit.entry.rep_every  || edit.entry.rep_weekdays);
 	}
 	if (have_shell) {
 		XtPopdown(shell);
@@ -108,8 +108,8 @@ void create_recycle_popup()
 	n = 0;
 	XtSetArg(args[n], XmNdeleteResponse,	XmDO_NOTHING);		n++;
 	XtSetArg(args[n], XmNiconic,		False);			n++;
-	shell = XtAppCreateShell("Schedule Recycler", "plan",
-			applicationShellWidgetClass, display, args, n);
+	shell = XtCreatePopupShell("Schedule Recycler",
+			applicationShellWidgetClass, toplevel, args, n);
 #	ifdef EDITRES
 	XtAddEventHandler(shell, (EventMask)0, TRUE, 
  			_XEditResCheckMessages, NULL);
@@ -567,7 +567,7 @@ static void last_text_callback(widget, item, data)
 	Arg				args;
 
 	edit.entry.rep_last = parse_datestring(text, edit.entry.time);
-	if (edit.entry.rep_last < edit.entry.time) {
+	if (edit.entry.rep_last < edit.entry.time + 86400-1) {
 		char *p, date[40];
 		strcpy(date, mkdatestring(edit.entry.time));
 		p = strchr(date, ' ');
