@@ -23,7 +23,9 @@ typedef unsigned long XtInputId;
 #endif
 #include <stdio.h>
 #include <errno.h>
-#ifndef VARARGS
+#ifdef VARARGS
+#include <varargs.h>
+#else
 #include <stdarg.h>
 #endif
 #include <sys/types.h>
@@ -34,6 +36,7 @@ typedef unsigned long XtInputId;
 #include <string.h>
 #include "config.h"
 #include "proto.h"
+#include "version.h"
 #include <pwd.h>
 #include <utmp.h>
 #ifdef AIXV3
@@ -81,13 +84,14 @@ struct plist		*mainlist;	/* list of all schedule entries */
 extern time_t		cutoff;		/* all triggers before this are done */
 static BOOL		reread;		/* caught SIGHUP, re-read mainlist */
 static BOOL		debug;		/* print debugging information */
+static BOOL		opt_t;		/* TRUE if -t (tty) */
 extern int		errno;		/* system error */
 static void sighand(), alert(), usage();
 
-int write_one_entry() {}		/* never used, for network.o */
+int write_one_entry() {return(0);}	/* never used, for network.o */
 int curr_year;				/* dummy for DST calculating routines*/
 Widget mainwindow;			/* dummy for error popup in network.c*/
-XtInputId register_X_input() {}		/* more dummies */
+XtInputId register_X_input(){return(0);}/* more dummies */
 void unregister_X_input() {}
 
 
@@ -101,11 +105,14 @@ main(argc, argv)
 	time_t			total_wait;
 	time_t			ancient=ANCIENT;/* initially accept past trig*/
 	struct entry		*entry;		/* next entry that triggers */
-	int			opt_k = FALSE;	/* TRUE if -k (kill & run) */
-	int			opt_K = FALSE;	/* TRUE if -K (kill & exit) */
-	int			opt_l = TRUE;	/* TRUE if -l (watch logout) */
+	BOOL			opt_k = FALSE;	/* TRUE if -k (kill & run) */
+	BOOL			opt_K = FALSE;	/* TRUE if -K (kill & exit) */
+	BOOL			opt_l = TRUE;	/* TRUE if -l (watch logout) */
+	BOOL			opt_s = FALSE;	/* TRUE if -s (silent) */
 	struct utmp		*u;		/* for scanning /etc/utmp */
 	struct passwd		*pw = 0;	/* user's password entry */
+	BOOL			got_login=FALSE;/* was user ever logged in? */
+	BOOL			logged_in=TRUE;	/* found in utmp? */
 #ifdef ITIMER
  	struct itimerval	itv;		/* for sleep() replacement */
 #endif
@@ -123,14 +130,16 @@ main(argc, argv)
 				  case 'd': debug = TRUE;  break;
 				  case 'l': opt_l = TRUE;  break;
 				  case 'L': opt_l = FALSE; break;
+				  case 's': opt_s = TRUE;  break;
+				  case 't': opt_t = TRUE;  break;
 				  default:  usage();
 				}
 		} else
 			usage();
 
-	if (opt_l && !(pw = getpwuid(getuid())))
+	if ((opt_l || opt_s) && !(pw = getpwuid(getuid())))
 		fprintf(stderr, "%s: WARNING: can't read user name.\n%s",
-				progname, "Try running without -l option.\n");
+				progname, "Try running with -L option.\n");
 	if (!debug) {
 		(void)umask(0077);
 		setuid(getuid());
@@ -179,99 +188,97 @@ main(argc, argv)
 	itv.it_value.tv_usec = 0;
 #endif
 	for (;;) {
+	    if (reread) {
+		if (debug)
+		    printf("%s: re-reading database\n", progname);
+		destroy_list(&mainlist);
+		create_list(&mainlist);
+		read_mainlist();
+		(void)recycle_all(mainlist, FALSE, 0);
+		reread = FALSE;
+	    }
+	    (void)recycle_all(mainlist, FALSE, 0);
+
+	    if (!(opt_s && !logged_in)) {
 		set_tzone();
 		now = get_time();
 		reason = find_next_trigger(mainlist, now, &entry, &twait);
 		if (reason && twait <= 0) {
-			alert(entry, reason);
-			entry->triggered = reason;
-		} else {
-		    cutoff = now - ancient;
-		    ancient = 0;
-		    if (!reason)
-			    twait = HIBERNATE;
-		    total_wait = twait;
-		    while (total_wait && !reread) {
-			twait = total_wait>10*60 ? 10*60 : total_wait;
-			if (debug)
-				printf("%s: sleeping for %d:%02d:%02d\n",
-					progname,
-					(int)twait/3600, (int)(twait/60)%60,
-					(int)twait%60);
+		    alert(entry, reason);
+		    entry->triggered = reason;
+		    continue;
+		}
+	    }
+	    cutoff = now - ancient;
+	    ancient = 0;
+	    if (!reason)
+		twait = HIBERNATE;
+	    total_wait = twait;
+	    while (total_wait && !reread) {
+		twait = total_wait>1*60 ? 1*60 : total_wait;
+		if (debug)
+		    printf("%s: sleeping for %d:%02d:%02d\n", progname,
+			   (int)twait/3600, (int)(twait/60)%60, (int)twait%60);
 #ifdef ITIMER
-			itv.it_value.tv_sec = twait;
-			setitimer(ITIMER_REAL, &itv, NULL);
+		itv.it_value.tv_sec = twait;
+		setitimer(ITIMER_REAL, &itv, NULL);
 #if defined(SVR4) || defined(SOLARIS2)
-			sigpause(SIGALRM);
+		sigpause(SIGALRM);
 #else
-			sigpause(0);
+		sigpause(0);
 #endif
 #else
-			sleep(twait);
+		sleep(twait);
 #endif
-			refresh_lock(LOCK_PATH);
-#ifdef SUN
-			if (pw) {
-			    struct utmp	ut;
-			    int		fd;
-			    int		l;
-			    short	pid = getpid();
+		refresh_lock(LOCK_PATH);
+		logged_in = FALSE;
+		if (pw) {
+#if defined(SUN) || defined(BSD) || defined(__FreeBSD__)
+		    struct utmp	ut;
+		    int		fd;
+		    int		l;
+		    short		pid = getpid();
 
-			    fd = open("/etc/utmp",0);
-			    if (fd >= 0) {
-				while ((l=read(fd,&ut,sizeof(ut)))==sizeof(ut)){
-				    if (ut.ut_name[0]			&&
-				       (!strncmp(ut.ut_line, "tty", 3)	&&
-						(ut.ut_line[3] == 'p' ||
-						 ut.ut_line[3] == 'q' ||
-						 ut.ut_line[3] == 'r' ||
-						 ut.ut_line[3] == 's') ||
-					!strncmp(ut.ut_line, "console", 7) &&
-					!strncmp(pw->pw_name, ut.ut_name, 8))){
-					    break;
-				    }
-				}
-				if (l != sizeof(ut)) {
-				    printf("%s: logout, exiting\n", progname);
-				    if (debug)
-					printf("%s: if this message is spurious, you may have to run with -L.\n",
-								progname);
-				    exit(0);
-				}
-				close(fd);
+		    fd = open("/etc/utmp", 0);
+		    if (fd >= 0) {
+			while ((l = read(fd, &ut, sizeof(ut))) == sizeof(ut)) {
+			    if (ut.ut_name[0]			&&
+			       (!strncmp(ut.ut_line, "tty", 3)	&&
+			        strchr("pqrs", ut.ut_line[3]) ||
+				!strncmp(ut.ut_line, "console", 7) &&
+				!strncmp(pw->pw_name, ut.ut_name, 8))){
+				    logged_in = TRUE;
+				    break;
 			    }
 			}
+			close(fd);
+		    }
 #else
-			if (pw) {
-			    short pid = getpid();
-			    setutent();
-			    while (u = getutent())
-				if (u->ut_type == USER_PROCESS &&
-				    u->ut_pid != pid &&
-				    !strncmp(pw->pw_name, u->ut_user, 8))
-					break;
-			    if (!u) {
-				printf("%s: logout, exiting\n", progname);
-				if (debug)
-					printf("%s: if this message is spurious, you may have to run with -L.\n",
-								progname);
-				exit(0);
-			    }
+		    short pid = getpid();
+		    setutent();
+		    while (u = getutent())
+			if (u->ut_type == USER_PROCESS &&
+			    u->ut_pid != pid &&
+			    !strncmp(pw->pw_name, u->ut_user, 8)) {
+				logged_in = TRUE;
+				break;
 			}
 #endif
-			total_wait -= twait;
-		    }
 		}
-		if (reread) {
-			if (debug)
-				printf("%s: re-reading database\n", progname);
-			destroy_list(&mainlist);
-			create_list(&mainlist);
-			read_mainlist();
-			(void)recycle_all(mainlist, FALSE, 0);
-			reread = FALSE;
+		if (got_login && !logged_in && !opt_s) {
+		    printf("%s: you have logged out, exiting (try -L?)\n",
+							    progname);
+		    exit(0);
 		}
-		(void)recycle_all(mainlist, FALSE, 0);
+		if (logged_in) {
+		    got_login = TRUE;
+		    if (debug)
+			printf("%s: utmp works, watching for logout",
+							    progname);
+		} else if (opt_s && debug)
+			printf("%s: logged out, keeping quiet", progname);
+		total_wait -= twait;
+	    }
 	}
 }
 
@@ -349,6 +356,11 @@ static void popup_window(reason, entry)
 	char			prog[1024];	/* path of notifier program */
 	char			cmd[2048];	/* command string */
 
+	if (opt_t) {
+		printf("*** %s\n*** %s\n", get_subject(reason, entry, FALSE),
+					   get_subtitle(entry));
+		return;
+	}
 	if (!find_file(prog, ALARM_FN, TRUE)) {
 		fprintf(stderr, "%s: %s: not found\n", progname, ALARM_FN);
 		return;
@@ -411,7 +423,7 @@ static void reaper(sig)				/* lay dead children to rest */
 	int sig;
 {
 	char			path[40];	/* script to delete */
-#if (defined BSD && !defined OSF)
+#if (defined BSD && !defined OSF && !defined __FreeBSD__)
 	union wait		dummy;
 #else
 	int			dummy;
@@ -981,13 +993,16 @@ void create_error_popup(widget, error, fmt, a, b, c, d)
 
 static void usage()
 {
-	fprintf(stderr, "%s: plan calendar daemon. Usage: pland [-dkKlL]\n",
-			progname);
-	fprintf(stderr, "-d	debug: stay in foreground, print actions\n");
-	fprintf(stderr, "-k	kill existing daemon, then continue\n");
-	fprintf(stderr, "-K	kill existing daemon only\n");
-	fprintf(stderr, "-l	watch utmp and exit when user logs out\n");
-	fprintf(stderr, "-L	don't watch utmp (utmp is often buggy)\n");
+	fprintf(stderr, "%s: plan calendar daemon, version %s.\n",
+							progname, VERSION);
+	fprintf(stderr, "Usage: pland [-dkKlL]\n");
+	fprintf(stderr, "-d\tdebug: stay in foreground, print actions\n");
+	fprintf(stderr, "-k\tkill existing daemon, then continue\n");
+	fprintf(stderr, "-K\tkill existing daemon only\n");
+	fprintf(stderr, "-t\tdon't pop up notifier windows, print to stdout\n");
+	fprintf(stderr, "-s\twatch utmp and sleep while user is logged out\n");
+	fprintf(stderr, "-l\twatch utmp and exit when user logs out\n");
+	fprintf(stderr, "-L\tdon't watch utmp (utmp is often buggy)\n");
 	fprintf(stderr,
 #	ifdef RABBITS
 		"-L is the default. Try -l if you get too many pland's.\n");
