@@ -2,7 +2,7 @@
  * draws the day view. PostScript printing is also here because it uses
  * the same high-level routines, only the low-level output is diverted.
  *
- *	clicked_day_calendar(x, y)	Press on the day display detected.
+ *	locate_in_day_calendar()	find out where we are in the calendar
  *	draw_day_calendar(region)	If there is a day menu, resize, and
  *					redraw
  *	print_day_calendar(fp,l)	print PostScript day to fp
@@ -25,8 +25,9 @@
 static void gen_day(), draw_bar(), draw_octagon(), draw_boxtext();
 
 #ifdef JAPAN
-int truncate_strpack();
-void g_draw_strpack();
+extern int truncate_strpack();
+extern int mixedstrlen_in_pixels();
+extern void g_draw_strpack();
 #endif
 
 extern Display		*display;	/* everybody uses the same server */
@@ -45,91 +46,112 @@ extern char *monthname[];
 
 
 /*
- * got a click in the calendar area. Traverse the node tree, first down until
- * the correct line is found, and then horizontally until the node is found.
+ * for a given position on the canvas, identify the entry there and return
+ * a pointer to it and which area it was hit in (one of the M_ constants).
+ * Locate an event if epp!=0, and return exactly where the event was hit.
+ * If none was found, return the time if possible for creating new events.
+ * If epp is 0, we are dragging an event right now; just return a new time.
+ *
+ *   ret	*epp	*time	*trig	xp..ysp	event
+ *
+ *   M_ILLEGAL	0	0	-	-	no recognizable position
+ *   M_OUTSIDE	0	time	-	-	no event but in a valid day
+ *   M_INSIDE	entry	time	trigg	-	in event center, edit it
+ *   M_LEFT	entry	time	trigg	pos/sz	near top edge, move start date
+ *   M_RIGHT	entry	time	trigg	pos/sz	near bottom edge, move length
  */
 
-void clicked_day_calendar(xc, yc, time)
+MOUSE locate_in_day_calendar(epp, warn, time, trigger, xp,yp,xsp,ysp, xc,yc)
+	struct entry		**epp;		/* returned entry or 0 */
+	BOOL			*warn;		/* is *epp an n-days-ahead w?*/
+	time_t			*time;		/* if *epp=0, clicked where */
+	time_t			*trigger;	/* if *epp=0, entry trigger */
+	int			*xp,  *yp;	/* if M_INSIDE, rubber box */
+	int			*xsp, *ysp;	/* position and size */
 	int			xc, yc;		/* pixel pos clicked */
-	Time			time;		/* current time in ms */
 {
 	register struct config	*c = &config;
-	struct weeknode		*vert, *horz;	/* day node scan pointers */
+	struct weeknode		*vert, *horz;	/* yov node scan pointers */
 	register struct entry	*ep;		/* entry to test */
-	static struct entry	*last_ep;	/* last entry found (dblclk?)*/
-	static Time		last_time;	/* time of last press */
-	int			d;		/* day counter */
+	int			d;		/* day strip counter */
 	int			x0, y0;		/* XY start coord of day box*/
 	int			x, y;		/* upper left of day bk box */
 	int			xs;		/* width of bar col w/ gaps */
-	time_t			btime;		/* start time of bar */
-	int			b, e, l;	/* Y start end end pos of bar*/
+	time_t			bt;		/* start time of bar */
+	int			b, e;		/* Y start end end pos of bar*/
 	int			yend;		/* bottom margin of chart */
-	int			u;		/* index into user[] array */
 
 	x0 = c->day_margin + c->day_hourwidth + 2;
-	y0 = c->day_margin + c->day_title + c->day_headline + 2;
+	y0 = c->day_margin + c->day_title + 2 * c->day_headline + 2;
 	xs = c->day_barwidth + c->day_gap;
-	yend = y + c->day_hourheight * (c->day_maxhour - c->day_minhour);
+	yend = y0 + c->day_hourheight * (c->day_maxhour - c->day_minhour);
 
-	for (d=0; d < c->day_ndays; d++) {
-	    x = x0;
-	    for (y=y0, vert=day.tree[d]; vert; vert=vert->down, x+=xs) {
-		if (xc < x || xc >= x+xs)
-			continue;
-		if (yc < y) {
-			create_list_popup(mainlist, curr_day + d*86400,
-							86400, 0, 0, 0, 0);
-			return;
+	if (yc > yend || yc < y0 || xc < x0)
+		return(M_ILLEGAL);
+	if (warn)
+		*warn = FALSE;
+						/* calc time in calendar */
+	*time = 0;
+	for (x=x0, d=0; d < c->day_ndays; d++) {
+		if (xc < x)
+			break;
+		x += day.nlines[d] * xs + c->day_margin;
+		if (xc < x) {
+			*time = curr_day + d * 86400 + c->day_minhour * 3600 +
+					(yc - y0) * 3600 / c->day_hourheight;
+			break;
 		}
-		for (horz=vert; horz; horz=horz->next) {
-			u  = name_to_user(horz->entry->user);
-			ep = horz->entry;
-			btime = BOUND(TOD(ep->time), c->day_minhour*3600,
+	}
+						/* find entry under mouse */
+	if (epp)
+	    for (d=0; d < c->day_ndays; d++) {
+		for (x=x0,y=y0,vert=day.tree[d]; vert; vert=vert->down,x+=xs) {
+		    if (xc < x || xc >= x+xs)
+			continue;
+		    if (yc < y)
+			return(M_OUTSIDE);
+		    for (horz=vert; horz; horz=horz->next) {
+			ep = *epp = horz->entry;
+			bt = BOUND(TOD(ep->time), c->day_minhour*3600,
 						     c->day_maxhour*3600);
-			e  = BOUND(y + YPOS(btime), y0, yend);
+			e  = BOUND(y + YPOS(bt), y0, yend);
 			if (config.weekwarn)
-				btime -= ep->early_warn > ep->late_warn
-						? ep->early_warn
-						: ep->late_warn;
-			b = BOUND(y + YPOS(btime), y0, yend);
+				bt -= ep->early_warn > ep->late_warn ?
+					ep->early_warn : ep->late_warn;
+			b = BOUND(y + YPOS(bt), y0, yend);
 			if (ep->notime)
 				e = config.week_bignotime
 					? y + YPOS(c->day_maxhour*3600)
 					: b + YPOS((c->day_minhour+1)*3600);
 			else {
-				l  = ep->length * c->day_hourheight/3600;
+				int l = ep->length * c->day_hourheight/3600;
 				e += BOUND(l, MINLEN, 9999);
 			}
-			if (yc >= b && yc < e ||
-			    yc >= yend-MINLEN && b >= yend) {
-				if (time-last_time < DBLTIME && ep == last_ep){
-					if (u >= 0 && user[u].readonly)
-						print_button(day.info,
-						 "Cannot edit, no permission");
-					else
-						create_list_popup(mainlist,
-							0, 0, 0, ep, 0, 0);
-					last_ep = 0;
-					return;
-				} else if (horz->entry->user)
-					print_button(day.info, "%s %s: %s",
-						mktimestring(ep->time, FALSE),
-						horz->entry->user,
-						ep->note ? ep->note : "");
-				else
-					print_button(day.info, "%s %s",
-						mktimestring(ep->time, FALSE),
-						ep->note ? ep->note : "");
-				last_ep   = ep;
-				last_time = time;
-				return;
+			if (trigger)
+				*trigger = horz->trigger;
+			if (warn)
+				*warn = horz->days_warn;
+			if (xp) {
+				*xp   = x + 1;
+				*yp   = b + 1;
+				*xsp  = c->day_barwidth & ~1;
+				*ysp  = e - b - 1;
 			}
+			if (yc < b - 1)
+				continue;
+			if (yc < b + (e-b)/3)
+				return(M_LEFT);
+			if (yc < b + (e-b)*2/3)
+				return(M_INSIDE);
+			if (yc < e + 1)
+				return(M_RIGHT);
+		    }
 		}
+		x0 += day.nlines[d] * xs + c->day_margin;
 	    }
-	    x0 += day.nlines[d] * xs + c->day_margin;
-	}
-	print_button(day.info, " ");
+	if (epp)
+		*epp = 0;
+	return(*time ? M_OUTSIDE : M_ILLEGAL);
 }
 
 
@@ -186,9 +208,15 @@ static void gen_day()
 	int			dx, dy;		/* size inside day line box */
 	int			cy_h, cy_d;	/* character height */
 	struct weeknode		*vert, *horz;	/* day node scan pointers */
+	struct holiday		*hp, *shp;	/* to check for holidays */
+	struct tm		*tm;		/* today's date as m/d/y */
+#ifdef JAPAN
+	strpack			partialstr[MAXPARTIALSTRING];
+	unsigned char		strpool[MAXPARTIALCHAR];
+#endif
 
 	x0 = c->day_margin;
-	y0 = c->day_margin + c->day_title + c->day_headline + 2;
+	y0 = c->day_margin + c->day_title + 3*c->day_headline + 2;
 	xs = c->day_barwidth + c->day_gap;
 	dy = c->day_hourheight * (c->day_maxhour - c->day_minhour);
 	cy_h = font[FONT_WHOUR]->max_bounds.ascent;
@@ -199,7 +227,7 @@ static void gen_day()
 	strcat(buf, mkdatestring(curr_day + (c->day_ndays-1)*86400));
 	g_setcolor(COL_WTITLE);
 	g_setfont(FONT_WTITLE);
-	g_drawtext((day.canvas_xs - strlen_in_pixels(buf,FONT_WTITLE))/2, 
+	g_drawtext((day.canvas_xs - strlen_in_pixels(buf, FONT_WTITLE))/2, 
 		   c->day_margin + c->day_title * 3/4,
 		   buf, strlen(buf));
 
@@ -213,23 +241,45 @@ static void gen_day()
 			   p, strlen(p));
 	}
 	x0 += c->day_hourwidth + 2;
+								/* holidays */
+	tm = time_to_tm(curr_day);
+	if (p = parse_holidays(tm->tm_year, FALSE))
+		create_error_popup(mainmenu.cal, 0, p);
+	shp = &sm_holiday[tm->tm_yday];
+	hp  =    &holiday[tm->tm_yday];
 
-	for (d=0; d < c->day_ndays; d++) {			/* day loop */
+	for (d=0; d < c->day_ndays; d++, hp++, shp++) {		/* day loop */
 		if (!(dx = day.nlines[d] * xs))
 			continue;
 
 		today = get_time() - (curr_day + d*86400);	/* green? */
 		if (today >= 0 && today < 86400) {
 			g_setcolor(COL_CALTODAY);
-			g_fillrect(x0-2, y0-2-c->day_headline,
-				   dx+4, c->day_headline);
+			g_fillrect(x0-2, y0-2-2*c->day_headline,
+				   dx+4, 2*c->day_headline);
 		}
 		g_setcolor(COL_WDAY);				/* date */
 		g_setfont(FONT_WHOUR);
 		p = mkdatestring(curr_day + d * 86400);
 		truncate_string(p, dx, FONT_WHOUR);
-		g_drawtext(x0, y0-4, p, strlen(p));
-
+		g_drawtext(x0, y0-4-c->day_headline, p, strlen(p));
+								/* holiday */
+		if (p = hp->string ? hp->string : shp->string) {
+			g_setcolor(shp->stringcolor ? shp->stringcolor :
+				   shp->daycolor    ? shp->daycolor
+						    : COL_WDAY);
+#ifdef JAPAN
+			partialstr->strptr = strpool;
+			if (mixedstrlen_in_pixels(shp->string,
+				      partialstr, FONT_WNOTE, FONT_JNOTE) > dx)
+				truncate_strpack(partialstr, dx, FONT_WNOTE);
+			g_draw_strpack(x0, y0-4, partialstr);
+#else
+			strncpy(buf, shp->string, sizeof(buf)-1);
+			truncate_string(buf, dx, FONT_WDAY);
+			g_drawtext(x0, y0-4, p, strlen(p));
+#endif
+		}
 		g_setcolor(COL_WBOXBACK);			/* box bgnd */
 		g_fillrect(x0,    y0,    dx,   dy);
 
@@ -288,19 +338,20 @@ static void draw_bar(node, color, x, y)
 		e = config.week_bignotime ? y + YPOS(c->day_maxhour*3600)
 					  : b + YPOS((c->day_minhour+1)*3600);
 	}
-	if (config.weekwarn && !ep->notime)
+	if (config.weekwarn && !ep->notime && !node->days_warn)
 		for (i=0; i < 2; i++)
-			draw_octagon(COL_WWARN, x, w[i], e, yend);
-
-	draw_octagon(ep->suspended ? COL_WWARN : COL_WUSER_0 + color,
-						x, b, e, yend);
+			draw_octagon(COL_WWARN, FALSE, x, w[i], e, yend);
 
+	draw_octagon(ep->suspended || node->days_warn ? COL_WWARN :
+			COL_WUSER_0 + (color & 7), color > 7, x, b, e, yend);
 	*msg = 0;
 	if (!ep->notime) {
 		strcpy(msg, mktimestring(node->trigger, FALSE));
 		if (ep->length)
 			sprintf(msg+strlen(msg), "-%s", mktimestring(
 					node->trigger + ep->length, FALSE));
+		if (node->days_warn)
+			strcat(msg, " (warn)");
 	}
 	if (ep->user && strcmp(ep->user, user[0].name))
 		sprintf(msg+strlen(msg), " (%s)", ep->user);
@@ -320,8 +371,9 @@ static void draw_bar(node, color, x, y)
  * from the topmost vertex at the left edge (10-o'clock position)
  */
 
-static void draw_octagon(color, x, y, ye, yend)
+static void draw_octagon(color, checker, x, y, ye, yend)
 	int	color;		/* color, 0..7 */
+	BOOL	checker;	/* add checkerboard pattern */
 	int	x, y;		/* top left XY pos in canvas */
 	int	ye;		/* Y bottom in canvas */
 	int	yend;		/* bottom Y clip limit */
@@ -353,7 +405,7 @@ static void draw_octagon(color, x, y, ye, yend)
 	point[3].y = BOUND(ye, y, yend);
 
 	g_setcolor(color);
-	g_drawpoly(point, 8);
+	g_drawpoly(point, 8, checker);
 	g_setcolor(COL_WFRAME);
 	g_drawlines(point, 9);
 }
@@ -364,11 +416,7 @@ static void draw_octagon(color, x, y, ye, yend)
  * multiple lines. Use the small note font.
  */
 
-#ifdef JAPAN
-#define BOXFONT FONT_JNOTE
-#else
 #define BOXFONT FONT_NOTE
-#endif
 #define ISSPACE(c) ((c)==' ' || (c)=='\t')
 #define MAXSPLIT 6
 
@@ -382,15 +430,100 @@ static void draw_boxtext(x, y, xs, ys, text, center)
 	int		len, len_s;	/* curr text length in pixels */
 	int		cys, lys;	/* character height, line height */
 	int		i;
+#ifdef JAPAN
+	int		n, plen;	/* pixel-length of partialstr */
+	strpack		partialstr[MAXPARTIALSTRING], save, saveasc;
+	unsigned char	strpool[MAXPARTIALCHAR];
+#endif
 
 	g_setfont(BOXFONT);
 	cys = font[BOXFONT]->max_bounds.ascent;
+#ifdef JAPAN
+	if (cys < font[FONT_JNOTE]->max_bounds.ascent)
+	  cys = font[FONT_JNOTE]->max_bounds.ascent;
+#endif
 	lys = cys * 5/4;
 	i   = (ys-cys) / lys;
 	y  += (ys - cys - i*lys) / 2 + cys;
 	x  += 3;
 	xs -= 5;
 
+#ifdef JAPAN
+	partialstr->strptr = strpool;
+	(void) mixedstrlen_in_pixels(text, partialstr, FONT_NOTE, FONT_JNOTE);
+	for (n = plen = 0; partialstr[n].strptr != NULL && n < MAXPARTIALSTRING; n++) {
+	  if (partialstr[n].asciistr == False) {	/* case of Japanese font */
+	    if (n + 1 < MAXPARTIALSTRING)
+	      saveasc = partialstr[n + 1];
+	    while (plen + partialstr[n].pixlen > xs) {
+	      save = partialstr[n];
+	      (void) truncate_strpack(partialstr + n, xs - plen, FONT_NOTE);
+	      if (partialstr[n].strptr == NULL) {
+		partialstr[n] = save;
+	      } else {
+		i = center ? x + (xs - partialstr[n].length) / 2 : x + plen;
+		g_drawtext16(i, y, partialstr[n].strptr, partialstr[n].length / 2);
+		partialstr[n].strptr += partialstr[n].length;
+		partialstr[n].pixlen *= partialstr[n].length =
+		  save.length - partialstr[n].length;
+		partialstr[n].pixlen /= save.length;
+	      }
+	      plen = 0;
+	      y += lys;
+	      if ((ys -= lys) < cys)
+		goto NO_MORE_SPACE;
+	    }
+	    i = center ? x + (xs - partialstr[n].pixlen) / 2 : x + plen;
+	    g_drawtext16(i, y, partialstr[n].strptr, partialstr[n].length / 2);
+	    if (center) {
+	      plen = 0;
+	      y += lys;
+	      if ((ys -= lys) < cys)
+		break;
+	    } else
+	      plen += partialstr[n].pixlen;
+	    if (n + 1 < MAXPARTIALSTRING)
+	      partialstr[n + 1] = saveasc;
+	  } else {					/* case of ASCII font */
+	    for (len_s = len = plen, beg = p_s = p = (char *) partialstr[n].strptr; *p; p++) {
+	      if (*p == ' ' || *p == '\t') {
+		p_s   = p;
+		len_s = len;
+	      }
+	      len += CHARWIDTH(FONT_NOTE, *p);
+	      if (!p[1])
+		p++;
+	      if (len > xs || !*p) {
+		if (len > xs && p_s != beg
+		    && p_s >= p-MAXSPLIT && ys >= lys+cys) {
+		  p   = p_s;
+		  len = len_s;
+		}
+		i = center ? x + (xs - len)/2 : x + plen;
+		g_drawtext(i, y, beg, p - beg);
+		if (*p) {
+		  while (ISSPACE(*p)) p++;
+		  beg = p_s = p--;
+		  plen = len = len_s = 0;
+		  y += lys;
+		  if ((ys -= lys) < cys)
+		    goto NO_MORE_SPACE;
+		} else {
+		  if (center) {
+		    plen = 0;
+		    y += lys;
+		    if ((ys -= lys) < cys)
+		      goto NO_MORE_SPACE;
+		  } else
+		    plen = len;
+		  break;
+		}
+	      }
+	    }
+	  }
+	}
+NO_MORE_SPACE:;		/* It seems that some compilers need a SEMICOLON. */
+#else
 	for (len_s=len=0, beg=p_s=p=text; *p; p++) {
 		if (*p == ' ' || *p == '\t') {
 			p_s   = p;
@@ -415,4 +548,5 @@ static void draw_boxtext(x, y, xs, ys, text, center)
 				break;
 		}
 	}
+#endif
 }
