@@ -307,6 +307,10 @@ int resort_entry(list, n)
  * The idea is that there are probably relatively few repeating entries, and
  * they can be found by following the chain. All other entries can still be
  * found using binary searches.
+ * n-days-ahead entries are also added to this list, even if they are not
+ * repeating (which means that they are in the binary-sorted entry list also)
+ * This avoids having to make copies, and n-days-ahead warnings for repeating
+ * entries are difficult.
  * Rebuilding could be part of add_entry(), but that would be inefficient
  * when a file is read in, because then it's sufficient to build the chain
  * once at EOF, not every time an entry is added.
@@ -324,9 +328,9 @@ void rebuild_repeat_chain(list)
 		prev = &list->repeating;
 		for (n=0, ep=list->entry; n < list->nentries; n++, ep++)
 			if (ep->rep_every || ep->rep_weekdays ||
-			    ep->rep_days  || ep->rep_yearly) {
+			    ep->rep_days  || ep->rep_yearly || ep->days_warn) {
 				*prev = n;
-				prev = &ep->nextrep;
+				prev  = &ep->nextrep;
 			}
 		*prev = -1;
 		list->locked--;
@@ -366,26 +370,29 @@ BOOL lookup_entry(lookup, list, time, exact, norep, which, usr)
 {
 	int			repindex, regindex;
 	time_t			reptime = -1, regtime = -1;
+	BOOL			warn = 0;
 
 	list->locked++;
-	lookup->regindex = lookup->repindex = -1;
-	lookup->regtime  = lookup->reptime  = 0;
-	lookup->which	 = which;
-	lookup->user	 = usr;
+	lookup->regindex  = lookup->repindex = -1;
+	lookup->regtime   = lookup->reptime  = 0;
+	lookup->which	  = which;
+	lookup->user	  = usr;
+	lookup->days_warn = 0;
 	regindex = lookup_regular_entry(list, time, exact, norep);
 	repindex = norep ? -1
 			 : lookup_repeating_entry(list, time, exact,
-							 &reptime, which, usr);
+						  &reptime, which, usr, &warn);
 	if (regindex >= 0)
 		regtime = list->entry[regindex].time;
 	list->locked--;
 
 	if (repindex >= 0 && (regtime < 0 || reptime < regtime)) {
-		lookup->index   = lookup->repindex = repindex;
-		lookup->trigger = lookup->reptime  = reptime;
+		lookup->index	  = lookup->repindex = repindex;
+		lookup->trigger	  = lookup->reptime  = reptime;
+		lookup->days_warn = warn;
 	} else {
-		lookup->index   = lookup->regindex = regindex;
-		lookup->trigger = lookup->regtime  = regtime;
+		lookup->index	  = lookup->regindex = regindex;
+		lookup->trigger	  = lookup->regtime  = regtime;
 	}
 	lookup->list = list;
 	return(lookup->index >= 0 && lookup->index < list->nentries);
@@ -427,18 +434,20 @@ static int lookup_regular_entry(list, time, exact, norep)
 }
 
 
-static int lookup_repeating_entry(list, time, exact, reptime, which, usr)
+static int lookup_repeating_entry(list, time, exact, reptime, which, usr, warn)
 	struct plist		*list;		/* list of entries */
 	time_t			time;		/* time to locate */
 	BOOL			exact;		/* need entry, not insert pt */
 	time_t			*reptime;	/* returned repeat time */
 	char			which;		/* 'mywd': month,yr,week,day */
 	int			usr;		/* user number or -1 */
+	BOOL			*warn;		/* found a n-days-ahead warn?*/
 {
 	register struct entry	*entry;		/* entry array */
-	register int		n;		/* next repeating entry */
+	register int		n, i;		/* next repeating entry */
 	register time_t		earliest = 0;	/* earliest future entry yet */
 	register int		earliest_n= -1;	/* index of that entry */
+	BOOL			earliest_w = 0;	/* is it a n-days-ahead warn?*/
 	int			prev_n = -1;	/* entry before n */
 	int			earliest_prev_n=-1;/* entry before earliest_n*/
 	time_t			test;		/* next repeater trigger */
@@ -465,10 +474,12 @@ static int lookup_repeating_entry(list, time, exact, reptime, which, usr)
 				 strcmp(user[usr].name, entry->user)))
 			continue;
 
-		if (recycle(entry, time, &test) == 1)
+		i = recycle(entry, time, &test);
+		if (i == 1 || i == 3)
 			if (!earliest || test + exact < earliest) {
 				earliest   = test;
 				earliest_n = n;
+				earliest_w = i == 3;
 				earliest_prev_n = prev_n;
 			}
 	}
@@ -479,6 +490,7 @@ static int lookup_repeating_entry(list, time, exact, reptime, which, usr)
 		list->repeating = earliest_n;
 	}
 	*reptime = earliest;
+	*warn	 = earliest_w;
 	return(earliest_n);
 }
 
@@ -498,7 +510,7 @@ BOOL lookup_next_entry(lookup)
 	register struct lookup	*lookup;	/* previously found entry */
 {
 	register struct entry	*entry;		/* entry array */
-	register int		n;		/* next repeating entry */
+	register int		n, i;		/* next repeating entry */
 	int			repindex = -1, regindex = -1;
 	time_t			reptime, regtime = 0;
 	struct plist		*list = lookup->list;
@@ -506,6 +518,7 @@ BOOL lookup_next_entry(lookup)
 	int			prev_n, earliest_prev_n = -1;
 	register time_t		earliest = 0;
 	register int		earliest_n = -1;
+	BOOL			earliest_w = 0;
 	int			u;
 
 							/* next repeating */
@@ -539,10 +552,12 @@ BOOL lookup_next_entry(lookup)
 		if (lookup->user >= 0 && lookup->user != u)
 			continue;
 
-		if (recycle(entry, lookup->trigger, &time) == 1)
+		i = recycle(entry, lookup->trigger, &time);
+		if (i == 1 || i == 3)
 			if (!earliest || time < earliest) {
 				earliest   = time;
 				earliest_n = n;
+				earliest_w = i == 3;
 				earliest_prev_n = prev_n;
 			}
 	}
@@ -567,9 +582,9 @@ BOOL lookup_next_entry(lookup)
 
 							/* next regular */
 	if (lookup->regindex < 0) {
-		regindex = lookup_regular_entry(list,
-				lookup->trigger, TRUE, FALSE);
-		regtime = list->entry[regindex].time;
+		if ((regindex = lookup_regular_entry(list,
+					lookup->trigger, TRUE, FALSE)) >= 0)
+			regtime = list->entry[regindex].time;
 	} else {
 		n = lookup->regindex + 1;
 		entry = &list->entry[n];
@@ -583,11 +598,13 @@ BOOL lookup_next_entry(lookup)
 	}
 							/* use rep or reg */
 	if (repindex >= 0 && (regindex < 0 || reptime < regtime)) {
-		lookup->index    = lookup->repindex = repindex;
-		lookup->trigger  = lookup->reptime  = reptime;
+		lookup->index     = lookup->repindex = repindex;
+		lookup->trigger   = lookup->reptime  = reptime;
+		lookup->days_warn = earliest_w;
 	} else {
-		lookup->index    = lookup->regindex = regindex;
-		lookup->trigger  = lookup->regtime  = regtime;
+		lookup->index     = lookup->regindex = regindex;
+		lookup->trigger   = lookup->regtime  = regtime;
+		lookup->days_warn = 0;
 	}
 	return(lookup->index >= 0 && lookup->index < list->nentries);
 }
@@ -600,6 +617,9 @@ BOOL lookup_next_entry(lookup)
  *  1: early_warn of *entryp is nonzero, and early warn time is reached,
  *  2: late_warn of *entryp is nonzero, and late warn time is reached,
  *  3: the alarm time of *entryp is reached.
+ * Reaching the n-days-ahead warn time is the same as reaching the main
+ * alarm time; the warning is treated like a repetition except that the
+ * early and late warning times are ignored.
  *
  * The algorithm uses the fact that no warning can come more than 24 hours
  * before the trigger time. The first entry that is at most 24 hours early
@@ -649,7 +669,8 @@ int find_next_trigger(list, time, entryp, wait_time)
 			if (ep->except[i] == trigger - trigger % 86400)
 				break;
 		u = name_to_user(ep->user);
-		if (!ep->suspended && !ep->notime && i==NEXC && user[u].alarm)
+		if (!ep->suspended && !ep->notime && i==NEXC && user[u].alarm
+				   && (ep->triggered==2 || !lookup.days_warn))
 			switch (ep->triggered) {
 			  case 0:
 				if (ep->early_warn &&
@@ -674,7 +695,7 @@ int find_next_trigger(list, time, entryp, wait_time)
 			  case 2:
 				if (trigger > cutoff) {
 					dist = trigger - time;
-					if (dist < mindist){
+					if (dist < mindist) {
 						mindist = dist;
 						*entryp = ep;
 						reason  = 3;
