@@ -36,12 +36,28 @@
 #include <Xm/PushBP.h>
 #include <Xm/PushBG.h>
 #include <Xm/ToggleB.h>
+#include <Xm/ArrowB.h>
 #include <Xm/ScrolledW.h>
 #include <Xm/Text.h>
 #include <Xm/Protocols.h>
 #include "cal.h"
 
-#define NOTEWIDTH	180		/* default width of note button */
+#define NBLANK		3	/* add blank rows 3 this many at a time */
+				/* dimensions of text entry area */
+#define X_MARGIN	4	/* width of margin at left/right edges */
+#define X_ENABLE	30	/* width of enable checkbutton */
+#define X_LABEL		48	/* width of label text before text field */
+#define X_DATE		100	/* width of text field for dates */
+#define X_TIME		65	/* width of text fields except date and note */
+#define X_ARROW		13	/* width of up/down arrows, Y_ROW/2 */
+#define X_NOTEGAP	10	/* width of blank gap before note text field */
+#define X_OPTION	26	/* width of square option button */
+#define X_NOTE		(X_LABEL+5*X_OPTION)
+#define X_TOTAL		(X_MARGIN*2 + X_ENABLE + X_LABEL*3 + X_ARROW*3 +\
+			 X_DATE + X_TIME*3 + X_NOTEGAP + X_NOTE)
+#define Y_MARGIN	8	/* height of margin at top/bottom edges */
+#define Y_ROW		30	/* height of a single row */
+#define Y_GAP		6	/* height of gap between entry row pairs */
 
 void create_entry_widgets(), edit_list_button(), list_widget_pos();
 static void create_list_widgets();
@@ -49,6 +65,7 @@ static void list_confirm(), list_undo(), list_dup(), list_delete(),
 	    list_done(), list_pin(), list_own(), list_entry(), list_edit();
 
 extern Display		*display;	/* everybody uses the same server */
+extern Widget		toplevel;	/* top-level shell for icon name */
 extern GC		gc;		/* everybody uses this context */
 extern Pixel		color[NCOLS];	/* colors: COL_* */
 extern Pixmap		pixmap[NPICS];	/* common symbols */
@@ -62,10 +79,6 @@ extern struct holiday	sm_holiday[366];/* full-line texts under, and small */
 extern short		monthbegin[12];	/* julian date each month begins with*/
 extern int		curr_month;	/* month being displayed, 0..11 */
 extern int		curr_year;	/* year being displayed, since 1900 */
-					/* most recent popup created at: */
-extern int		edit_month;	/*  month being edited, 0..11 */
-extern int		edit_year;	/*  year being edited, since 1900 */
-extern int		edit_day;	/*  day being edited, 1..31 or 0 */
 
 static struct listmenu	*listmenu;	/* all important list popup widgets */
 
@@ -164,7 +177,7 @@ static struct listmenu *create_list_menu(time, period, key, entry,
  * destroy a popup. Remove it from the screen, but do not destroy its
  * widgets. They will be re-used when a new popup is created. Release
  * the sublist so it won't be regenerated if some entry somewhere
- * changes. Turn off the yellow highlight too.
+ * changes.
  */
 
 static void destroy_list_menu(lw)
@@ -173,18 +186,12 @@ static void destroy_list_menu(lw)
 	struct tm		*tm;		/* time to m/d/y conv */
 	int			oldday;
 
-	edit_list_button(FALSE, lw, 0, 0);
+	edit_list_button(0, lw, 0, 0);
 	XmToggleButtonSetState(lw->pin, lw->pinned = FALSE, FALSE);
 	XtPopdown(lw->shell);
 	lw->popped = FALSE;
 	confirm_new_entry();
 	tm = time_to_tm(lw->time);
-	if (oldday = edit_day) {
-		edit_day = 0;
-		draw_day(oldday);
-		draw_year_day(oldday, tm->tm_mon, tm->tm_year);
-		draw_week_day(oldday, tm->tm_mon, tm->tm_year);
-	}
 	destroy_sublist(lw);
 }
 
@@ -227,7 +234,8 @@ void destroy_all_listmenus()
 		free((char *)scan);
 	}
 	listmenu = 0;
-	edit.menu = 0;
+	edit.menu     = 0;
+	edit.srcentry = 0;
 }
 
 
@@ -246,7 +254,7 @@ void create_list_popup(list, time, period, key, entry, user_plus_1, private)
 	int			user_plus_1;	/* which user file */
 	BOOL			private;	/* only private */
 {
-	int			n = list->nentries + 5;
+	int			n = list->nentries + NBLANK;
 	struct listmenu		*w;
 	char			title[200];	/* menu title string */
 	char			*p;		/* temp for holiday lookup */
@@ -258,7 +266,7 @@ void create_list_popup(list, time, period, key, entry, user_plus_1, private)
 	time   -= time   % 86400;
 	period -= period % 86400;
 	w = create_list_menu(time, period, key, entry, user_plus_1, private);
-	create_list_widgets(w, n - n%5);
+	create_list_widgets(w, n - n%NBLANK);
 	if (time) {
 		strcpy(title, mkdatestring(time));
 		if (period) {
@@ -327,8 +335,9 @@ static void create_list_widgets(w, nentries)
 	n = 0;
 	XtSetArg(args[n], XmNdeleteResponse,	XmUNMAP);		n++;
 	XtSetArg(args[n], XmNiconic,		False);			n++;
-	w->shell = XtAppCreateShell("Schedule", "plan",
-			applicationShellWidgetClass, display, args, n);
+	w->shell = XtCreatePopupShell("Schedule",
+			applicationShellWidgetClass, toplevel, args, n);
+	
 #	ifdef EDITRES
 	XtAddEventHandler(w->shell, (EventMask)0, TRUE, 
  			_XEditResCheckMessages, NULL);
@@ -353,7 +362,7 @@ static void create_list_widgets(w, nentries)
 	XtSetArg(args[n], XmNbottomOffset,	8);			n++;
 	XtSetArg(args[n], XmNleftAttachment,	XmATTACH_FORM);		n++;
 	XtSetArg(args[n], XmNleftOffset,	8);			n++;
-	XtSetArg(args[n], XmNwidth,		80);			n++;
+	XtSetArg(args[n], XmNwidth,		64);			n++;
 	XtSetArg(args[n], XmNsensitive,		False);			n++;
 	w->confirm = XtCreateManagedWidget("Confirm", xmPushButtonWidgetClass,
 			form, args, n);
@@ -368,7 +377,7 @@ static void create_list_widgets(w, nentries)
 	XtSetArg(args[n], XmNleftAttachment,	XmATTACH_WIDGET);	n++;
 	XtSetArg(args[n], XmNleftWidget,	w->confirm);		n++;
 	XtSetArg(args[n], XmNleftOffset,	8);			n++;
-	XtSetArg(args[n], XmNwidth,		80);			n++;
+	XtSetArg(args[n], XmNwidth,		64);			n++;
 	XtSetArg(args[n], XmNsensitive,		False);			n++;
 	w->undo = XtCreateManagedWidget("Undo", xmPushButtonWidgetClass,
 			form, args, n);
@@ -383,7 +392,7 @@ static void create_list_widgets(w, nentries)
 	XtSetArg(args[n], XmNleftAttachment,	XmATTACH_WIDGET);	n++;
 	XtSetArg(args[n], XmNleftWidget,	w->undo);		n++;
 	XtSetArg(args[n], XmNleftOffset,	8);			n++;
-	XtSetArg(args[n], XmNwidth,		80);			n++;
+	XtSetArg(args[n], XmNwidth,		64);			n++;
 	XtSetArg(args[n], XmNsensitive,		False);			n++;
 	w->dup = XtCreateManagedWidget("Dup", xmPushButtonWidgetClass,
 			form, args, n);
@@ -397,7 +406,7 @@ static void create_list_widgets(w, nentries)
 	XtSetArg(args[n], XmNleftAttachment,	XmATTACH_WIDGET);	n++;
 	XtSetArg(args[n], XmNleftWidget,	w->dup);		n++;
 	XtSetArg(args[n], XmNleftOffset,	8);			n++;
-	XtSetArg(args[n], XmNwidth,		80);			n++;
+	XtSetArg(args[n], XmNwidth,		64);			n++;
 	XtSetArg(args[n], XmNsensitive,		False);			n++;
 	w->del = XtCreateManagedWidget("Delete", xmPushButtonWidgetClass,
 			form, args, n);
@@ -411,7 +420,7 @@ static void create_list_widgets(w, nentries)
 	XtSetArg(args[n], XmNleftAttachment,	XmATTACH_WIDGET);	n++;
 	XtSetArg(args[n], XmNleftWidget,	w->del);		n++;
 	XtSetArg(args[n], XmNleftOffset,	32);			n++;
-	XtSetArg(args[n], XmNwidth,		80);			n++;
+	XtSetArg(args[n], XmNwidth,		64);			n++;
 	help = XtCreateManagedWidget("Help", xmPushButtonWidgetClass,
 			form, args, n);
 	XtAddCallback(help, XmNactivateCallback,
@@ -424,8 +433,8 @@ static void create_list_widgets(w, nentries)
 	XtSetArg(args[n], XmNbottomOffset,	8);			n++;
 	XtSetArg(args[n], XmNleftAttachment,	XmATTACH_WIDGET);	n++;
 	XtSetArg(args[n], XmNleftWidget,	help);			n++;
-	XtSetArg(args[n], XmNleftOffset,	32);			n++;
-	XtSetArg(args[n], XmNwidth,		80);			n++;
+	XtSetArg(args[n], XmNleftOffset,	8);			n++;
+	XtSetArg(args[n], XmNwidth,		64);			n++;
 	w->done = XtCreateManagedWidget("Done", xmPushButtonWidgetClass,
 			form, args, n);
 	XtAddCallback(w->done, XmNactivateCallback, list_done, (XtPointer)0);
@@ -460,8 +469,8 @@ static void create_list_widgets(w, nentries)
 
 							/*-- scroll --*/
 	n = 0;
-	width = (config.bigwarning ? 640 : 590) +
-		(config.notewidth > 9 ? config.notewidth : NOTEWIDTH);
+	width = X_TOTAL + 32 +
+		(config.notewidth > X_NOTE ? config.notewidth - X_NOTE : 0);
 	XtSetArg(args[n], XmNtopAttachment,	XmATTACH_WIDGET);	n++;
 	XtSetArg(args[n], XmNtopWidget,		w->title);		n++;
 	XtSetArg(args[n], XmNtopOffset,		8);			n++;
@@ -473,7 +482,7 @@ static void create_list_widgets(w, nentries)
 	XtSetArg(args[n], XmNrightAttachment,	XmATTACH_FORM);		n++;
 	XtSetArg(args[n], XmNrightOffset,	8);			n++;
 	XtSetArg(args[n], XmNwidth,		width);			n++;
-	XtSetArg(args[n], XmNheight,		220);			n++;
+	XtSetArg(args[n], XmNheight,		240);			n++;
 	XtSetArg(args[n], XmNscrollingPolicy,	XmAUTOMATIC);		n++;
 	scroll = XtCreateWidget("lscroll", xmScrolledWindowWidgetClass,
 			form, args, n);
@@ -485,7 +494,7 @@ static void create_list_widgets(w, nentries)
 			scroll, args, n);
 	XtManageChild(w->list);
 
-	create_entry_widgets(w, nentries+5 - nentries%5);
+	create_entry_widgets(w, nentries+NBLANK - nentries%NBLANK);
 
 	XtManageChild(scroll);
 	XtManageChild(form);
@@ -510,9 +519,15 @@ static void create_list_widgets(w, nentries)
  */
 
 static char *help_name[SC_N] = {
-	"day_enable", "day_user", "day_date", "day_time", "day_length",
-	"day_advance", "day_recycle", "day_message", "day_script",
-	"day_except", "day_private", "day_note"
+	"day_enable", "day_user", "day_date", "day_enddate", "day_time",
+	"day_length", "day_advance", "day_advdays", "day_note", "day_recycle",
+	"day_message", "day_script", "day_except", "day_private",
+	"day_date", "day_enddate", "day_time", "day_length", "day_advance",
+	"day_advdays",
+	"day_date", "day_enddate", "day_time", "day_length", "day_advance",
+	"day_advdays",
+	"day_date", "day_enddate", "day_time", "day_length", "day_advance",
+	"day_advdays", "day_flags"
 };
 
 void create_entry_widgets(w, nentries)
@@ -528,13 +543,13 @@ void create_entry_widgets(w, nentries)
 	char			*name;
 	WidgetClass		class;
 
-	nentries = 5;					/* check # of rows: */
+	nentries = NBLANK;				/* check # of rows: */
 	n = w->sublist ? w->sublist->nentries : 0;
 	if (n+2 > w->nentries) {			/* need more? */
-		nentries = n+5 - n%5;
+		nentries = n+NBLANK - n%NBLANK;
 
-	} else if (n+5 < w->nentries) {			/* have too many? */
-		nentries = n+5 - n%5;
+	} else if (n+NBLANK < w->nentries) {		/* have too many? */
+		nentries = n+NBLANK - n%NBLANK;
 		for (y=nentries+1; y <= w->nentries; y++)
 			for (x=0; x < SC_N; x++)
 				XtDestroyWidget(w->entry[y][x]);
@@ -553,8 +568,8 @@ void create_entry_widgets(w, nentries)
 	    for (y=w->nentries; y <= nentries; y++) {
 		list_widget_pos(x, y, &xpos, &ypos, &width, &height, &align);
 		XtUnmanageChild(w->list);
-		class  = xmPushButtonWidgetClass;
-		name   = "";
+		class = xmPushButtonWidgetClass;
+		name  = "";
 		n = 0;
 		switch(x) {
 		  case SC_ENABLE:
@@ -562,39 +577,55 @@ void create_entry_widgets(w, nentries)
 			XtSetArg(args[n], XmNselectColor, color[COL_TOGGLE]);
 			n++;
 			break;
-		  case SC_USER:
-			name   = "Group";
-			break;
-		  case SC_DATE:
-			name   = "Date";
-			break;
-		  case SC_TIME:
-			name   = "Time";
-			break;
-		  case SC_LENGTH:
-			name   = "Length";
-			break;
-		  case SC_ADVANCE:
-			if (config.bigwarning) {
-				name = "Warn";
-				break;
-			} /* fall through */
+
+		  case SC_USER:     name = "Group"; break;
+		  case SC_DATE:     name = "Date";  break;
+		  case SC_TIME:     name = "Time";  break;
+		  case SC_ADVANCE:  name = "Warn";  break;
+		  case SC_NOTE:     name = "Note";  break;
+
 		  case SC_RECYCLE:
 		  case SC_MESSAGE:
 		  case SC_SCRIPT:
 		  case SC_EXCEPT:
 		  case SC_PRIVATE:
-			XtSetArg(args[n], XmNlabelType,   XmPIXMAP);    n++;
-			XtSetArg(args[n], XmNlabelPixmap,
-				pixmap[y ? PIC_BLANK : x-SC_FLAGS]);    n++;
+			XtSetArg(args[n], XmNlabelType, XmPIXMAP); n++;
 			break;
-		  case SC_NOTE:
-			name  = "Note";
+
+		  case SC_I_DATE:
+		  case SC_I_ENDDATE:
+		  case SC_I_TIME:
+		  case SC_I_LENGTH:
+		  case SC_I_ADVANCE:
+		  case SC_I_ADVDAYS:
+			XtSetArg(args[n], XmNarrowDirection, XmARROW_UP); n++;
+			class = xmArrowButtonWidgetClass;
+			break;
+
+		  case SC_D_DATE:
+		  case SC_D_ENDDATE:
+		  case SC_D_TIME:
+		  case SC_D_LENGTH:
+		  case SC_D_ADVANCE:
+		  case SC_D_ADVDAYS:
+			XtSetArg(args[n], XmNarrowDirection, XmARROW_DOWN);n++;
+			class = xmArrowButtonWidgetClass;
+			break;
+
+		  case SC_L_DATE:    name = "beg:";  break;
+		  case SC_L_ENDDATE: name = "end:";  break;
+		  case SC_L_TIME:    name = "beg:";  break;
+		  case SC_L_LENGTH:  name = "len:";  break;
+		  case SC_L_ADVANCE: name = "min:";  break;
+		  case SC_L_ADVDAYS: name = "days:"; break;
+		  case SC_L_FLAGS:   name = "opt:";  break;
 		}
-		if (x && y)
-			name  = " ";
-		if (y == 0)
+		if (!y || x >= SC_L)
 			class = xmLabelWidgetClass;
+		if (x && y && x < SC_L || !y && x >= SC_L)
+			name  = " ";
+		if (!y && class != xmLabelWidgetClass)
+			continue;
 
 		XtSetArg(args[n], XmNx,			xpos);		n++;
 		XtSetArg(args[n], XmNy,			ypos);		n++;
@@ -604,15 +635,17 @@ void create_entry_widgets(w, nentries)
 		XtSetArg(args[n], XmNrecomputeSize,	False);		n++;
 		XtSetArg(args[n], XmNtraversalOn,	True);		n++;
 		XtSetArg(args[n], XmNhighlightThickness,0);		n++;
-		XtSetArg(args[n], XmNshadowThickness,	x && y);	n++;
+		XtSetArg(args[n], XmNshadowThickness,	x&&y&&x<SC_N);	n++;
 		w->entry[y][x] = XtCreateManagedWidget(name, class,
 				w->list, args, n);
-		if (y && x)
+		if (x < SC_L) {
+		    if (y && x)
 			XtAddCallback(w->entry[y][x], XmNactivateCallback,
 				list_entry, (XtPointer)(x + y * SC_N));
-		if (y && !x)
+		    if (y && !x)
 			XtAddCallback(w->entry[y][x], XmNvalueChangedCallback,
 				list_entry, (XtPointer)(x + y * SC_N));
+		}
 		XtAddCallback(w->entry[y][x], XmNhelpCallback, help_callback,
 						(XtPointer)help_name[x]);
 	    }
@@ -633,51 +666,110 @@ void list_widget_pos(x, y, xpos, ypos, width, height, align)
 	int			*height;	/* returned height */
 	int			*align;		/* ret. text alignment: l,c,r*/
 {
-	int			warn_xs;	/* width of warning column */
-
-	*width  = 70;
-	*height = 30;
-	*ypos   = 10 + y * *height;
-	*align  = XmALIGNMENT_CENTER;
-	warn_xs = config.bigwarning ? *width : 30;
-
+	*width  = X_LABEL;
+	*height = Y_ROW;
+	*xpos   = X_MARGIN + X_ENABLE;
+	*ypos   = Y_MARGIN + (y ? y * (Y_ROW*2 + Y_GAP) - Y_ROW : 0);
+	*align  = x < SC_L ? XmALIGNMENT_CENTER : XmALIGNMENT_END;
+	if (x >= SC_I && x < SC_L) {
+		*width  = X_ARROW;
+		*height = Y_ROW/2;
+		if (x >= SC_D)
+			*ypos += *height;
+	}
 	switch(x) {
 	  case SC_ENABLE:
-		*xpos  = 4;
-		*width = 30;
+		*xpos  = X_MARGIN;
+		*width = X_ENABLE;
 		break;
 	  case SC_USER:
-		*xpos  = 4+40;
-		*width = 70;
+		*width = X_TIME;
 		break;
 	  case SC_DATE:
-		*xpos  = 4+40+70;
-		*width = 125;
+		*xpos += X_LABEL + X_TIME;
+		*width = X_DATE;
+		break;
+	  case SC_ENDDATE:
+		*ypos += Y_ROW;
+		*xpos += X_LABEL + X_TIME;
+		*width = X_DATE;
 		break;
 	  case SC_TIME:
-		*xpos  = 4+40+70+125;
-		*width = 70;
+		*xpos += X_LABEL*2 + X_TIME + X_DATE + X_ARROW;
+		*width = X_TIME;
 		break;
 	  case SC_LENGTH:
-		*xpos  = 4+40+70+125+70;
-		*width = 70;
+		*ypos += Y_ROW;
+		*xpos += X_LABEL*2 + X_TIME + X_DATE + X_ARROW;
+		*width = X_TIME;
 		break;
 	  case SC_ADVANCE:
-		*xpos  = 4+40+70+125+70+70;
-		*width = warn_xs;
+		*xpos += X_LABEL*3 + X_TIME*2 + X_DATE + X_ARROW*2;
+		*width = X_TIME;
 		break;
+	  case SC_ADVDAYS:
+		*ypos += Y_ROW;
+		*xpos += X_LABEL*3 + X_TIME*2 + X_DATE + X_ARROW*2;
+		*width = X_TIME;
+		break;
+	  case SC_NOTE:
+		*xpos += X_LABEL*3 + X_TIME*3 + X_DATE + X_ARROW*3 + X_NOTEGAP;
+		*width = config.notewidth > X_NOTE ? config.notewidth : X_NOTE;
+		*align = XmALIGNMENT_BEGINNING;
+		break;
+
 	  case SC_RECYCLE:
 	  case SC_MESSAGE:
 	  case SC_SCRIPT:
 	  case SC_EXCEPT:
 	  case SC_PRIVATE:
-		*xpos  = 4+40+70+125+70+70+warn_xs+30*(x-SC_FLAGS-1);
-		*width = 30;
+		*ypos += Y_ROW;
+		*xpos += X_LABEL*4 + X_TIME*3 + X_DATE + X_ARROW*3 +
+					X_NOTEGAP + X_OPTION * (x-SC_FLAGS);
+		*width = X_OPTION;
+		break;
+
+	  case SC_I_ENDDATE:
+	  case SC_D_ENDDATE:
+		*ypos += Y_ROW;
+	  case SC_I_DATE:
+	  case SC_D_DATE:
+		*xpos += X_LABEL + X_TIME + X_DATE;
+		break;
+	  case SC_I_LENGTH:
+	  case SC_D_LENGTH:
+		*ypos += Y_ROW;
+	  case SC_I_TIME:
+	  case SC_D_TIME:
+		*xpos += X_LABEL*2 + X_TIME*2 + X_DATE + X_ARROW;
+		break;
+	  case SC_I_ADVDAYS:
+	  case SC_D_ADVDAYS:
+		*ypos += Y_ROW;
+	  case SC_I_ADVANCE:
+	  case SC_D_ADVANCE:
+		*xpos += X_LABEL*3 + X_TIME*3 + X_DATE + X_ARROW*2;
+		break;
+
+	  case SC_L_ENDDATE:
+		*ypos += Y_ROW;
+	  case SC_L_DATE:
+		*xpos += X_TIME;
+		break;
+	  case SC_L_LENGTH:
+		*ypos += Y_ROW;
+	  case SC_L_TIME:
+		*xpos += X_LABEL + X_TIME + X_DATE + X_ARROW;
+		break;
+	  case SC_L_ADVDAYS:
+		*ypos += Y_ROW;
+	  case SC_L_ADVANCE:
+		*xpos += X_LABEL*2 + X_TIME*2 + X_DATE + X_ARROW*2;
+		break;
+	  case SC_L_FLAGS:
+		*ypos += Y_ROW;
+		*xpos += X_LABEL*3 + X_TIME*3 + X_DATE + X_ARROW*3 + X_NOTEGAP;
 		break;
-	  case SC_NOTE:
-		*xpos  = 4+40+70+125+70+70+warn_xs+30*(x-SC_FLAGS-1);
-		*width = config.notewidth > 9 ? config.notewidth : NOTEWIDTH;
-		*align = XmALIGNMENT_BEGINNING;
 	}
 	if (y == 0)
 		*align = XmALIGNMENT_CENTER;
@@ -690,10 +782,13 @@ void list_widget_pos(x, y, xpos, ypos, width, height, align)
  * turn a text label into a Text button, to allow user input. This is done
  * by simply installing a text widget on top of the label widget. <text>
  * is put into the text widget. The previously edited button is un-edited.
+ * doedit==2 is like 0 (unedit) but with the intention to delete, so don't
+ * store the new text and don't cause got_entry_text on the note to confirm
+ * because that would overwrite edit.entry, including the ID for netplan.
  */
 
 void edit_list_button(doedit, w, x, y)
-	BOOL			doedit;		/* TRUE=edit, FALSE=unedit */
+	int			doedit;		/* 1=edit, 0=unedit, 2=del */
 	struct listmenu		*w;		/* create which menu */
 	int			x;		/* column, SC_* */
 	int			y;		/* row, y=0: title */
@@ -710,11 +805,13 @@ void edit_list_button(doedit, w, x, y)
 	if (busy || !w)				/* not editing */
 		return;
 	if (w->text) {
-		char *string = XmTextGetString(w->text);
-		busy = TRUE;
-		got_entry_text(w, w->xedit, w->yedit, string);
-		busy = FALSE;
-		XtFree(string);
+		if (doedit != 2) {
+			char *string = XmTextGetString(w->text);
+			busy = TRUE;
+			got_entry_text(w, w->xedit, w->yedit, string);
+			busy = FALSE;
+			XtFree(string);
+		}
 		XmProcessTraversal(w->list, XmTRAVERSE_CURRENT);
 		XtDestroyWidget(w->text);
 		if (x && y)
@@ -722,7 +819,7 @@ void edit_list_button(doedit, w, x, y)
 						mkbuttonstring(w, x, y));
 	}
 	w->text = 0;
-	if (!doedit)
+	if (doedit != 1)
 		return;
 
 	list_widget_pos(x, y, &xpos, &ypos, &width, &height, &align);
@@ -744,9 +841,11 @@ void edit_list_button(doedit, w, x, y)
 	XmProcessTraversal(w->text, XmTRAVERSE_CURRENT);
 
 	text = "";
-	if (w->sublist && w->sublist->nentries >= y) {
+	if (w->sublist && w->sublist->nentries >= y-1 &&
+			!(w->sublist->nentries == y-1 && x == SC_TIME)) {
 		text = mkbuttonstring(w, x, y);
-		if (x == SC_DATE && (space = strchr(text, ' ')))
+		if ((x == SC_DATE || x == SC_ENDDATE) &&
+						(space = strchr(text, ' ')))
 			text = space+1;
 	}
 	print_text_button(w->text, "%s", text);
@@ -769,7 +868,11 @@ BOOL prepare_for_modify(name)
 {
 	int		u;		/* index into user array */
 
-	u = name_to_user(name);
+	if (name) {
+		u = name_to_user(name);
+	} else {
+		name = user[u=0].name;
+	}
 	if (user[u].readonly) {
 		if (user[u].fromserver)
 			create_error_popup(0, 0,
@@ -827,7 +930,7 @@ static void list_undo(widget, item, data)
 	struct listmenu *lw = find_widget_in_list_menu(widget);
 	if (lw == edit.menu && prepare_for_modify(edit.entry.user)) {
 		undo_new_entry();
-		edit_list_button(FALSE, lw, 0, 0);
+		edit_list_button(0, lw, 0, 0);
 	}
 }
 
@@ -854,7 +957,7 @@ static void list_dup(widget, item, data)
 			draw_list(lw);
 			edit.changed = TRUE;
 			sensitize_edit_buttons();
-			got_entry_press(lw, SC_DATE, edit.y, FALSE);
+			got_entry_press(lw, SC_DATE, edit.y, FALSE, 0);
 		}
 	}
 }
@@ -873,9 +976,8 @@ static void list_delete(widget, item, data)
 		return;
 
 	destroy_recycle_popup();
-	destroy_advance_popup();
 	destroy_text_popup();
-	edit_list_button(FALSE, lw, 0, 0);
+	edit_list_button(2, lw, 0, 0);
 	edit.editing = FALSE;
 	edit.changed = FALSE;
 	sensitize_edit_buttons();
@@ -936,7 +1038,8 @@ static void list_entry(widget, item, data)
 	int				y = item / SC_N;
 
 	if (lw)
-		got_entry_press(lw, x, y, data->set);
+		got_entry_press(lw, x, y, data->set,
+					data->event->xbutton.state & 1);
 }
 
 /*ARGSUSED*/
