@@ -33,6 +33,7 @@ static void	hide_m_callback(), hide_y_callback(), hide_w_callback();
 static void	hide_o_callback(), hide_d_callback();
 
 extern Display		*display;	/* everybody uses the same server */
+extern Widget		toplevel;	/* top-level shell for icon name */
 extern struct edit	edit;		/* info about entry being edited */
 extern struct config	config;		/* global configuration data */
 extern XtAppContext	app;		/* application handle */
@@ -45,6 +46,7 @@ static BOOL		have_shell;	/* message popup exists if TRUE */
 static Widget		shell;		/* popup menu shell */
 static Widget		text_w[NEXC];	/* exception date text widgets */
 static Widget		acol_w;		/* appointment text color button */
+static Widget		acol_sw[9];	/* choices in acol_w color popup */
 
 
 /*
@@ -54,23 +56,27 @@ static Widget		acol_w;		/* appointment text color button */
 
 static void fillout_except_menu()
 {
-	Arg			arg;
-	int			i;
+	Arg			args[2];
+	int			i, col;
 
 	for (i=0; i < NEXC; i++)
 		print_text_button(text_w[i], edit.entry.except[i] ?
 				mkdatestring(edit.entry.except[i]) : " ");
 
-	print_pixmap(edit.menu->entry[edit.y][SC_EXCEPT],
-			show_except_pixmap(&edit.entry) ? PIC_EXCEPT : -1);
+	print_pixmap(edit.menu->entry[edit.y][SC_EXCEPT], PIC_EXCEPT,
+				show_except_pixmap(&edit.entry));
 
-	XtSetArg(arg, XmNforeground, color[
-		edit.entry.acolor == 0 ? COL_STD
-				       : COL_HBLACK + edit.entry.acolor-1]);
-	XtSetValues(acol_w, &arg, 1);
-
-	print_button(acol_w, edit.entry.acolor == 0 ? "default" : "%d",
+	col = edit.entry.acolor ? COL_HBLACK + edit.entry.acolor-1 : COL_STD;
+#ifdef XmCSimpleOptionMenu
+	XtSetArg(args[0], XmNforeground, color[col]);
+	XtSetArg(args[1], XmNmenuHistory, acol_sw[edit.entry.acolor]);
+	XtSetValues(XmOptionButtonGadget(acol_w), args, 2);
+#else
+	XtSetArg(args[0], XmNforeground, color[col]);
+	XtSetValues(acol_w, args, 1);
+	print_button(acol_w, edit.entry.acolor ? "%d" : "default",
 							edit.entry.acolor-1);
+#endif
 }
 
 
@@ -111,8 +117,8 @@ void destroy_except_popup()
 		have_shell = FALSE;
 		edit.changed = TRUE;
 		edit.entry.suspended = FALSE;
-		print_pixmap(edit.menu->entry[edit.y][SC_EXCEPT],
-			show_except_pixmap(&edit.entry) ? PIC_EXCEPT  : -1);
+		print_pixmap(edit.menu->entry[edit.y][SC_EXCEPT], PIC_EXCEPT,
+					show_except_pixmap(&edit.entry));
 	}
 }
 
@@ -124,18 +130,19 @@ void destroy_except_popup()
 
 void create_except_popup()
 {
-	Widget			form, frame, form2, line, w;
+	Widget			form, frame, form2, line, popup, w, pw;
 	Arg			args[20];
 	int			n, i;
 	Atom			closewindow;
+	XmString		str;
 
 	destroy_except_popup();
 
 	n = 0;
 	XtSetArg(args[n], XmNdeleteResponse,	XmDO_NOTHING);		n++;
 	XtSetArg(args[n], XmNiconic,		False);			n++;
-	shell = XtAppCreateShell("Exception Dates", "plan",
-			applicationShellWidgetClass, display, args, n);
+	shell = XtCreatePopupShell("Exception Dates",
+			applicationShellWidgetClass, toplevel, args, n);
 #	ifdef EDITRES
 	XtAddEventHandler(shell, (EventMask)0, TRUE, 
  			_XEditResCheckMessages, NULL);
@@ -318,6 +325,41 @@ void create_except_popup()
 						notime_callback, (XtPointer)0);
 
 							/*-- appt colors --*/
+#ifdef XmCSimpleOptionMenu
+	popup = XmCreatePulldownMenu(form, "pulldown", NULL, 0);
+	for (i=0; i <= 8; i++) {
+		char name[10];
+		sprintf(name, i ? "%d" : "default", i-1);
+		XtSetArg(args[0], XmNalignment, XmALIGNMENT_CENTER);
+		XtSetArg(args[1], XmNforeground,
+					color[i ? COL_HBLACK+i-1 : COL_STD]);
+		acol_sw[i] = XtCreateManagedWidget(name,
+				xmPushButtonWidgetClass, popup, args, 2);
+		XtAddCallback(acol_sw[i], XmNactivateCallback,
+				(XtCallbackProc)acol_callback, (XtPointer)i);
+	}
+	str = XmStringCreateSimple("Appointment text color:");
+	n = 0;
+	XtSetArg(args[n], XmNtopAttachment,	XmATTACH_WIDGET);	n++;
+	XtSetArg(args[n], XmNtopWidget,		w);			n++;
+	XtSetArg(args[n], XmNtopOffset,		16);			n++;
+	XtSetArg(args[n], XmNbottomAttachment,	XmATTACH_FORM);		n++;
+	XtSetArg(args[n], XmNbottomOffset,	8);			n++;
+	XtSetArg(args[n], XmNleftAttachment,	XmATTACH_FORM);		n++;
+	XtSetArg(args[n], XmNleftOffset,	8);			n++;
+	XtSetArg(args[n], XmNrightAttachment,	XmATTACH_FORM);		n++;
+	XtSetArg(args[n], XmNrightOffset,	8);			n++;
+	XtSetArg(args[n], XmNmarginHeight,	0);			n++;
+	XtSetArg(args[n], XmNwidth,		60);			n++;
+	XtSetArg(args[n], XmNhighlightThickness,1);			n++;
+	XtSetArg(args[n], XmNsubMenuId,		popup);			n++;
+	XtSetArg(args[n], XmNlabelString,	str);			n++;
+	acol_w = w = XmCreateOptionMenu(form2, "exc_acolor", args, n);
+	XtAddCallback(acol_w, XmNhelpCallback, help_callback,
+						(XtPointer)"exc_acolor");
+	XmStringFree(str);
+	XtManageChild(acol_w);
+#else
 	n = 0;
 	XtSetArg(args[n], XmNtopAttachment,	XmATTACH_WIDGET);	n++;
 	XtSetArg(args[n], XmNtopWidget,		w);			n++;
@@ -344,7 +386,6 @@ void create_except_popup()
 	XtAddCallback(acol_w, XmNactivateCallback, acol_callback,(XtPointer)0);
 	XtAddCallback(acol_w, XmNhelpCallback, help_callback,
 						(XtPointer)"exc_acolor");
-
 	n = 0;
 	XtSetArg(args[n], XmNtopAttachment,	XmATTACH_FORM);		n++;
 	XtSetArg(args[n], XmNbottomAttachment,	XmATTACH_FORM);		n++;
@@ -353,6 +394,7 @@ void create_except_popup()
 	XtSetArg(args[n], XmNleftOffset,	16);			n++;
 	w = XtCreateManagedWidget("Appointment text color",
 			xmLabelWidgetClass, line, args, n);
+#endif
 
 							/*-- buttons --*/
 	n = 0;
@@ -449,8 +491,10 @@ static void split_callback(widget, item, data)
 
 	string = XmTextGetString(text_w[item]);
 	for (p=string; *p == ' ' || *p == '\t'; p++);
-	if (!*p)
+	if (!*p) {
+		XtFree (string);
 		return;
+	}
 	time = parse_datestring(p, 0);
 	XtFree(string);
 	clone_entry(&new, &edit.entry);
@@ -488,8 +532,8 @@ static void hide_m_callback(widget, item, data)
 {
 	edit.entry.hide_in_m ^= TRUE;
 	edit.changed = TRUE;
-	print_pixmap(edit.menu->entry[edit.y][SC_EXCEPT],
-			show_except_pixmap(&edit.entry) ? PIC_EXCEPT  : -1);
+	print_pixmap(edit.menu->entry[edit.y][SC_EXCEPT], PIC_EXCEPT,
+					show_except_pixmap(&edit.entry));
 }
 
 
@@ -501,8 +545,8 @@ static void hide_y_callback(widget, item, data)
 {
 	edit.entry.hide_in_y ^= TRUE;
 	edit.changed = TRUE;
-	print_pixmap(edit.menu->entry[edit.y][SC_EXCEPT],
-			show_except_pixmap(&edit.entry) ? PIC_EXCEPT  : -1);
+	print_pixmap(edit.menu->entry[edit.y][SC_EXCEPT], PIC_EXCEPT,
+					show_except_pixmap(&edit.entry));
 }
 
 
@@ -514,8 +558,8 @@ static void hide_w_callback(widget, item, data)
 {
 	edit.entry.hide_in_w ^= TRUE;
 	edit.changed = TRUE;
-	print_pixmap(edit.menu->entry[edit.y][SC_EXCEPT],
-			show_except_pixmap(&edit.entry) ? PIC_EXCEPT  : -1);
+	print_pixmap(edit.menu->entry[edit.y][SC_EXCEPT], PIC_EXCEPT,
+					show_except_pixmap(&edit.entry));
 }
 
 
@@ -527,8 +571,8 @@ static void hide_d_callback(widget, item, data)
 {
 	edit.entry.hide_in_d ^= TRUE;
 	edit.changed = TRUE;
-	print_pixmap(edit.menu->entry[edit.y][SC_EXCEPT],
-			show_except_pixmap(&edit.entry) ? PIC_EXCEPT  : -1);
+	print_pixmap(edit.menu->entry[edit.y][SC_EXCEPT], PIC_EXCEPT,
+					show_except_pixmap(&edit.entry));
 }
 
 
@@ -540,8 +584,8 @@ static void hide_o_callback(widget, item, data)
 {
 	edit.entry.hide_in_o ^= TRUE;
 	edit.changed = TRUE;
-	print_pixmap(edit.menu->entry[edit.y][SC_EXCEPT],
-			show_except_pixmap(&edit.entry) ? PIC_EXCEPT  : -1);
+	print_pixmap(edit.menu->entry[edit.y][SC_EXCEPT], PIC_EXCEPT,
+					show_except_pixmap(&edit.entry));
 }
 
 
@@ -554,8 +598,8 @@ static void notime_callback(widget, item, data)
 	edit.entry.notime ^= TRUE;
 	edit.changed = TRUE;
 	draw_list(edit.menu);
-	print_pixmap(edit.menu->entry[edit.y][SC_EXCEPT],
-			show_except_pixmap(&edit.entry) ? PIC_EXCEPT  : -1);
+	print_pixmap(edit.menu->entry[edit.y][SC_EXCEPT], PIC_EXCEPT,
+					show_except_pixmap(&edit.entry));
 }
 
 
@@ -570,6 +614,10 @@ static void acol_callback(widget, item, data)
 	XmToggleButtonCallbackStruct	*data;
 {
 	edit.changed = TRUE;
+#ifdef XmCSimpleOptionMenu
+	edit.entry.acolor = item;
+#else
 	edit.entry.acolor = (edit.entry.acolor + 1) % 9;
+#endif
 	fillout_except_menu();
 }
