@@ -187,12 +187,12 @@ static int readfile(list, path, u, conf)
 	char			line[1024];	/* line buffer */
 	int			lineno = 0;	/* first line == "grok"? */
 
+	if (conf)				/* defaults for 't' line */
+		guess_tzone();
 	path = resolve_tilde(path);
 	writable = !access(path, W_OK);
 	if (!(fp = fopen(path, writable ? "r+" : "r")))
 		return(errno);
-	if (conf)		/* defaults for 't' line */
-		guess_tzone();
 	if (writable)
 		lockfile(fp, TRUE);
 	for (;;) {
@@ -223,7 +223,8 @@ static int readfile(list, path, u, conf)
 		}
 		if (!conf && strchr("OotelaPpmUu", *line))
 			continue;
-		if (*line >= '0' && *line <= '9' && user
+		if (conf &&
+		    *line >= '0' && *line <= '9' && user
 						 && user[u].fromserver)
 			break;
 		parse_file_line(list, path, user ? user[u].name:0, conf, line);
@@ -259,7 +260,7 @@ void parse_file_line(list, path, name, conf, line)
 	int			n, m;		/* tmp entry # of ep in list */
 	struct tm		tm;		/* m/d/y h:m:s to time conv */
 	int			tmp[9];		/* temps for sscanf */
-	int			acol;		/* more temps for sscanf */
+	int			acol, ndays;	/* more temps for sscanf */
 	char			sflag, pflag, nflag;
 	char			nmflag, nyflag, nwflag, noflag, ndflag;
 	char			**string;	/* string to add to */
@@ -301,7 +302,6 @@ void parse_file_line(list, path, name, conf, line)
 		c->gpsweek        = line[7]  == 'g';
 		c->weeknum        = line[7]  != '-';
 		c->nopast         = line[8]  != '-';
-		c->bigwarning     = line[9]  != '-';
 		c->weekwarn       = line[10] != '-';
 		c->weekuser       = line[11] != '-';
 		c->week_bignotime = line[12] != '-';
@@ -457,16 +457,17 @@ void parse_file_line(list, path, name, conf, line)
 		clone_entry(&entry, (struct entry *)0);
 		sflag = pflag = nflag =
 		nmflag = nyflag = nwflag = noflag = ndflag = 0;
-		acol = 0;
+		acol = ndays = 0;
 		sscanf(line,
-	"%d/%d/%d %d:%d:%d %d:%d:%d %d:%d:%d %d:%d:%d %c%c%c%c%c%c%c%c-- %d",
+       "%d/%d/%d %d:%d:%d %d:%d:%d %d:%d:%d %d:%d:%d %c%c%c%c%c%c%c%c-- %d %d",
 			&tm.tm_mon, &tm.tm_mday, &tm.tm_year,
 			&tm.tm_hour, &tm.tm_min, &tm.tm_sec,
 			tmp+0, tmp+1, tmp+2,
 			tmp+3, tmp+4, tmp+5,
 			tmp+6, tmp+7, tmp+8,
 			&sflag, &pflag, &nflag,
-			&nmflag, &nyflag, &nwflag, &noflag, &ndflag, &acol);
+			&nmflag, &nyflag, &nwflag, &noflag, &ndflag,
+			&acol, &ndays);
 		tm.tm_mon--;
 		if (tm.tm_year < 70)
 			tm.tm_year += 100;
@@ -478,6 +479,7 @@ void parse_file_line(list, path, name, conf, line)
 		entry.length       = tmp[0]*3600 + tmp[1]*60 + tmp[2];
 		entry.early_warn   = tmp[3]*3600 + tmp[4]*60 + tmp[5];
 		entry.late_warn    = tmp[6]*3600 + tmp[7]*60 + tmp[8];
+		entry.days_warn    = ndays * 86400;
 		entry.suspended    = sflag  == 'S';
 		entry.private      = pflag  == 'P';
 		entry.noalarm      = nflag  == 'N';
