@@ -234,7 +234,7 @@ BOOL writefile(list, path, u, what)
 		fprintf(fp, "plan %s\n", VERSION);
 		write_error |= errno && errno != ENOTTY;
 		fprintf(fp,
-			"o\t%c%c%c%c%c%c%c%c%c%c%c%c%c%c %d %d %d %d %d %d\n",
+			"o\t%c%c%c%c%c%c%c-%c%c%c%c%c%c %d %d %d %d %d %d\n",
 					c->sunday_first	  ? 's' : '-',
 					c->ampm		  ? 'a' : '-',
 					c->mmddyy	  ? 'm' : '-',
@@ -243,7 +243,6 @@ BOOL writefile(list, path, u, what)
 					c->gpsweek	  ? 'g' :
 					c->weeknum	  ? 'w' : '-',
 					c->nopast	  ? 'n' : '-',
-					c->bigwarning	  ? 'b' : '-',
 					c->weekwarn	  ? 'w' : '-',
 					c->weekuser	  ? 'u' : '-',
 					c->week_bignotime ? 's' : '-',
@@ -387,7 +386,7 @@ int write_one_entry(callback, ep)
 
 	tm = time_to_tm(ep->time);
 	sprintf(line,
-"%d/%d/%d  %d:%d:%d  %d:%d:%d  %d:%d:%d  %d:%d:%d  %c%c%c%c%c%c%c%c-- %d\n",
+"%d/%d/%d  %d:%d:%d  %d:%d:%d  %d:%d:%d  %d:%d:%d  %c%c%c%c%c%c%c%c-- %d %d\n",
 		(int)tm->tm_mon+1,
 		(int)tm->tm_mday,
 		(int)tm->tm_year + 1900,
@@ -411,7 +410,8 @@ int write_one_entry(callback, ep)
 		(int)ep->hide_in_w ? 'W' : '-',
 		(int)ep->hide_in_o ? 'O' : '-',
 		(int)ep->hide_in_d ? 'D' : '-',
-		(int)ep->acolor);
+		(int)ep->acolor,
+		(int)ep->days_warn / 86400);
 
 	write_error |= (*callback)(line);
 	if (ep->rep_every || ep->rep_weekdays || ep->rep_days
