@@ -18,8 +18,12 @@
 #include <unistd.h>
 #include <time.h>
 #ifndef NEWSOS4
+#ifdef STDLIBMALLOC
+#include <stdlib.h>
+#else
 #include <malloc.h>
 #endif
+#endif
 #ifdef CPP_PATH
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -35,7 +39,7 @@
  */
 
 #define LEAPYEAR(y)	!((y)&3)
-#define JULIAN(m,d)	(monthbegin[m] + (d)-1 + ((m)>1 && LEAPYEAR(parse_year)))
+#define JULIAN(m,d)	(monthbegin[m] + (d)-1+((m)>1 && LEAPYEAR(parse_year)))
 #define LAST		999
 #define ANY		0
 #define	BEFORE		-1
@@ -43,7 +47,7 @@
 
 static int	 calc_easter();
 
-#if defined(bsdi) || defined(linux) || defined(__NetBSD__)
+#if defined(bsdi)||defined(linux)||defined(__NetBSD__)||defined(__FreeBSD__)||defined(HPGCC)
 int yylineno;
 #else
 extern int	 yylineno;		/* current line # being parsed */
@@ -79,6 +83,7 @@ yyerror(msg) char *msg;
 		sprintf(errormsg,
 		      "Problem with holiday file %s:\n%.80s in line %d",
 					filename, msg, yylineno+1);
+	return(0);
 }
 
 
@@ -197,13 +202,12 @@ void setwday(num, wday, month, off, length)
 		if (num == LAST)
 			for (l=0; l < length; l++)
 				setliteraldate(m, d+28<=mlen ? d+28 : d+21,
-								off+l, dup++);
+								off+l, &dup);
 		else
 			for (d+=min_num*7, n=min_num; n <= max_num; n++, d+=7)
 				if (d >= 1 && d <= mlen)
 					for (l=0; l < length; l++)
-						setliteraldate(m, d, off+l,
-									dup++);
+						setliteraldate(m,d,off+l,&dup);
 	}
 }
 
@@ -249,7 +253,7 @@ void setdoff(wday, rel, month, day, year, off, length)
 			nd = (((wday - wday1 + 7) % 7) -
 						((rel == BEFORE) ? 7 : 0)) % 7;
 			for (l=0; l < length; l++)
-				setliteraldate(m,monthlen[m]+nd, off+l, dup++);
+				setliteraldate(m,monthlen[m]+nd, off+l, &dup);
 		} else
 			for (d=min_day; d <= max_day; d++) {
 				(void)date_to_time(d, m, parse_year,
@@ -257,7 +261,7 @@ void setdoff(wday, rel, month, day, year, off, length)
 				nd = (((wday - wday1 + 7) % 7) -
 						((rel == BEFORE) ? 7 : 0)) % 7;
 				for (l=0; l < length; l++)
-					setliteraldate(m, d+nd, off+l, dup++);
+					setliteraldate(m, d+nd, off+l, &dup);
 			}
 }
 
@@ -297,11 +301,11 @@ void setdate(month, day, year, off, length)
 	for (m=min_month; m <= max_month; m++)
 		if (day == LAST)
 			for (l=0; l < length; l++)
-				setliteraldate(m, monthlen[m], off+l, dup++);
+				setliteraldate(m, monthlen[m], off+l, &dup);
 		else
 			for (d=min_day; d <= max_day; d++)
 				for (l=0; l < length; l++)
-					setliteraldate(m, d, off+l, dup++);
+					setliteraldate(m, d, off+l, &dup);
 }
 
 
@@ -317,7 +321,7 @@ void setdate(month, day, year, off, length)
  * by the lexer, and should therefore only be freed once).
  */
 
-static colormap[10] = {
+static int colormap[10] = {
 	0, COL_HBLACK, COL_HRED, COL_HGREEN, COL_HYELLOW,
 	COL_HBLUE, COL_HMAGENTA, COL_HCYAN, COL_HWHITE, COL_WEEKEND };
 
@@ -325,19 +329,19 @@ static void setliteraldate(month, day, off, dup)
 	int		month;		/* 0..11 */
 	int		day;		/* 1..31 */
 	int		off;		/* offset in days */
-	int		dup;		/* flag for later free() */
+	int		*dup;		/* flag for later free() */
 {
 	int julian = JULIAN(month, day) + off;
 	struct holiday *hp = yacc_small ? &sm_holiday[julian]
 					: &holiday[julian];
 
 	if (julian >= 0 && julian <= 365 && !hp->string) {
-		if (!dup)
+		if (!*dup)
 			holiday_name = mystrdup(holiday_name);
 		hp->string	= holiday_name;
 		hp->stringcolor	= colormap[yacc_stringcolor];
 		hp->daycolor	= colormap[yacc_daycolor];
-		hp->dup		= dup;
+		hp->dup		= (*dup)++;
 	}
 }
 
