@@ -5,12 +5,6 @@
  *
  *	main(argc, argv)		Guess.
  *	fatal(char *fmt, ...)		Prints an error message and exits.
- *	print_button(w, fmt, ...)	Prints a string into a string Label
- *					or PushButton.
- *	print_text_button(w, fmt, ...)	Prints a string into a Text button.
- *	print_pixmap(w, n)		Prints a pixmap into a Pixmap label.
- *	set_color(col)			Sets the foreground color to one of
- *					the predefined colors (COL_*).
  *	resynchronize_daemon()		Called when the database on disk has
  *					changed; tell the daemon to re-read.
  *	register_X_input(fd)		network.c opened new server socket
@@ -31,6 +25,7 @@
 #include <unistd.h>
 #include <stdlib.h>
 #endif
+#include <errno.h>
 #include <ctype.h>
 #include <signal.h>
 #include <string.h>
@@ -50,6 +45,7 @@ static void init_resources(), init_colors(), init_fonts(), init_daemon();
 static void timer_callback(), usage(), non_interactive(), sighand();
 static void socket_callback();
 static int  dumptoday();
+extern void run_daemon();
 #ifdef MIPS
 void exit();
 extern char *getenv();
@@ -57,6 +53,8 @@ extern char *getenv();
 
 Display			*display;	/* everybody uses the same server */
 GC			jgc, gc;	/* graphic context, Japanese and std */
+GC			xor_gc;		/* XOR gc for rubberbanding */
+GC			chk_gc;		/* checkered gc for light background */
 XtAppContext		app;		/* application handle */
 Widget			toplevel;	/* top-level shell for icon name */
 struct mainmenu		mainmenu;	/* all important main window widgets */
@@ -70,10 +68,13 @@ int			curr_year;	/* year being displayed, since 1900 */
 struct plist		*mainlist;	/* list of all schedule entries */
 extern struct user	*user;			/* user list for week view */
 extern Pixmap		pixmap[NPICS];	/* common symbols */
+extern Pixmap	    graypixmap[NPICS];	/* common symbols, grayed-out */
 static BOOL		nodaemon;	/* don't insist on a daemon (-s) */
+static BOOL		rundaemon;	/* auto-run daemon if there is none */
 static XtIntervalId	timer_id;	/* identifies 60-second timer */
 BOOL			reread;		/* need to re-read database file */
 extern char		lockpath[];	/* lockfile path (from lock.c) */
+extern struct edit	edit;		/* info about entry being edited */
 
 #ifdef JAPAN
 char			*localename;			/* locale name */
@@ -102,6 +103,7 @@ int main(argc, argv)
 	BOOL			nolock = FALSE;
 	BOOL			others = FALSE;
 	char			*errmsg;
+	XGCValues		gcval;
 
 	interactive = FALSE;
 	if ((progname = strrchr(argv[0], '/')) && progname[1])
@@ -136,6 +138,9 @@ int main(argc, argv)
 			  case 's':
 				nodaemon = TRUE;
 				break;
+			  case 'S':
+				rundaemon = TRUE;
+				break;
 			  case 'f':
 				nofork = TRUE;
 				break;
@@ -204,14 +209,22 @@ int main(argc, argv)
 				_XEditResCheckMessages, NULL);
 #	endif
 	display = XtDisplay(toplevel);
-	jgc = XCreateGC(display, DefaultRootWindow(display), 0, 0);
-	gc  = XCreateGC(display, DefaultRootWindow(display), 0, 0);
+	jgc	= XCreateGC(display, DefaultRootWindow(display), 0, 0);
+	gc	= XCreateGC(display, DefaultRootWindow(display), 0, 0);
+	xor_gc	= XCreateGC(display, DefaultRootWindow(display), 0, 0);
+	chk_gc	= XCreateGC(display, DefaultRootWindow(display), 0, 0);
+	gcval.function = GXxor;
+	(void)XChangeGC(display, xor_gc, GCFunction, &gcval);
+	XSetForeground(display, xor_gc,
+			WhitePixelOfScreen(DefaultScreenOfDisplay(display)));
 
 	init_resources();
 	init_colors();
 	init_fonts();
 	init_pixmaps();
+
 	set_icon(toplevel, 0);
+	XSetForeground(display, xor_gc, color[COL_WIREFRAME]);
 
 	create_list(&mainlist);
 	create_cal_widgets(toplevel);
@@ -229,7 +242,7 @@ int main(argc, argv)
 	if (errmsg = parse_holidays(curr_year, TRUE))
 		create_error_popup(mainmenu.cal, 0, errmsg);
 							/* start */
-	timer_id = XtAppAddTimeOut(app, TIMER_PERIOD, timer_callback, 0);
+	timer_id = XtAppAddTimeOut(app, 100, timer_callback, 0);
 	XtRealizeWidget(toplevel);
 	resynchronize_daemon();
 	XtAppMainLoop(app);
@@ -252,44 +265,68 @@ static void non_interactive(argc, argv)
 	struct tm	tm1;			/* for time conversion */
 	time_t		trigger;		/* trigger date and time */
 	char		msg[1024];		/* note from command line */
-	struct entry	entry;			/* appointment to add */
-	int		n, i;			/* char and arg counters */
+	int		n, i, j;		/* char and arg counters */
 	char		lockfile[80];		/* plan lockfile, for sighup */
 	FILE		*fp;			/* for reading lockfile */
 	PID_T		pid = 0;		/* interactive plan PID */
 
-	n = strlen(argv[1]);
-	if (n != 4 && n != 8)
-		usage();
-	i = atoi(argv[1]);
+	create_list(&mainlist);
+	read_mainlist();
+	set_tzone();
 
-	now = get_time();
-	tm = time_to_tm(now);
-	tm1.tm_sec   = 0;
-	tm1.tm_min   = i % 100;
-	tm1.tm_hour  = (i/100) % 100;
-	tm1.tm_mday  = n == 8 ? (i/10000)%100     : tm->tm_mday;
-	tm1.tm_mon   = n == 8 ? (i/1000000)%100-1 : tm->tm_mon;
-	tm1.tm_year  = curr_year + (tm1.tm_mon < tm->tm_mon);
-	trigger = tm_to_time(&tm1);
+	n = strlen(argv[1]);
+	for (j=0; j < n; j++)
+		if (argv[1][j] < '0' || argv[1][j] > '9')
+			break;
+	if (j == n) {				/* digits only: [mmdd]hhmm */
+		if (n != 4 && n != 8 && j == n)
+			usage();
+		i = atoi(argv[1]);
+		now = get_time();
+		tm = time_to_tm(now);
+		tm1.tm_sec   = 0;
+		tm1.tm_min   = i % 100;
+		tm1.tm_hour  = (i/100) % 100;
+		tm1.tm_mday  = n == 8 ? (i/10000)%100     : tm->tm_mday;
+		tm1.tm_mon   = n == 8 ? (i/1000000)%100-1 : tm->tm_mon;
+		tm1.tm_year  = curr_year + (tm1.tm_mon < tm->tm_mon);
+		trigger = tm_to_time(&tm1);
+	} else {				/* try free-form date+time */
+		trigger = parse_datetimestring(argv[1]);
+		if (!trigger)
+			usage();
+	}
 	*msg = 0;
-	for (i=2; i < argc; i++) {
+	memset(&edit, 0, sizeof(struct edit));
+	for (i=2; i+1 < argc && *argv[i] == '-'; i+=2)
+		switch(argv[i][1]) {
+		  case 'l':
+			edit.entry.length = parse_timestring(argv[i+1], TRUE);
+			break;
+		  case 'w':
+			edit.entry.early_warn = atoi(argv[i+1]) * 60;
+			break;
+		  case 'W':
+			edit.entry.late_warn = atoi(argv[i+1]) * 60;
+			break;
+		  default:
+			usage();
+		}
+	for (; i < argc; i++) {
 		if (strlen(msg) + strlen(argv[i]) > 1021)
 			break;
 		strcat(msg, argv[i]);
 		if (i < argc-1)
 			strcat(msg, " ");
 	}
-	for (i=0; i < sizeof(struct entry); i++)
-		((char *)&entry)[i] = 0;
-	entry.note = msg;
-	entry.time = trigger;
-
-	create_list(&mainlist);
-	read_mainlist();
+	edit.editing	= TRUE;
+	edit.changed	= TRUE;
+	edit.entry.note = mystrdup(msg);
+	edit.entry.time = trigger;
 	if (user)
-		entry.user = mystrdup(user[0].name);
-	(void)add_entry(&mainlist, &entry);
+		edit.entry.user = mystrdup(user[0].name);
+
+	confirm_new_entry();
 	(void)write_mainlist();
 	resynchronize_daemon();
 	printf("%s %s \"%.57s\"\n", mkdatestring(trigger),
@@ -375,9 +412,8 @@ static int dumptoday(date, ndays, lengthmode, others)
 XtInputId register_X_input(fd)
 	int		fd;		/* new connection */
 {
-	if (app)
-		return(XtAppAddInput(app, fd, (XtPointer)XtInputReadMask,
-					socket_callback, (XtPointer)0));
+	return(app ? XtAppAddInput(app, fd, (XtPointer)XtInputReadMask,
+					socket_callback, (XtPointer)0) : 0);
 }
 
 
@@ -422,20 +458,27 @@ static void sighand(sig)
 static void usage()
 {
 	fprintf(stderr,
-		"Usage 1: %s [options]\nOptions:\n%s%s%s%s%s%s%s%s%s%s\n",
-		progname,
-		"\t-h\tprint this help text\n",
-		"\t-d\tdump fallback app-defaults and exit\n",
-		"\t-v\tprint version string\n",
-		"\t-t [D [n]]\ttext dump of n days, starting with date D\n",
-		"\t-T [D [n]]\tsame as -t, print end time instead of length\n",
-		"\t-o\tmodifies -t/-T output to include other users\n",
-		"\t\teg: -t, -t +3, -T tomorrow, -o -t 24.12. 3, -o -T wed 7\n",
-		"\t-s\tstandalone, don't offer to start daemon\n",
-		"\t-f\tdon't fork on startup\n",
-		"\t-k\tignore lock file and start up in any case\n");
-	fprintf(stderr, "Usage 2: %s [mmdd]hhmm [message]*\n%s\n", progname,
-			"\tAdd appointment at mm/dd hh:mm (default is today)");
+	    "Usage 1: %s [options]\n\tOptions:\n%s%s%s%s%s%s%s%s%s%s%s\n",
+	    progname,
+	    "\t-h\t\tprint this help text\n",
+	    "\t-d\t\tdump fallback app-defaults and exit\n",
+	    "\t-v\t\tprint version string\n",
+	    "\t-t [D [n]]\ttext dump of n days, starting with date D\n",
+	    "\t-T [D [n]]\tsame as -t, print end time instead of length\n",
+	    "\t-o\t\tmodifies -t/-T output to include other users\n",
+	    "\t\t\teg: -t, -t +3, -T tomorrow, -o -t 24.12. 3, -o -T wed 7\n",
+	    "\t-s\t\tstandalone, don't offer to start daemon\n",
+	    "\t-S\t\tautomatically start daemon if it doesn't exist\n",
+	    "\t-f\t\tdon't fork on startup\n",
+	    "\t-k\t\tignore lock file and start up in any case\n");
+	fprintf(stderr,
+	    "Usage 2: %s [mmdd]hhmm [options] [message]*\n%s%s%s%s%s",
+	    progname,
+	    "\tAdd appointment at mm/dd hh:mm (default is today).\n",
+	    "\tOptions:\n",
+	    "\t-l T\t\tappointment length is T (eg., 1:00 is one hour)\n",
+	    "\t-w N\t\tearly warning is N minutes before hhmm time\n",
+	    "\t-W N\t\tlate warning is N minutes before hhmm time\n");
 	exit(1);
 }
 
@@ -465,110 +508,14 @@ void exit(ret)
 
 
 /*---------------------------------------------------------------------------*/
-/*
- * draw some text into a button. This is here because it's used by many
- * routines. There are two versions, stdarg and varargs, because Sun has
- * the gall to ship a pre-ansi compiler unless you pay extra...
- */
-
-#ifndef VARARGS
-void print_button(Widget w, char *fmt, ...)
-{
-	va_list			parm;
-	Arg			args;
-	XmString		string;
-	char			buf[1024];
-
-	if (fmt && w) {
-		va_start(parm, fmt);
-		vsprintf(buf, fmt, parm);
-		va_end(parm);
-		string = XmStringCreateSimple(*buf ? buf : " ");
-		XtSetArg(args, XmNlabelString, string);
-		XtSetValues(w, &args, 1);
-		XmStringFree(string);
-	}
-}
-
-void print_text_button(Widget w, char *fmt, ...)
-{
-	va_list			parm;
-	char			buf[1024];
-
-	if (w) {
-		va_start(parm, fmt);
-		vsprintf(buf, fmt, parm);
-		va_end(parm);
-		XmTextSetString(w, *buf ? buf : " ");
-		XmTextSetInsertionPosition(w, strlen(buf));
-	}
-}
-
-#else /* VARARGS */
-
-void print_button(w, fmt, a, b, c, d)
-	Widget			w;
-	char			*fmt;
-	int			a, b, c, d;
-{
-	Arg			args;
-	XmString		string;
-	char			buf[1024];
-
-	if (fmt && w) {
-		sprintf(buf, fmt, a, b, c, d);
-		string = XmStringCreateSimple(buf);
-		XtSetArg(args, XmNlabelString, string);
-		XtSetValues(w, &args, 1);
-		XmStringFree(string);
-	}
-}
-
-void print_text_button(w, fmt, a, b, c, d)
-	Widget			w;
-	char			*fmt;
-	int			a, b, c, d;
-{
-	XmString		string;
-	char			buf[1024];
-
-	if (w) {
-		sprintf(buf, fmt, a, b, c, d);
-		string = XmStringCreateSimple(buf);
-		XmTextSetString(w, *buf ? buf : " ");
-		XmTextSetInsertionPosition(w, strlen(buf));
-		XmStringFree(string);
-	}
-}
-#endif /* VARARGS */
-
-
-
-/*
- * draw a pixmap into a button, or remove the pixmap. The button must have
- * been a pixmap button all along, this routine can't mix text and pixmaps.
- * If <n> is -1, draw a blank pixmap.
- */
-
-void print_pixmap(w, n)
-	Widget			w;		/* button to draw into */
-	int			n;		/* pixmap #, one of PIC_* */
-{
-	Arg			args;
-
-	if (w) {
-		XtSetArg(args, XmNlabelPixmap, pixmap[n >= 0 ? n : PIC_BLANK]);
-		XtSetValues(w, &args, 1);
-	}
-}
-
-
 /*
  * read resources and put them into the config struct. This routine is used
  * for getting three types of resources: res_type={XtRInt,XtRBool,XtRString}.
  */
 
-void get_rsrc(ret, res_name, res_class_name, res_type)
+#define get_rsrc(A,B,C,D) do_get_rsrc((XtPointer)A, B, C, D)
+
+void do_get_rsrc(ret, res_name, res_class_name, res_type)
 	XtPointer	ret;
 	char		*res_name;
 	char		*res_class_name;
@@ -624,12 +571,12 @@ static void init_resources()
 	get_rsrc(&c->yov_wheight,    "yovWHeight",     "YovWHeight",   XtRInt);
 	get_rsrc(&c->yov_daywidth,   "yovDayWidth",    "YovDayWidth",  XtRInt);
 	get_rsrc(&c->showicontime,   "showIconTime",   "ShowIconTime",XtRBool);
+	get_rsrc(&c->showicondate,   "showIconDate",   "ShowIconDate",XtRBool);
 	get_rsrc(&c->noiconclock,    "noIconClock",    "NoIconClock", XtRBool);
 	get_rsrc(&c->frame_today,    "frameToday",     "FrameToday",  XtRBool);
 	get_rsrc(&c->noicon,	     "noIcon",         "NoIcon",      XtRBool);
 	get_rsrc(&c->sgimode,        "sgiMode",        "SgiMode",     XtRBool);
 	get_rsrc(&c->nomonthshadow,  "noMonthShadow", "NoMonthShadow",XtRBool);
-
 	get_rsrc(&startup_as,        "startupAs",     "StartupAs",  XtRString);
 	config.smallmonth = startup_as && *startup_as == 's';
 }
@@ -656,7 +603,6 @@ static void init_colors()
 		  case COL_BACK:	n = "colBack";		d=0;	break;
 		  case COL_CALBACK:	n = "colCalBack";	d=0;	break;
 		  case COL_CALSHADE:	n = "colCalShade";	d=0;	break;
-		  case COL_CALACT:	n = "colCalAct";	d=0;	break;
 		  case COL_CALTODAY:	n = "colCalToday";	d=0;	break;
 		  case COL_CALFRAME:	n = "colCalFrame";	d=0;	break;
 		  case COL_GRID:	n = "colGrid";		d=1;	break;
@@ -698,6 +644,8 @@ static void init_colors()
 		  case COL_WUSER_5:	n = "colWeekUser_5";	d=0;	break;
 		  case COL_WUSER_6:	n = "colWeekUser_6";	d=0;	break;
 		  case COL_WUSER_7:	n = "colWeekUser_7";	d=0;	break;
+		  case COL_GRAYICON:	n = "colGrayIcon";	d=0;	break;
+		  case COL_WIREFRAME:	n = "colWireFrame";	d=1;	break;
 		}
 		strcpy(class_name, n);
 		class_name[0] &= ~('a'^'A');
@@ -718,14 +666,6 @@ static void init_colors()
 }
 
 
-void set_color(col)
-	int			col;
-{
-	XSetForeground(display, jgc, color[col]);
-	XSetForeground(display, gc,  color[col]);
-}
-
-
 /*
  * load all fonts and make them available in the "fonts" struct. They are
  * loaded into the GC as necessary.
@@ -787,18 +727,31 @@ static PID_T		daemon_pid;	/* process ID of daemon (gets SIGHUP)*/
 
 static void init_daemon()
 {
-	int			lockfd;		/* lockfile descriptor */
+	int			i, lockfd;	/* lockfile descriptor */
 	char			path[256];	/* lockfile path */
 	char			buf[12];	/* contents of file (pid) */
 
 	daemon_pid = 0;
 	sprintf(path, LOCK_PATH, (int)getuid());
 	if ((lockfd = open(path, O_RDONLY)) < 0) {
-		if (!nodaemon) {
-			fprintf(stderr, "%s: WARNING: no daemon: ",progname);
-			perror("");
+		if (rundaemon) {
+			run_daemon(0);
+			for (i=0; i < 5; i++) {
+				if ((lockfd = open(path, O_RDONLY)) >= 0)
+					break;
+				sleep(1);
+			}
+		}
+		if (lockfd < 0) {
+			if (!nodaemon) {
+				int e = errno;
+				fprintf(stderr, "%s: WARNING - no daemon, ",
+							progname);
+				errno = e;
+				perror(path);
+			}
+			return;
 		}
-		return;
 	}
 	if (read(lockfd, buf, 10) < 5) {
 		fprintf(stderr,"%s: WARNING: daemon lockfile %s unreadable: ",
