@@ -1,7 +1,7 @@
 /*
  * Main calendar window widgets.
  *
- *	clicked_calendar(x, y)		Press in calendar day box detected,
+ *	locate_in_month_calendar()	find out where we are in the calendar
  *					pop up a one-day list menu
  *	draw_month_year()		Draw the month and year into the
  *					main calendar window
@@ -9,22 +9,18 @@
  *					window
  *	draw_day(day)			Draw one day box into the day grid
  *					in the main calendar window
- *
- *	day_to_pos(x, y, day)		Return the position of the day box
- *					for a particular day.
- *	pos_to_day(day, x, y)		Reverse operation, convert position
- *					to day number.
  */
 
 #include <stdio.h>
+#include <assert.h>
 #include <time.h>
 #include <Xm/Xm.h>
 #include "cal.h"
 
 static BOOL set_day_bkg_color();
 static void draw_day_notes();
+static int day_to_pos(), pos_to_day();
 void draw_day();
-int day_to_pos(), pos_to_day();
 
 #ifdef JAPAN
 static int		mixedstrtok(), chkctype();
@@ -34,6 +30,7 @@ extern unsigned short	(*kanji2jis)();	/* kanji2jis == e2j or sj2j */
 #endif
 extern Display		*display;	/* everybody uses the same server */
 extern GC		jgc, gc;	/* graphic context, Japanese and std */
+extern GC		chk_gc;		/* checkered gc for light background */
 extern XFontStruct	*font[NFONTS];	/* fonts: FONT_* */
 extern struct mainmenu	mainmenu;	/* all important main window widgets */
 extern struct config	config;		/* global configuration data */
@@ -47,34 +44,48 @@ extern struct holiday	holiday[366];	/* info for each day, separate for */
 extern struct holiday	sm_holiday[366];/* full-line texts under, and small */
 extern short		monthbegin[12];	/* julian date each month begins with*/
 extern short		monthlen[];	/* length of each month */
-
-					/* most recent popup made for: (only */
-					/* used to keep track of yellow box) */
-int			edit_month;	/*  month being edited, 0..11 */
-int			edit_year;	/*  year being edited, since 1900 */
-int			edit_day;	/*  day being edited, 1..31 or 0 */
-
+static int		lines_per_day;	/* max # of entries in a day box */
+static int		line_height;	/* height of one entry line in pixels*/
+static struct entry	**drawn;	/* 31*n entries drawn in month cal, */
+					/* for locating entries for drag&drop*/
 extern char *weekday_name[];
 extern char *monthname[];
 
 
 /*
- * got a click in the calendar area. If it's in a grid box with a valid
- * day init, "edit" it by storing its day number in the edit_* variables.
- * The previously edited day, and the new edited day are redrawn to move
- * the yellow highlight. If the user clicked to the left of the calendar,
- * create a week view.
- * Also, create a day popup menu.
+ * for a given position on the canvas, identify the entry there and return
+ * a pointer to it and which area it was hit in (one of the M_ constants).
+ * Locate an event if epp!=0, and return exactly where the event was hit.
+ * If none was found, return the time if possible for creating new events.
+ * If epp is 0, we are dragging an event right now; just return a new time.
+ * In month calendars, there are no multiday appointments and the trigger
+ * always agrees with the time if set.
+ *
+ *   ret	*epp	*time	*trig	xp..ysp	event
+ *
+ *   M_ILLEGAL	0	0	-	-	no recognizable position
+ *   M_OUTSIDE	0	time	-	-	no event but in a valid day
+ *   M_INSIDE	entry	time	trigg	-	in event center, edit it
+ *   M_LEFT	entry	time	trigg	pos/sz	near left edge, move start date
+ *   M_RIGHT	entry	time	trigg	pos/sz	near right edge, move end date
+ *   M_DAY_CAL	-	time	-	-	pop up a day calendar view
+ *   M_WEEK_CAL	-	time	-	-	pop up a week calendar view
  */
 
-void clicked_calendar(x, y)
-	int		x, y;		/* pixel pos clicked */
+MOUSE locate_in_month_calendar(epp, warn, time, trigger, xp,yp,xsp,ysp, xc,yc)
+	struct entry	**epp;		/* returned entry or 0 */
+	BOOL		*warn;		/* is *epp an n-days-ahead w?*/
+	time_t		*time;		/* if *epp=0, clicked where */
+	time_t		*trigger;	/* if *epp=0, entry trigger */
+	int		*xp,  *yp;	/* if M_INSIDE, rubber box */
+	int		*xsp, *ysp;	/* position and size */
+	int		xc, yc;		/* pixel pos clicked */
 {
+	register struct config	*c = &config;
 	int		xd, yd;		/* top left of clicked day box */
-	int		day;		/* clicked day number */
+	int		day, n;		/* clicked day number, note line */
 	struct tm	tm;		/* for pos->time_t conversion */
-	time_t		time;		/* time of day box, in sec */
-	int		sm = config.smallmonth;
+	int		sm = c->smallmonth;
 
 	tm.tm_year = curr_year;
 	tm.tm_mon  = curr_month;
@@ -82,11 +93,12 @@ void clicked_calendar(x, y)
 	tm.tm_hour = 0;
 	tm.tm_min  = 0;
 	tm.tm_sec  = 0;
-	if (x < config.calbox_marg[sm] + config.calbox_arrow[sm]) {
-		y -= config.calbox_marg[sm] + config.calbox_title[sm];
-		y /= config.calbox_ys[sm];
+							/* call week view? */
+	if (xc < c->calbox_marg[sm] + c->calbox_arrow[sm]) {
+		yc -= c->calbox_marg[sm] + c->calbox_title[sm];
+		yc /= c->calbox_ys[sm];
 		(void)day_to_pos(&xd, &yd, 1);
-		tm.tm_mday = 7 * y - xd +1;
+		tm.tm_mday = 7 * yc - xd +1;
 		if (tm.tm_mday < 1) {
 			if (!tm.tm_mon--) {
 				tm.tm_mon = 11;
@@ -95,42 +107,45 @@ void clicked_calendar(x, y)
 			tm.tm_mday += monthlen[tm.tm_mon] +
 					(tm.tm_mon==1 && !(tm.tm_year&3));
 		}
-		if ((time = tm_to_time(&tm)) != (time_t)-1) {
-			curr_week = time;
-			create_view('w');
-		}
-		return;
+		*time = tm_to_time(&tm);
+		return(*time != (time_t)-1 ? M_WEEK_CAL : M_ILLEGAL);
 	}
-	if (!pos_to_day(&day, x, y))
-		return;
+							/* call day view? */
+	if (!pos_to_day(&day, xc, yc))
+		return(M_ILLEGAL);
 	if (day_to_pos(&xd, &yd, day)) {
-		yd = config.calbox_marg[sm] + config.calbox_title[sm] +
-						yd * config.calbox_ys[sm];
-		if (y < yd + font[FONT_DAY+sm]->max_bounds.ascent) {
+		yd = c->calbox_marg[sm] + c->calbox_title[sm] +
+							yd * c->calbox_ys[sm];
+		if (yc < yd + font[FONT_DAY+sm]->max_bounds.ascent) {
 			tm.tm_mday = day;
-			curr_day = tm_to_time(&tm);
-			create_view('d');
-			return;
+			*time = tm_to_time(&tm);
+			return(M_DAY_CAL);
 		}
 	}
-	if (edit_day) {
-		int oldday = edit_day;
-		edit_day = 0;
-		draw_day(oldday);
-		draw_year_day(oldday, edit_month, edit_year);
-		draw_week_day(oldday, edit_month, edit_year);
+							/* hit appointment */
+	if (warn)
+		*warn = FALSE;
+	if (epp)
+		*epp = 0;
+	if (*time = date_to_time(day, curr_month, curr_year, 0, 0, 0)) {
+		int day_xs = c->calbox_xs[sm];
+		int yo = font[FONT_DAY+sm]->max_bounds.ascent;
+		xd  = c->calbox_marg[sm] + c->calbox_arrow[sm] + xd * day_xs;
+		yd += yo + (c->calbox_ys[sm]-yo-lines_per_day*line_height)/2+3;
+		n   = (yc - yd) / line_height;
+		assert(drawn && day >= 1 && day <= 31);
+		if (!epp || n < 0 || n > lines_per_day ||
+				!(*epp = drawn[(day-1) * lines_per_day + n]))
+			return(M_OUTSIDE);
+		if (xp) {
+			*xp   = xd + 1;
+			*yp   = yd + n * line_height;
+			*xsp  = day_xs - 5;
+			*ysp  = line_height - 1;
+		}
+		return(xc < xd + day_xs/3   ? M_LEFT  : M_INSIDE);
 	}
-	if (!can_edit_appts())
-		return;
-	edit_day   = day;
-	edit_month = curr_month;
-	edit_year  = curr_year;
-	draw_day(edit_day);
-	draw_year_day(edit_day, edit_month, edit_year);
-	draw_week_day(edit_day, edit_month, edit_year);
-	if (time = date_to_time(day, curr_month, curr_year, 0, 0, 0))
-		create_list_popup(mainlist, time,
-			(time_t)0, (char *)0, (struct entry *)0, 0, 0);
+	return(M_ILLEGAL);
 }
 
 
@@ -147,11 +162,11 @@ void draw_month_year()
 
 
 /*
- * draw all day boxes
+ * draw all day boxes into a clip region, or all if the region is null
  */
 
 void draw_calendar(region)
-	Region region;
+	Region			region;
 {
 	register struct config	*c = &config;
 	Window			window;
@@ -165,9 +180,10 @@ void draw_calendar(region)
 			XDestroyRegion(region);
 		return;
 	}
-	if (region)
-		XSetRegion(display, gc, region);
-
+	if (region) {
+		XSetRegion(display, gc,     region);
+		XSetRegion(display, chk_gc, region);
+	}
 	window = XtWindow(mainmenu.cal);
 	set_color(COL_CALBACK);				/* clear */
 	XFillRectangle(display, window, gc, 0, 0,
@@ -235,12 +251,13 @@ void draw_calendar(region)
 			c->calbox_marg[sm] + c->calbox_title[sm] * 3/4,
 			weekday_name[j%7], strlen(weekday_name[j%7]));
 
-	day_to_pos(&x, &y, 1);
+	day_to_pos(&x, &y, 1);				/* day boxes */
 	for (i=1-x; i < 43-x; i++)
 		draw_day(i);
 
 	if (region) {
-		XSetClipMask(display, gc, None);
+		XSetClipMask(display, gc,     None);
+		XSetClipMask(display, chk_gc, None);
 		XDestroyRegion(region);
 	}
 }
@@ -274,7 +291,6 @@ void draw_day(day)
 
 	if (!mainmenu.cal)
 		return;
-
 	orig_day = day;
 	tmp_curr_month = curr_month;
 	tmp_curr_year  = curr_year;
@@ -326,6 +342,7 @@ void draw_day(day)
 	hp  =    &holiday[jul];
 	shp = &sm_holiday[jul];
 
+							/* day number */
 	set_color(othermonth    ? COL_NOTEOFF   :
 		  hp->daycolor	?  hp->daycolor :
 		  shp->daycolor	? shp->daycolor :
@@ -334,6 +351,7 @@ void draw_day(day)
 	XDrawString(display, window, gc,
 		x+2, y+font[FONT_DAY+sm]->max_bounds.ascent, buf, strlen(buf));
 
+							/* julian & week num */
 	if (config.julian || config.weeknum) {
 		int wkday, weeknum;
 		(void)date_to_time(day, curr_month, curr_year,
@@ -357,6 +375,7 @@ void draw_day(day)
 							buf, strlen(buf));
 
 	}
+							/* small holiday */
 	if (shp->string && (!config.julian && !config.weeknum ||
 				font[FONT_DAY+sm]->max_bounds.ascent >=
 				font[FONT_NOTE]->max_bounds.ascent * 2 - 1)) {
@@ -386,10 +405,12 @@ void draw_day(day)
 							buf, strlen(buf));
 #endif
 	}
+							/* entry notes */
 	set_color(COL_STD);
 	curr_month = tmp_curr_month;
 	curr_year  = tmp_curr_year;
 	draw_day_notes(orig_day);
+							/* opt. today frame */
 	if (today && config.frame_today) {
 		XDrawRectangle(display, window, gc, x+1, y+1,
 			       config.calbox_xs[sm]-6, config.calbox_ys[sm]-6);
@@ -401,6 +422,8 @@ void draw_day(day)
 
 /*
  * draw (or undraw) the notes in a day box. Looks up the day in the database.
+ * Remember which entry was drawn where; this simplifies drag&drop because
+ * it's easy to find the entry under the cursor.
  */
 
 static void draw_day_notes(day)
@@ -429,10 +452,12 @@ static void draw_day_notes(day)
 	char			*errmsg;	/* parser error */
 	Window			window;
 	int			sm = config.smallmonth;
+	int			ascent, descent;
+	struct entry		**drawn_entry = 0;
 
 	if (!mainmenu.cal)
 		return;
-
+							/* which day box? */
 	orig_day = day;
 	tmp_curr_month = curr_month;
 	tmp_curr_year = curr_year;
@@ -467,68 +492,67 @@ static void draw_day_notes(day)
 	today  = day        == tm->tm_mday &&
 		 curr_month == tm->tm_mon  &&
 		 curr_year  == tm->tm_year;
-
+							/* day box geometry */
 	x  = config.calbox_marg[sm] + x * config.calbox_xs[sm] + 3 +
 						config.calbox_arrow[sm];
 	y  = config.calbox_marg[sm] + y * config.calbox_ys[sm] + 3 +
 						config.calbox_title[sm];
 	ye = y + config.calbox_ys[sm] - 3;
-#ifdef JAPAN
-	ys = font[FONT_JNOTE]->max_bounds.ascent +
-	     font[FONT_JNOTE]->max_bounds.descent/2;
-	if (ys < font[FONT_NOTE]->max_bounds.ascent +
-		 font[FONT_NOTE]->max_bounds.descent/2)
-#endif
-	ys = font[FONT_NOTE]  ->max_bounds.ascent +
-	     font[FONT_NOTE]  ->max_bounds.descent/2;
-	y += font[FONT_DAY+sm]->max_bounds.ascent + (ye - y) % ys / 2;
 
-	(void)set_day_bkg_color(day);
-	XFillRectangle(display, window, gc, x,y, config.calbox_xs[sm]-3, ye-y);
+	ascent  = font[FONT_NOTE]->max_bounds.ascent;
+	descent = font[FONT_NOTE]->max_bounds.descent/2;
 	XSetFont(display, gc,  font[FONT_NOTE]->fid);
 #ifdef JAPAN
 	XSetFont(display, jgc, font[FONT_JNOTE]->fid);
+	if (ascent + descent < font[FONT_JNOTE]->max_bounds.ascent +
+			       font[FONT_JNOTE]->max_bounds.descent/2) {
+		ascent  = font[FONT_JNOTE]->max_bounds.ascent;
+		descent = font[FONT_JNOTE]->max_bounds.descent/2;
+	}
 #endif
+	ys = ascent + descent;
+	y += font[FONT_DAY+sm]->max_bounds.ascent + (ye - y) % ys / 2;
 
+	if (!othermonth) {				/* drawn entry buffer*/
+		i = (ye - y) / ys;
+		if (!drawn || i > lines_per_day) {
+			if (drawn)
+				free(drawn);
+			drawn = (struct entry **)malloc(31 * i *
+							sizeof(struct entry *));
+			memset(drawn, 0, 31 * i * sizeof(struct entry *));
+			lines_per_day = i;
+			line_height   = ys;
+		}
+		drawn_entry = drawn + (day - 1) * i;
+		memset(drawn_entry, 0, i * sizeof(struct entry *));
+	}
+							/* background */
+	(void)set_day_bkg_color(day);
+	XFillRectangle(display, window, gc, x,y, config.calbox_xs[sm]-3, ye-y);
+
+							/* large holiday? */
 	if (errmsg = parse_holidays(curr_year, FALSE))
 		create_error_popup(mainmenu.cal, 0, errmsg);
 	jul = monthbegin[curr_month] +day-1 + (curr_month>1 && !(curr_year&3));
 	hp  = &holiday[jul];
-#ifdef JAPAN
-	if (hp->string && *hp->string &&
-			y + ((font[FONT_NOTE ]->max_bounds.descent <
-			      font[FONT_JNOTE]->max_bounds.descent ?
-			      font[FONT_JNOTE]->max_bounds.descent :
-			      font[FONT_NOTE ]->max_bounds.descent)<<2) <= ye){
-		
-		y += (font[FONT_NOTE ]->max_bounds.ascent <
-		      font[FONT_JNOTE]->max_bounds.ascent) ?
-		      font[FONT_JNOTE]->max_bounds.ascent :
-		      font[FONT_NOTE ]->max_bounds.ascent;
-
+	if (hp->string && *hp->string && y + descent*2 <= ye) {
+		y += ascent;
 		set_color(othermonth ? COL_NOTEOFF :
 			   hp->stringcolor ? hp->stringcolor : COL_NOTE);
-		mixedstr_draw(window, x+2, y, hp->string, config.calbox_xs[sm] - 3,
-			      FONT_NOTE, FONT_JNOTE);
-		y += (font[FONT_NOTE ]->max_bounds.descent <
-		      font[FONT_JNOTE]->max_bounds.descent) ?
-		      font[FONT_JNOTE]->max_bounds.descent/2 :
-		      font[FONT_NOTE ]->max_bounds.descent/2;
-	}
+#ifdef JAPAN
+		mixedstr_draw(window, x+2, y, hp->string,
+				config.calbox_xs[sm]-3, FONT_NOTE, FONT_JNOTE);
 #else
-	if (hp->string && *hp->string &&
-				y + font[FONT_NOTE]->max_bounds.descent
-				  + font[FONT_NOTE]->max_bounds.descent <= ye){
 		strncpy(buf, hp->string, sizeof(buf)-1);
 		truncate_string(buf, config.calbox_xs[sm] - 3, FONT_NOTE);
-		y += font[FONT_NOTE]->max_bounds.ascent;
-		set_color(othermonth ? COL_NOTEOFF :
-			   hp->stringcolor ? hp->stringcolor : COL_NOTE);
 		XDrawString(display, window, gc, x+2, y, buf, strlen(buf));
-		y += font[FONT_NOTE]->max_bounds.descent/2;
-	}
 #endif
-
+		y += descent/2;
+		if (drawn_entry)
+			drawn_entry++;
+	}
+							/* entry loop */
 	found = lookup_entry(&lookup, mainlist, daytime, TRUE, FALSE, 'm', -1);
 	for (; found; found = lookup_next_entry(&lookup)) {
 
@@ -560,16 +584,23 @@ static void draw_day_notes(day)
 			break;
 		}
 		if (config.colorcode_m && u > 0 && !othermonth) {
-			set_color(COL_WUSER_0 + user[u].color);
+			set_color(COL_WUSER_0 + (user[u].color & 7));
 			XFillRectangle(display, window, gc,
-				x, y+1, config.calbox_xs[sm] - 3,
-				font[FONT_NOTE]->max_bounds.ascent+1);
+					x, y+1, config.calbox_xs[sm] - 3,
+					font[FONT_NOTE]->max_bounds.ascent+1);
+			if (user[u].color > 7)
+				XFillRectangle(display, window, chk_gc,
+					x, y+1, config.calbox_xs[sm] - 3,
+					font[FONT_NOTE]->max_bounds.ascent+1);
 		}
 		if (ep->notime)
-			sprintf(buf, "%.90s",  mknotestring(ep));
+			sprintf(buf, "%s%.90s", mknotestring(ep),
+						lookup.days_warn ? "W: " : "");
 		else
-			sprintf(buf, "%s %.90s", mktimestring(lookup.trigger,
-						FALSE), mknotestring(ep));
+			sprintf(buf, "%s %s%.90s", mktimestring(lookup.trigger,
+								FALSE),
+						lookup.days_warn ? "W: " : "",
+						mknotestring(ep));
 #ifdef JAPAN
 		y += (font[FONT_NOTE ]->max_bounds.ascent <
 		      font[FONT_JNOTE]->max_bounds.ascent) ?
@@ -594,10 +625,12 @@ static void draw_day_notes(day)
 		XDrawString(display, window, gc, x+2+xo, y, buf, strlen(buf));
 #endif
 		y += font[FONT_NOTE]->max_bounds.descent/2;
+		if (drawn_entry)
+			*drawn_entry++ = ep;
 	}
 	set_color(COL_STD);
 	curr_month = tmp_curr_month;
-	curr_year = tmp_curr_year;
+	curr_year  = tmp_curr_year;
 }
 
 
@@ -605,23 +638,13 @@ static BOOL set_day_bkg_color(day)
 	int			day;		/* 1..31 */
 {
 	struct tm		*tm;
+	BOOL			ret;
 
 	tm = time_to_tm(get_time());
-	if	  (day        == edit_day    &&
-		   curr_year  == edit_year   &&
-		   curr_month == edit_month) {
-						set_color(COL_CALACT);
-						return(FALSE);
-
-	} else if (day        == tm->tm_mday &&
-		   curr_month == tm->tm_mon  &&
-		   curr_year  == tm->tm_year) {
-						set_color(COL_CALTODAY);
-						return(TRUE);
-	} else {
-						set_color(COL_CALSHADE);
-						return(FALSE);
-	}
+	ret = day == tm->tm_mday && curr_month == tm->tm_mon
+				 && curr_year  == tm->tm_year;
+	set_color(ret ? COL_CALTODAY : COL_CALSHADE);
+	return(ret);
 }
 
 
@@ -630,7 +653,7 @@ static BOOL set_day_bkg_color(day)
  * convert day numbers 1..31 to a pixel position, and vice versa.
  */
 
-int day_to_pos(x, y, day)
+static int day_to_pos(x, y, day)
 	int			*x, *y;		/* returned box, 0..6/0..5 */
 	int			day;		/* 1..31 */
 {
@@ -638,13 +661,13 @@ int day_to_pos(x, y, day)
 
 	if (!date_to_time(day, curr_month, curr_year, &wkday, 0, 0))
 		return(FALSE);
-	*x = config.sunday_first ?  wkday : (wkday + 6) % 7;
+	*x = config.sunday_first ? wkday : (wkday + 6) % 7;
 	*y = (day-1 - *x + 6) / 7;
 	return(TRUE);
 }
 
 
-int pos_to_day(day, x, y)
+static int pos_to_day(day, x, y)
 	int			*day;		/* returned day #, 1..31 */
 	int			x, y;		/* pixel pos in drawable */
 {
