@@ -10,6 +10,7 @@ typedef void *Widget;	/* dummy for prototype declarations in config.h */
 #include <unistd.h>
 #include <stdarg.h>
 #include <time.h>
+#include <limits.h>
 #ifdef IBM
 #include <sys/select.h>
 #endif
@@ -23,6 +24,8 @@ typedef void *Widget;	/* dummy for prototype declarations in config.h */
 #endif
 #include <netinet/in.h>
 #include <netinet/tcp.h>
+#include <pwd.h>
+#include <grp.h>
 
 #include "netplan.h"
 #include "netplan_if.h"
@@ -33,20 +36,26 @@ typedef void *Widget;	/* dummy for prototype declarations in config.h */
 #define ORPHANIZE setsid()	/* signals. If -DBSD doesn't help, the */
 #endif				/* daemon works without, sort of. */
 
+#define ACLCHKINTER 60          /* interval (secs) for check of the acl file */
+
 static char		*progname;	/* name of this program, argv[0] */
 static BOOL		debug;		/* foreground, logging is enabled */
 static BOOL		foregnd;	/* foreground without logging */
-static BOOL		verbose;	/* debug mode, don't daemonize */
-static BOOL		allow_all;	/* no access list, never deny */
+BOOL			verbose;	/* debug mode, don't daemonize */
+static BOOL		allow_all = FALSE; /* no access list, never deny */
 static int		nclients;	/* size of array */
 static unsigned int	row_id = 1;	/* next available row ID */
-static unsigned int	file_id = 0;	/* next available file ID */
 static int		max_files;	/* size of file_list array */
 static struct pfile	*file_list;	/* array of open files */
-static struct client	*client_list;	/* array of clients (entry 0 unused) */
+struct client		*client_list;	/* array of clients (entry 0 unused) */
 static fd_set		rd0, wr0, ex0;	/* preset fd masks for main select */
 static time_t		update_time;	/* date of last unwritten change */
 extern int		errno;
+static BOOL             do_auth = FALSE;/* authentication required (identd) */
+int			netplan_port;	/* port we are listening on */
+
+static const char homepath[] = HOMEDIR "/" NETDIR "/.";
+static const char aclpath[]  = HOMEDIR "/" NETDIR "/" ACL_FILE;
 
 static int  init_inet_port(void);
 static BOOL message(int);
@@ -54,9 +63,9 @@ static void eval_message(int, char *);
 static void appendbuf(struct buffer *, char *, int);
 static void notify_client(int, unsigned int, int);
 static void close_client(int);
-static BOOL check_file(FILE *, char *, int);
-static int  open_file(char *, int);
-static void close_file(int, int);
+static BOOL check_file(FILE *, char *, int, struct client *);
+static int  open_file(char *, int, BOOL *);
+static void close_file(int, int, struct client *);
 static void write_file(int);
 static int  find_row(int, unsigned int, BOOL *);
 static unsigned int add_row(int, unsigned int, int, char *, int);
@@ -64,17 +73,24 @@ static void delete_row(int, unsigned int, int);
 static BOOL lock_row(int, unsigned int, int, BOOL);
 static void send_row(int, unsigned int, int, char);
 static int count_rows(int, int);
-static void log(char *, ...);
-static void reply(int, char *, ...);
+static void reply(int, struct client *, char *, ...);
 static void error(int, char *, ...);
 static void fatal(char *, ...);
-static char *ip_addr(register struct sockaddr_in *);
+char *ip_addr(register struct sockaddr_in *);
+static void setup_gids(struct client *, int, char *);
+static char *make_descr (struct client *, const char *username,
+			 const char *arg);
+void log(char *, ...);
 
-static void *allocate(int n)
+void *allocate(int n)
 	{void *p = malloc(n); if (!p) fatal("no memory"); return(p);}
-static void *reallocate(void *o, int n)
+void *reallocate(void *o, int n)
 	{void *p = o ? realloc(o, n) : malloc(n);
 	 if (!p) fatal("no memory"); return(p);}
+void release(void *p)
+	{if (p) free(p);}
+char *mystrdup(const char *s)
+	{char *p = (char *)allocate(strlen(s)+1); strcpy(p, s); return(p);}
 
 static char usage[] = "recognized server commands:\\\n\
 (S=success(t|f), V=version, F=file ID, R=row ID)\\\n\\\n\
@@ -106,22 +122,31 @@ main(
 	struct timeval	timeout;	/* for auto-sync after ten seconds */
 	fd_set		rd, wr, ex;	/* returned fd masks from select */
 	int		fd;		/* file descriptor/counter */
-	int		c, n, on;
+	int		c, i, n, on;
 	struct client	*cp;
-	char		*path;
+	time_t          last_acl_check = 0;
+	time_t          acl_mod_time = 0;
+	time_t          t;
+	struct stat     st;
+	static const char nobody[] = "anonymous";
+                                        /* descriptive name of unknown users */
 
 	progname = argv[0];				/* cmd line options */
-	while ((c = getopt(argc, argv, "dv")) != EOF)
+	while ((c = getopt(argc, argv, "advf")) != EOF)
 		switch(c) {
+		  case 'a': do_auth = TRUE;	break;
 		  case 'v': verbose = TRUE;	break;
 		  case 'd': debug   = TRUE;	break;
 		  case 'f': foregnd = TRUE;	break;
-		  default:  fatal("usage: %s [-f] [-d [-v]]\n\
--f=foreground, -d=debug, -v=verbose", progname);
+		  default:  fatal("usage: %s [-f] [-d [-v]] [-a]\n"
+				  "-f=foreground, -d=debug, -v=verbose "
+				  "-a=authentication via identd required",
+				  progname);
 		}
 
-	log("netplan version %s\n", DVERSION);
-	log("database directory is %s/%s\n", HOMEDIR, NETDIR);
+	t = time(0);
+	log("netplan version %s starting at %s", DVERSION, ctime(&t));
+	log("database directory is %.200s/%.200s\n", HOMEDIR, NETDIR);
 
 	if (NOBODY_UID == 0 || NOBODY_GID == 0)
 		fatal("netplan compiled with nobody == root");
@@ -169,14 +194,11 @@ main(
 
 	log("running with uid=%d gid=%d euid=%d egid=%d\n",
 				getuid(), getgid(), geteuid(), getegid());
-	path = allocate(strlen(HOMEDIR) + strlen(NETDIR) + 4);
-	sprintf(path, "%s/%s/.", HOMEDIR, NETDIR);
-	if (access(path, R_OK | W_OK | X_OK)) {
+	if (access(homepath, R_OK | W_OK | X_OK)) {
 		fprintf(stderr, "%s: no read/write access to ", argv[0]);
-		perror(path);
+		perror(homepath);
 		_exit(1);
 	}
-	free(path);
 	if (!debug && !foregnd) {
 		(void)umask(0077);
 		setuid(getuid());
@@ -194,11 +216,19 @@ main(
 			_exit(0);
 		}
 	}
-	path = HOMEDIR "/" NETDIR "/" ACL_FILE;		/* access list */
-	log("reading access list file %s\n", path);
-	if (allow_all = !acl_read(path))
-		log("cannot read %s, allowing everything\n", path);
-
+	last_acl_check = time(NULL);
+	if (!stat(aclpath, &st))
+		acl_mod_time = st.st_mtime;
+	log("reading access list file %.200s\n", aclpath);
+	if (!acl_read(aclpath)) {
+		if (do_auth)
+			log("cannot read %.200s, allowing NOTHING\n", aclpath);
+		else {
+			log("cannot read %.200s, allowing everything\n",
+								aclpath);
+			allow_all = TRUE;
+		}
+	}
 	nclients = sizeof(fd_set)*8;			/* max # of clients */
 	client_list = allocate(nclients * sizeof(struct client));
 	memset(client_list, 0, nclients * sizeof(struct client));
@@ -225,10 +255,31 @@ main(
 		}
 		if (n == 0)
 			continue;
+		if ((t = time(NULL)) > last_acl_check + ACLCHKINTER &&
+							!stat(aclpath, &st)) {
+			last_acl_check = t;
+			if (st.st_mtime > acl_mod_time) {
+				acl_mod_time = st.st_mtime;
+				log("rereading acl file\n");
+				acl_exit();
+				if (!acl_read(aclpath))
+					if (allow_all)
+						log("cannot read %.200s, "
+						    "allowing everything\n",
+						    aclpath);
+					else
+						log("cannot read %.200s, "
+						    "allowing NOTHING\n",
+						    aclpath);
+				else
+					allow_all = FALSE;
+			}
+		}
 
 		if (FD_ISSET(fd_inet, &rd)) {		/* connect */
 			n = sizeof(addr);
-			if ((fd = accept(fd_inet, &addr, &n)) < 0)
+			if ((fd = accept(fd_inet, (struct sockaddr*)&addr, &n))
+									< 0)
 				perror(progname);
 			FD_SET(fd, &rd0);
 			on = 1;
@@ -236,10 +287,18 @@ main(
 			on = 1;
 			setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, &on, 4);
 			cp = &client_list[fd];
+
 			memset(cp, 0, sizeof(struct client));
-			log("client %d connected from %s\n",fd,ip_addr(&addr));
-			reply(fd, "!t%s enter ? for help", DVERSION);
 			memcpy(&cp->addr, &addr, n);
+			cp->uid     = NOBODY_UID;
+			cp->gids[0] = NOBODY_GID;
+			cp->descr   = mystrdup(nobody);
+			cp->user    = mystrdup(nobody);
+			for (i=1; i < NGROUPS_MAX; i++)
+				cp->gids[i] = -1;
+
+			log("client %d connected from %s\n",fd,ip_addr(&addr));
+			reply(fd, cp, "!t%s enter ? for help", DVERSION);
 			cp->time = time(0);
 		}
 		for (fd=0; fd < nclients; fd++) {
@@ -247,10 +306,11 @@ main(
 				continue;
 			if (FD_ISSET(fd, &rd))			/* recv cmd */
 				if (!message(fd)) {
+					log("client %d (%.200s) disconnected\n"
+						,fd, client_list[fd].descr);
 					close_client(fd);
 					FD_CLR(fd, &rd0);
 					close(fd);
-					log("client %d disconnected\n", fd);
 				}
 			if (FD_ISSET(fd, &wr)) {		/* send reply*/
 				register struct buffer *b;
@@ -289,6 +349,76 @@ void exit(
 }
 
 
+/*
+ * fill in group access list for a connection
+ */
+
+static void setup_gids(
+	struct client	*cp,
+	int		gid,
+	char		*user)
+{
+	struct group	*gr;
+	int		ngroups = 0;
+	char		**p;
+
+	cp->gids[ngroups++] = gid;
+	setgrent();
+	while (gr = getgrent())
+		for (p=gr->gr_mem; *p; p++)
+			if (!strcmp(user, *p)) {
+				if (ngroups < NGROUPS_MAX)
+					cp->gids[ngroups] = gr->gr_gid;
+				ngroups++;
+				break;
+			}
+	if (ngroups > NGROUPS_MAX)
+		log("too many groups (%d, used %d)\n", ngroups, NGROUPS_MAX);
+}
+
+
+/*
+ * make a readable description for a client authenticated by identd
+ */
+
+static char *make_descr(
+	struct client	*cp,
+	const char	*username,
+	const char	*arg)
+{
+	char		buf[512];
+	struct hostent	*h = gethostbyaddr((const char *)&cp->addr.sin_addr,
+					sizeof(cp->addr.sin_addr), AF_INET);
+	char		*hostname = ip_addr(&cp->addr);
+	const char	*start_id = strchr(arg, '<');
+	const char	*end_id = strstr(arg, "pid=");
+	char		*p;
+
+	if (h)
+		hostname = h->h_name;
+	p = buf;
+	if (start_id) {
+		register int n = start_id - arg + 1;
+		if (n > 40) n = 40;
+		strncpy (p, arg, n);
+		p += n;
+	} else
+		*p++ = '<';
+
+	sprintf(p, "%.10s on %.20s", username, hostname);
+	p += strlen (p);
+	if (end_id > start_id) {
+		*p++ = ',';
+		strncpy(p, end_id, 40);
+		p[40] = 0;
+	} else {
+		*p++ = '>';
+		*p++ = 0;
+	}
+	return(mystrdup(buf));
+}
+
+
 /*
  * base socket connection, used for accepting
  */
@@ -300,22 +430,30 @@ static int init_inet_port(void)
 	struct servent		*serv;
 
 	fd = socket(AF_INET, SOCK_STREAM, 0);
-	if (fd < 0)
+	if (fd < 0) {
+		perror("socket");
 		fatal("cannot open socket (error %d)", errno);
-	if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on)))
+	}
+	if (setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on))) {
+		perror("setsockopt");
 		fatal("cannot setsockopt socket", 0);
+	}
 	if (!(serv = getservbyname("netplan", "tcp")))
 		log("netplan/tcp not found in /etc/services, using port %d\n",
 									PORT);
+	netplan_port = serv ? ntohs(serv->s_port) : PORT;
 	addr.sin_family      = AF_INET;
-	addr.sin_port	     = serv ? serv->s_port : htons(PORT);
+	addr.sin_port	     = htons(netplan_port);
 	addr.sin_addr.s_addr = INADDR_ANY;
-	if (bind(fd, &addr, sizeof(addr)))
+	if (bind(fd, (struct sockaddr *)&addr, sizeof(addr))) {
+		perror("bind");
 		fatal("cannot bind to port %d, is another daemon running?",
-								addr.sin_port);
-	if (listen(fd, 100))
-		fatal("cannot listen to socket bound to port %d",
-								addr.sin_port);
+								netplan_port);
+	}
+	if (listen(fd, 100)) {
+		perror("listen");
+		fatal("cannot listen to socket bound to port %d",netplan_port);
+	}
 	return(fd);
 }
 
@@ -333,13 +471,11 @@ static void close_client(
 	int		fid;
 
 	for (fid=0; fid < max_files; fid++)
-		close_file(fid, fd);
-	if (c->descr)
-		free(c->descr);
-	if (c->in.data)
-		free(c->in.data);
-	if (c->out.data)
-		free(c->out.data);
+		close_file(fid, fd, c);
+	release(c->descr);
+	release(c->user);
+	release(c->in.data);
+	release(c->out.data);
 	memset(c, 0, sizeof(*c));
 }
 
@@ -360,7 +496,8 @@ static void close_client(
  * fragments may arrive in the same network read).
  */
 
-static BOOL message(int fd)
+static BOOL message(
+	int		fd)
 {
 	register struct client *c = &client_list[fd];
 	register char	*p, *q;		/* for eliminating \r and \\n->\n */
@@ -393,13 +530,13 @@ static BOOL message(int fd)
 				begin = q;
 				c->in.enq = 0;
 			}
-			c->esc  = FALSE;
+			c->esc = FALSE;
 		}
 		appendbuf(&c->in, begin, q-begin);
 		if (MAXMSGSZ && c->in.enq > MAXMSGSZ) {
 			error(fd, "dropped huge message (%d bytes)",c->in.enq);
-			error(fd, "Communication error, ignoring\n excessively"
-				  " large message, expect\nsynchronization"
+			error(fd, "Communication error, ignoring\\\n excessive"
+				  "ly large message, expect\\\nsynchronization"
 				  " problems");
 		}
 		FD_ZERO(&rd);
@@ -413,6 +550,7 @@ static BOOL message(int fd)
 	return(TRUE);
 }
 
+
 #define SKIP(p)								   \
 	while (p - c->in.data < c->in.enq && *p != ' ' && *p != '\n') p++; \
 	if (p - c->in.data < c->in.enq && *p == ' ') p++;
@@ -425,26 +563,77 @@ static void eval_message(
 	register char	*p, *q;		/* for eliminating \r and \\n->\n */
 	register int	n, i;		/* number of bytes read from socket */
 	int		fid;		/* file ID argument */
+	BOOL            ronly;          /* read-only flag */
 	unsigned int	rid;		/* row ID argument */
+	char            *id;
+	struct passwd   *pw;
+	char            buf[4096];
+
+	static const char * const cmode[] = {
+		"not connected",
+		"readonly",
+		"readwrite",
+		"modified" };
 
 	if (verbose)
-		log("client %d sends %s", fd, arg);
+		log("client %d (%.200s) sends %.400s", fd, c->descr, arg);
 	switch(*arg++) {
 	  case '\n':
 		break;
 
 	  case '=':
 		arg[strlen(arg)-1] = 0;
-	  	if (c->descr)
-			free(c->descr);
-		c->descr = allocate(strlen(arg)+1);
-		strcpy(c->descr, arg);
-		if (p = strstr(arg, "uid="))
-			c->uid = atoi(p+4);
-		if (p = strstr(arg, "gid="))
-			c->gid = atoi(p+4);
-		log("client %d is %s (uid %d, gid %d)\n",
-			fd, arg, c->uid, c->gid);
+		if ((id = ident_id(fd)) && (pw = getpwnam(id))) {
+			release(c->descr);
+			release(c->user);
+			c->descr     = make_descr(c, id, arg);
+			c->user      = id;
+			c->uid       = pw->pw_uid;
+			c->auth_fail = FALSE;
+			setup_gids(c, pw->pw_gid, id);
+			endgrent();
+			sprintf(buf, "client %d: identd authenticated user "
+				 "%.100s, uid %d, gids", fd, c->user, c->uid);
+			p = buf + strlen(buf);
+			for (i=0; i < NGROUPS_MAX && c->gids[i] >= 0; i++) {
+				sprintf(p, " %d", c->gids[i]);
+				p += strlen(p);
+			}
+			strcpy(p, "\n");
+			log(buf);
+
+		} else if (do_auth) {
+			c->auth_fail = TRUE;
+			error(fd, "Unable to verify your identity using identd"
+				",\\\nonly public files will be accessible.");
+			log("client %d: identd authentication failed\n", fd);
+		} else {
+			release(c->descr);
+			c->descr = mystrdup(arg);
+			c->auth_fail = FALSE;
+			if (p = strstr(arg, "uid=")) {
+				c->uid = atoi(p+4);
+				if (pw = getpwuid(c->uid)) {
+					release(c->user);
+					c->user = mystrdup(pw->pw_name);
+					setup_gids(c, pw->pw_gid, pw->pw_name);
+				} else
+					log("client %d: WARNING: unknown uid "
+						"%d, owner and group ACLs will"
+						" be ineffective\n",fd,c->uid);
+			}
+			sprintf(buf, "client %d is %.200s (uid %d, gids",
+				 			fd, arg, c->uid);
+			p = buf + strlen(buf);
+			for (i=0; i < NGROUPS_MAX && c->gids[i] >= 0; i++) {
+				sprintf(p, " %d", c->gids[i]);
+				p += strlen(p);
+			}
+			strcpy(p, "\n");
+			log(buf);
+		}
+		release(id);
+		endpwent();
 		break;
 
 	  case 't':
@@ -457,19 +646,19 @@ static void eval_message(
 	  case 'o':
 		for (p=arg; *p && *p != '\n'; p++);
 		*p = 0;
-		if ((n = open_file(arg, fd)) < 0)
-			reply(fd, "oft0");
+		if ((n = open_file(arg, fd, &ronly)) < 0)
+			reply(fd, c, "oft0");
 		else
-			reply(fd, "ot%c%d", "wr"[file_list[n].readonly], n);
+			reply(fd, c, "ot%c%d", ronly ? 'r' : 'w', n);
 		break;
 
 	  case 'n':
 		fid = atoi(arg);
-		reply(fd, "n%d %d", fid, count_rows(fid, fd));
+		reply(fd, c, "n%d %d", fid, count_rows(fid, fd));
 		break;
 
 	  case 'c':
-		close_file(atoi(arg), fd);
+		close_file(atoi(arg), fd, c);
 		break;
 
 	  case 'r':
@@ -481,8 +670,12 @@ static void eval_message(
 	  case 'w':
 		fid = atoi(arg); SKIP(arg);
 		rid = atoi(arg); SKIP(arg);
-		rid = add_row(fid, rid, fd, arg, c->in.enq - (arg-c->in.data));
-		reply(fd, "w%c%d", "tf"[!rid], rid);
+		if (fid < 0 || fid >= max_files || file_list[fid].client[fd]<2)
+			rid = 0; /* readonly or bad file */
+		else
+			rid = add_row(fid, rid, fd, arg,
+				      c->in.enq - (arg-c->in.data));
+		reply(fd, c, "w%c%d", "tf"[!rid], rid);
 		break;
 
 	  case 'd':
@@ -494,7 +687,7 @@ static void eval_message(
 	  case 'l':
 		fid = atoi(arg); SKIP(arg);
 		rid = atoi(arg);
-		reply(fd, "l%c", "ft"[lock_row(fid, rid, fd, TRUE)]);
+		reply(fd, c, "l%c", "ft"[lock_row(fid, rid, fd, TRUE)]);
 		break;
 
 	  case 'u':
@@ -507,12 +700,13 @@ static void eval_message(
 		BOOL pending_nl = FALSE;
 		struct client *cp;
 		struct buffer *out = &client_list[fd].out;
-		char *msg = allocate(4096);
+		char msg[512];
 		for (i=0; i < max_files; i++) {
 			register struct pfile *f = &file_list[i];
 			if (!f->name) continue;
-			sprintf(msg, "\\\n.file %2d: ro=%d mod=%d err=%d "
-					"type=%d nrows=%d name=\"%.255s\"",
+			sprintf(msg, "\\\n.file %2d: ro=%d mod=%d "
+				"err=%d type=%d nrows=%d "
+				"name=\"%.20s\"",
 					i, f->readonly, f->modified,
 					f->error, f->type, f->nrows, f->name);
 			n = pending_nl ? 0 : 2;
@@ -520,11 +714,11 @@ static void eval_message(
 			FD_SET(fd, &wr0);
 			for (cp=client_list, n=0; n < nclients; n++, cp++) {
 				if (!f->client[n]) continue;
-				sprintf(msg, "\\\n.client %2d: %s @ %s, "
-							"since %.15s%s",
-					n, cp->descr ? cp->descr : "<unknown>",
-					ip_addr(&cp->addr), ctime(&cp->time)+4,
-					f->client[n]==2 ? " (modified)" : "");
+				sprintf(msg, "\\\n.\tclient %2d: %.50s@"
+					"%.20s, since %.15s %.20s",
+					n, cp->descr, ip_addr(&cp->addr),
+					ctime(&cp->time)+4,
+					cmode[f->client[n]]);
 				appendbuf(out, msg, strlen(msg));
 				FD_SET(fd, &wr0);
 			}
@@ -532,11 +726,10 @@ static void eval_message(
 		}
 		appendbuf(out, "\n", 1);
 		FD_SET(fd, &wr0);
-		free(msg);
 		break; }
 
 	  default:
-		error(fd, "%s", usage);
+		error(fd, "%.2048s", usage);
 	}
 }
 
@@ -592,25 +785,29 @@ static void notify_client(
 static BOOL check_file(
 	FILE		*fp,		/* open file to check */
 	char		*fname,		/* file name for error messages */
-	int		fd)		/* client that requests check */
+	int		fd,		/* client fd */
+	struct client   *c)             /* client that requests check */
 {
 	struct stat	sbuf;		/* check for directories */
 	char		*msg = 0;	/* file error message */
 
-	if (fstat(fileno(fp), &sbuf) < 0)
-		msg = "cannot stat %s";
+	if (strstr("..", fname))
+		msg = "\"..\" in %.20s not allowed%s";
+	else if (fstat(fileno(fp), &sbuf) < 0)
+		msg = "cannot stat %.20s%s";
 	else if ((sbuf.st_mode & S_IFMT) == S_IFDIR)
-		msg = "%s is a directory";
+		msg = "%.20s is a directory%s";
 	else if ((sbuf.st_mode & S_IFMT) == S_IFLNK)
-		msg = "%s is a symbolic link%s";
+		msg = "%.20s is a symbolic link%s";
 	else if (sbuf.st_mode & S_IFCHR)
-		msg = "%s is a device node%s";
+		msg = "%.20s is a device node%s";
 	else if (sbuf.st_nlink != 1)
-		msg = "%s has more than one link%s";
+		msg = "%.20s has more than one link%s";
 	if (msg) {
 		error(0,  msg, fname, " (this is a security risk)");
 		error(fd, msg, fname, "\\\n(this is a security risk).");
-		log("client %d fails to open %s\n", fd, fname);
+		log("client %d (%.200s) fails to open %.20s\n",
+							fd, c->descr, fname);
 		return(FALSE);
 	}
 	return(TRUE);
@@ -630,14 +827,16 @@ static BOOL check_file(
 
 static int open_file(
 	char		*fname,		/* name of file to open */
-	int		fd)		/* client that requests open */
+	int		fd,		/* client that requests open */
+	BOOL            *ronly)         /* readonly flag */
 {
 	struct client	*c = &client_list[fd];
 	char		*path;		/* full path with directory part */
 	struct pfile	*f;		/* file entry */
 	int		fid, ffree;	/* file ID (file_list index), free */
 	FILE		*fp;		/* file to write */
-	BOOL		ro = FALSE;	/* read-only flag */
+	BOOL		ro = FALSE;	/* client's read-only flag */
+	BOOL            readonly = FALSE;/* file's read-only flag */
 	char		*buf = 0;	/* row buffer */
 	int		size = 0;	/* row buffer size */
 	int		i = 0;		/* row buffer index */
@@ -647,48 +846,58 @@ static int open_file(
 	int		r, w, d;	/* permissions from access list */
 
 	if (!*fname) {
-		error(0, "client %d opens null file name", fd);
+		error(0, "client %d (%.200s) opens null file name",
+								fd, c->descr);
 		error(fd, "cannot open null file name");
 		return(-1);
 	}
 	path = strrchr(fname, '/');			/* no dot files */
-	path = path ? path+1 : fname;
-
-	while (fname[0] == '.' && fname[1] == '/')	/* access list ok? */
-		fname += 2;
+	if (path)
+		fname = path + 1;
 	if (allow_all)
 		r = w = d = TRUE;
 	else
-		acl_verify(&r, &w, &d, fname, c->uid, c->gid,
-			   (unsigned int)c->addr.sin_addr.s_addr);
-	if (!r || *path == '.') {
-		error(0, "access to %s refused to client %d", fname, fd);
-		error(fd, "Access to file %s denied by server.", fname);
+		acl_verify(&r, &w, &d, fname, c);
+	ro = !w;
+	if (!r || *fname == '.') {
+		error(0, "access to %.20s refused to client %d (%.200s)",
+							fname, fd, c->descr);
+		error(fd, "Access to file %.20s denied by server.", fname);
 		return(-1);
 	}
 							/* already open? */
 	type = c->type;
-	for (f=file_list,fid=0,ffree=-1; fid < max_files; fid++,f++) {
+	for (f=file_list, fid=0, ffree=-1; fid < max_files; fid++, f++) {
 		if (!f->name)
 			ffree = fid;
 		else if (!strcmp(f->name, fname) && f->type == type) {
-			f->client[fd] = 1;
-			f->nclients++;
-			log("client %d opens cached file %s\n", fd, fname);
+			if (f->readonly)
+				ro = TRUE;
+			if (!f->client[fd])
+				f->nclients++;
+			f->client[fd] = ro ? 1 : 2;
+			log("client %d (%.200s) opens cached file %.200s "
+			    "%.20s\n", fd, c->descr, fname,
+			    (ro ? "readonly" : "readwrite"));
+			*ronly = ro;
 			return(fid);
 		}
 	}
-	path = allocate(strlen(HOMEDIR) + 1 +
-			strlen(NETDIR) + 1 + strlen(fname) + 1);
-	sprintf(path, "%s/%s/%s", HOMEDIR, NETDIR, fname);
+	path = allocate(strlen(HOMEDIR "/" NETDIR "/") + strlen(fname) + 1);
+	sprintf(path, HOMEDIR "/" NETDIR "/%s", fname);
 	if (fp = fopen(path, "r")) {			/* open and check */
-		if (!check_file(fp, path, fd)) {
+		if (!check_file(fp, path, fd, c)) {
 			fclose(fp);
 			return(-1);
 		}
-		ro = !w || !!access(path, W_OK);
+		if (access(path, W_OK))
+			ro = readonly = TRUE;
 	}
-	free(path);
+	if (ro && ! fp) {
+		error(fd, "File %.20s does not exist on server.", fname);
+		return -1;
+	}
+	release(path);
 	if (ffree != -1)				/* new file slot */
 		fid = ffree;
 	else {
@@ -700,7 +909,7 @@ static int open_file(
 	}
 	f = &file_list[fid];
 	memset(f, 0, sizeof(struct pfile));
-	f->readonly   = ro;
+	f->readonly   = readonly;
 	f->type       = type;
 	f->name       = allocate(strlen(fname)+1);
 	f->client     = allocate(nclients);
@@ -712,7 +921,7 @@ static int open_file(
 			char c = fgetc(fp);
 			if (feof(fp)) {
 				if (i)
-					add_row(fid, 0, fd, buf, i);
+					add_row(fid, 0, -1, buf, i);
 				break;
 			}
 			switch(type) {
@@ -720,7 +929,7 @@ static int open_file(
 				if (newline && c >= '0' && c <= '9') {
 					skip = FALSE;
 					if (i) {
-						add_row(fid, 0, fd, buf, i);
+						add_row(fid, 0, -1, buf, i);
 						i = 0;
 					}
 				}
@@ -729,7 +938,7 @@ static int open_file(
 			  case 1:			/* ...grok */
 				skip = FALSE;
 				if (newline && i) {
-					add_row(fid, 0, fd, buf, i);
+					add_row(fid, 0, -1, buf, i);
 					i = 0;
 				}
 				break;
@@ -741,17 +950,18 @@ static int open_file(
 				buf = reallocate(buf, size += BUFINC);
 			buf[i++] = c;
 		}
-		log("client %d opens new file %s (type %d), read %d entries\n",
-						fd, fname, f->type, f->nrows);
+		log("client %d (%.200s) opens new file %.20s (type %d), read %"
+			"d entries\n", fd, c->descr, fname, f->type, f->nrows);
 		fclose(fp);
 	} else
-		log("client %d opens non-existing file %s\n", fd, fname);
-	if (buf)
-		free(buf);
-	f->client[fd] = 1;	/* do this here to avoid notification msgs */
+		log("client %d (%.200s) opens non-existing file %.20s\n",
+			fd, c->descr, fname);
+	release(buf);
+	f->client[fd] = ro ? 1 : 2;	/* do this here to avoid notify msgs */
 	f->nclients   = 1;
 	for (i=0; i < f->nrows; i++)
 		f->row[i].modified = FALSE;
+	*ronly = ro;
 	return(fid);
 }
 
@@ -763,24 +973,25 @@ static int open_file(
 
 static void close_file(
 	int		fid,		/* file to close */
-	int		fd)		/* client that closes the file */
+	int		fd,		/* client's fd */
+	struct client  *c)              /* client that closes the file */
 {
 	int		r;		/* row counter for releasing */
 	struct pfile	*f = &file_list[fid];
 
 	if (fid < 0 || fid >= max_files || !f->name || !f->client[fd])
 		return;
-	log("client %d closes file %s\n", fd, f->name);
+	log("client %d (%.200s) closes file %.20s\n", fd, c->descr, f->name);
 	write_file(fid);			/* write changes */
 	f->client[fd] = 0;			/* detach client from file */
 	f->nclients--;
 	if (!f->nclients && !f->error) {	/* last client gone: uncache */
+		log("uncaching file %.20s\n", f->name);
 		for (r=0; r < f->nrows; r++)
-			if (f->row[r].data)
-				free(f->row[r].data);
-		free(f->row);
-		free(f->name);
-		free(f->client);
+			release(f->row[r].data);
+		release(f->row);
+		release(f->name);
+		release(f->client);
 		memset(f, 0, sizeof(struct pfile));
 	} else					/* else release stale locks */
 		for (r=0; r < f->nrows; r++)
@@ -792,71 +1003,82 @@ static void close_file(
 /*
  * write file <fid> to disk if it has been modified. This happens when the
  * last client closes the file, when the daemon exits, and periodically
- * from a timer interrupt.
+ * from a timer interrupt. This is split into two functions to avoid goto.
  * WARNING - still two security race conditions here!! Use creat/rename. <<<
  */
 
+static void do_write(struct pfile *, char *, char *);
+
 static void write_file(
 	int		fid)		/* file to close */
 {
 	struct pfile	*f = &file_list[fid];
 	char		*npath, *path;	/* full path with directory part */
+	int		n;
+
+	if (fid < 0 || fid >= max_files || !f->name || !f->modified)
+		return;
+	n = strlen(HOMEDIR "/" NETDIR "/") + strlen(f->name) + 1;
+	path = allocate(n);
+	sprintf(path, HOMEDIR "/" NETDIR "/%s", f->name);
+	npath = allocate(n + 4);
+	sprintf(npath, "%s.new", path);
+
+	do_write(f, path, npath);
+
+	release(path);
+	release(npath);
+}
+
+static void do_write(			/* called by previous function only */
+	struct pfile	*f,		/* file to write */
+	char		*path,		/* full path with directory part */
+	char		*npath)		/* temporary file */
+{
 	FILE		*fp;		/* file to write */
 	int		fd;		/* for error reporting */
 	int		r;		/* current row index */
 	struct row	*row;		/* current row */
 	BOOL		err = FALSE;	/* detect errors during writing */
-	int		n;
+	char            buf[256];
 
-	if (fid < 0 || fid >= max_files || !f->name || !f->modified)
-		return;
-	n = strlen(HOMEDIR) + 1 + strlen(NETDIR) + 1 + strlen(f->name) + 1;
-	path = allocate(n);
-	sprintf(path, "%s/%s/%s", HOMEDIR, NETDIR, f->name);
-	npath = allocate(n + 4);
-	sprintf(npath, "%s.new", path);
 	unlink(npath);
 	if (!(fp = fopen(npath, "w+"))) {
-		f->error = TRUE;
-		for (fd=0; fd < nclients; fd++)
-			if (!fd || f->client[fd] == 2)
-				error(fd,
-				     "cannot create temporary data file\\\n%s",
+		sprintf(buf, "cannot create temporary data file\\\n%.20s",
 									npath);
 		return;
 	}
-#ifdef sco
 	if (chmod(npath, 0666))
-#else
-	if (fchmod(fileno(fp), 0666))
-#endif
-		error(0, "WARNING - cannot chmod 666 %s", npath);
-	errno = 0;
+		error(0, "WARNING - cannot chmod 666 %.200s", npath);
 	for (r=0, row=f->row; r < f->nrows; r++, row++)
-		if (row->data) {
-			fputs(row->data, fp);
-			err |= errno;
-		}
-	fclose(fp);
-	if (err || errno) {
-		f->error = TRUE;
-		for (fd=0; fd < nclients; fd++)
-			if (!fd || f->client[fd] == 2)
-				error(fd,
-				      "cannot write temporary data file\\\n%s",
+		if (row->data)
+			if (fputs(row->data, fp) == EOF) {
+				err = TRUE;
+				break;
+			}
+	if (fclose(fp))
+		err = TRUE;
+	if (err) {
+		sprintf(buf, "cannot write temporary data file\\\n%.20s",
 									npath);
 		return;
 	}
 	unlink(path);
-	link(npath, path);
+	if (link(npath, path))
+		err = TRUE;
 	unlink(npath);
+	if (err) {
+		sprintf(buf, "cannot rename data file %.20s", path);
+		return;
+	}
 	for (r=0, row=f->row; r < f->nrows; r++, row++)
 		row->modified = FALSE;
+	for (fd=0; fd < nclients; fd++)
+		if (f->client[fd] == 3)
+			f->client[fd] = 2;
 	f->modified = FALSE;
 	f->error = FALSE;
-	log("written data file %s\n", path);
-	free(path);
-	free(npath);
+	log("written data file %.400s\n", path);
 }
 
 
@@ -921,18 +1143,23 @@ static unsigned int add_row(
 
 	if (fid < 0 || fid >= max_files || !f->name)
 		return(0);
-	if (!rid) {					/* new: make ID */
-		rid = row_id++;
-		row_id += !row_id;
-	}
-	index = find_row(fid, rid, &found);		/* insert-sort */
+	
+	if (!rid)					/* new: make ID */
+		do {
+			rid = row_id++;
+			row_id += !row_id;
+			index = find_row(fid, rid, &found); /* insert-sort */
+		} while (found);
+	else
+		index = find_row(fid, rid, &found);	/* insert-sort */
+	
 	if (!found) {					/* make new slot */
 		if (++f->nrows > f->maxrows) {		/* ...extend list? */
 			f->maxrows += ROWINC;
 			f->row = reallocate(f->row,
 					    f->maxrows * sizeof(struct row));
 		}
-		for (i=f->nrows; i > index; i--)
+		for (i=f->nrows-1; i > index; i--)
 			f->row[i] = f->row[i-1];
 		memset(&f->row[index], 0, sizeof(struct row));
 	}
@@ -940,9 +1167,12 @@ static unsigned int add_row(
 	row->data = reallocate(row->data, len+1);
 	memcpy(row->data, data, len);
 	row->data[len] = 0;
-	row->modified = f->modified = TRUE;
 	row->id = rid;
-	notify_client(fid, rid, fd);
+	if (fd >= 0) {			/* real insert (not at open time) */
+		row->modified = f->modified = TRUE;
+		f->client[fd] = 3;
+		notify_client(fid, rid, fd);
+	}
 	if (!update_time)
 		update_time = time(0);
 	return(rid);
@@ -962,8 +1192,10 @@ static void delete_row(
 	BOOL			found;		/* replace existing row? */
 	struct pfile		*f = &file_list[fid];
 
-	if (!rid || fid < 0 || fid >= max_files || !f->name)
+	if (!rid || fid < 0 || fid >= max_files || !f->name ||
+	    f->client[fd] < 2) {
 		return;
+	}
 	index = find_row(fid, rid, &found);		/* find rid */
 	if (!found)
 		return;
@@ -971,6 +1203,7 @@ static void delete_row(
 	while (index++ < f->nrows)			/* delete */
 		f->row[index-1] = f->row[index];
 	f->modified = TRUE;
+	f->client[fd] = 3;
 	notify_client(fid, rid, fd);
 	if (!update_time)
 		update_time = time(0);
@@ -992,7 +1225,9 @@ static BOOL lock_row(
 	BOOL			found;		/* replace existing row? */
 	struct pfile		*f = &file_list[fid];
 
-	if (!rid || fid < 0 || fid >= max_files || !f->name)
+	if (fid < 0 || fid >= max_files || !f->name || f->client[fd] < 2)
+		return(FALSE);
+	if (!rid)
 		return(TRUE);
 	index = find_row(fid, rid, &found);		/* find rid */
 	if (!found)
@@ -1030,13 +1265,13 @@ static void send_row(
 	register char	*p, *q;		/* for escaping \n with \ */
 
 	if (fid < 0 || fid >= max_files || !f->name || !f->client[fd]) {
-		reply(fd, "%cf%d 0 ", opcode, fid);
+		reply(fd, c, "%cf%d 0 ", opcode, fid);
 		return;
 	}
 	if (rid) {					/* single row */
 		beg = end = find_row(fid, rid, &found);
 		if (!found) {
-			reply(fd, "%cf%d %d ", opcode, fid, rid);
+			reply(fd, c, "%cf%d %d ", opcode, fid, rid);
 			return;
 		}
 	} else {					/* all rows */
@@ -1048,7 +1283,8 @@ static void send_row(
 		sprintf(buf, "%ct%d %d ", opcode, fid, row->id);
 		appendbuf(&c->out, buf, strlen(buf));
 		if (verbose)
-			log("client %d reply %s<data>\n", fd, buf);
+			log("client %d (%.200s) reply %.50s<data>\n",
+							fd, c->descr, buf);
 		for (p=row->data; *p; ) {
 			for (q=buf; *p && q < buf+sizeof(buf)-1; ) {
 				if (*p == '\n') {
@@ -1095,7 +1331,7 @@ static void fatal(char *fmt, ...)
 	fprintf(stderr, "%s: ", progname);
 	vfprintf(stderr, fmt, parm);
 	va_end(parm);
-	fprintf(stderr, ", exiting.\n", stderr);
+	fprintf(stderr, ", exiting.\n");
 	exit(1);
 }
 
@@ -1107,17 +1343,17 @@ static void fatal(char *fmt, ...)
  * is too small!
  */
 
-static void reply(int fd, char *fmt, ...)
+static void reply(int fd, struct client *c, char *fmt, ...)
 {
 	va_list		parm;
-	char		msg[1024];
+	char		msg[4096];
 
 	va_start(parm, fmt);
 	vsprintf(msg, fmt, parm);
 	strcat(msg, "\n");
 	appendbuf(&client_list[fd].out, msg, strlen(msg));
 	if (verbose)
-		log("client %d reply %s", fd, msg);
+		log("client %d (%.200s) reply %.400s", fd, c->descr, msg);
 	FD_SET(fd, &wr0);
 }
 
@@ -1128,16 +1364,16 @@ static void reply(int fd, char *fmt, ...)
  * to stderr. \r is used for newlines because \n is a message terminator.
  */
 
-static void error(int fd, char *fmt, ...)
+void error(int fd, char *fmt, ...)
 {
 	va_list		parm;
-	char		msg[1024];
+	char		msg[4096];
 
 	va_start(parm, fmt);
 	if (fd) {
-		strcpy(msg, "?Error on data server on host <");
+		strcpy(msg, "?Error on netplan server ");
 		gethostname(msg+strlen(msg), 256);
-		strcat(msg+strlen(msg), ">:\\\n");
+		strcat(msg+strlen(msg), ":\\\n");
 		vsprintf(msg+strlen(msg), fmt, parm);
 		strcat(msg, "\n");
 		appendbuf(&client_list[fd].out, msg, strlen(msg));
@@ -1156,7 +1392,7 @@ static void error(int fd, char *fmt, ...)
  * print verbose logging message
  */
 
-static void log(char *fmt, ...)
+void log(char *fmt, ...)
 {
 	va_list		parm;
 
@@ -1173,14 +1409,15 @@ static void log(char *fmt, ...)
  * return static string descriping an IP address with port number
  */
 
-static char *ip_addr(register struct sockaddr_in *addr)
+char *ip_addr(register struct sockaddr_in *addr)
 {
 	static char buf[64];
+	unsigned long a = ntohl(addr->sin_addr.s_addr);
 	sprintf(buf, "%d.%d.%d.%d:%d",
-		(addr->sin_addr.s_addr >> 24) & 255,
-		(addr->sin_addr.s_addr >> 16) & 255,
-		(addr->sin_addr.s_addr >>  8) & 255,
-		(addr->sin_addr.s_addr      ) & 255,
-		addr->sin_port);
+		(a >> 24) & 255,
+		(a >> 16) & 255,
+		(a >>  8) & 255,
+		(a      ) & 255,
+		ntohs(addr->sin_port));
 	return(buf);
 }
