@@ -87,6 +87,7 @@ static void lost_connection_error(fd, error)
 {
 	struct host	*hp, *prev = 0;
 	char		*host = 0;
+	int             u;
 
 	for (hp=host_list; hp; prev=hp, hp=hp->next)
 		if (fd == hp->fd) {
@@ -96,6 +97,9 @@ static void lost_connection_error(fd, error)
 			else
 				host_list = hp->next;
 			unregister_X_input(hp->x_id);
+			for (u = 0; u < nusers; u++)
+				if (user[u].host == hp)
+					user[u].host = 0;
 			close(hp->fd);
 			free(hp);
 			break;
@@ -104,9 +108,8 @@ static void lost_connection_error(fd, error)
 		create_error_popup(mainwindow, error,
 "Lost connection to netplan server %s.\n\n\
 Please start it up again, then choose\n\
-`File->File list' and `Done' to reconnect, or\n\
 `File->Reread files' to reconnect and re-read.\n\
-Either way, you may have lost recent changes.", host ? host : "<unknown>");
+You may have lost recent changes.", host ? host : "<unknown>");
 	if (host)
 		free(host);
 }
@@ -127,7 +130,7 @@ char *attach_to_network()
 	struct hostent	*hent;		/* for hostname -> IP addr lookup */
 	struct sockaddr_in addr;	/* IP address */
 	int		fd;		/* temporary socket descriptor */
-	int		on = TRUE;	/* for setting socket options */
+	int		on;		/* for setting socket options */
 	char		msg[10240];	/* problem report */
 	char		buf[1024];	/* banner reply from server */
 
@@ -175,7 +178,9 @@ char *attach_to_network()
 				user[u].name, uhost);
 			continue;
 		}
+		on = TRUE;
 		setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE, &on, sizeof(on));
+		on = TRUE;
 		setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, &on, sizeof(on));
 
 		if (gets_server(fd, buf, sizeof(buf), TRUE, FALSE) < 0)
@@ -287,29 +292,42 @@ BOOL server_entry_op(entry, op)
 	int		u;		/* number of user that owns entry */
 	struct host	*host;		/* for checking already-known hosts */
 	char		*name;		/* file name on server */
+	char            *hostname;      /* server's host name */
 
 	u = name_to_user(entry->user);
-	if (!entry->id && op != 'w' || !user[u].fromserver)
+	if (!entry->id && op != 'w' || !user[u].fromserver ||
+	    entry->private && (op == 'l' || op == 'w'))
 		return(TRUE);
 	host = user[u].host;
+	hostname = user[u].server;
+	if (!user[u].host)
+		return(FALSE);
 	name = user[u].name;
 	sprintf(buf, "%c%d %d%c", op, entry->file, entry->id,
 				  op == 'w' ? ' ' : '\n');
 	puts_server(host->fd, buf);
+	if (!user[u].host)
+		return(FALSE);
 	switch(op) {
 	  case 'l':
 		puts_server(host->fd, 0);
+		if (!user[u].host)
+			return(FALSE);
 		ret = gets_server(host->fd, buf, sizeof(buf), TRUE, FALSE);
-		if (!check_server_reply(ret, buf, op, host->name, name))
+		if (!check_server_reply(ret, buf, op, hostname, name))
 			return(FALSE);
 		return(buf[1] == 't');
 
 	  case 'w':
 		host_fd = host->fd;
 		ret = write_one_entry(write_to_net, entry);
+		if (!user[u].host)
+			return(FALSE);
 		puts_server(host->fd, 0);
+		if (!user[u].host)
+			return(FALSE);
 		r = gets_server(host->fd, buf, sizeof(buf), TRUE, FALSE);
-		if (!check_server_reply(r, buf, op, host->name, name))
+		if (!check_server_reply(r, buf, op, hostname, name))
 			return(FALSE);
 		if (buf[1] == 't')
 			entry->id = atoi(buf+2);
@@ -348,7 +366,7 @@ int open_and_count_rows_on_server(uid)
 		puts_server(host->fd, buf);
 		puts_server(host->fd, 0);
 		ret = gets_server(host->fd, buf, sizeof(buf), TRUE, FALSE);
-		if (!check_server_reply(ret, buf, 'o', host->name, uname))
+		if (!check_server_reply(ret, buf, 'o', user[uid].server,uname))
 			return(-1);
 		if (buf[1] == 'f')			/* file not found */
 			return(-1);
@@ -527,10 +545,10 @@ BOOL gets_server(fd, msg, num, wait, cont)
 			i = strlen(msg);
 			while (!ret && msg[i-1] == '\n') {
 				ret = gets_server_line(fd, msg+i, num-i, wait);
-				i += strlen(msg);
+				i += strlen(msg+i);
 			}
 			create_error_popup(mainwindow, 0,
-				"Server host %s reports a problem:\n%s",
+				"Server host %s reports a problem:\n\n%s",
 				host, msg+1);
 			break;
 		  case 'R':
