@@ -35,9 +35,11 @@
 
 #ifdef JAPAN
 #define PS_LIB_SJ  "plan_calSJ.ps"
-
 extern char		*localename;
 #endif
+#ifdef BSD
+extern FILE *popen();
+#endif
 
 int			Print_Calendar();
 static int		Print_Day();
@@ -483,7 +485,8 @@ int Print_Calendar(Fp, Mode, Month, Day, Year)
 #endif
 	find_file(PsLib, PS_LIB, FALSE);
 	if ((Pslib_Fd = open(PsLib, O_RDONLY)) == -1) {
-		create_error_popup(print_shell, errno, "Can't open %s", PsLib);
+		create_error_popup(print_shell, errno,
+			"Can't find PostScript skeleton\nfile \"%s\"", PS_LIB);
 		pclose(Fp);
 		return (-1);
 	}
