@@ -26,14 +26,17 @@
 #include <Xm/PushBP.h>
 #include <Xm/PushBG.h>
 #include <Xm/ToggleB.h>
+#include <Xm/RowColumn.h>
 #include <Xm/ScrolledW.h>
 #include <Xm/Text.h>
 #include <Xm/Protocols.h>
 #include "cal.h"
 
 enum Columns { C_EN_MONTH=0, C_EN_WEEK, C_EN_DAY, C_EN_YROV, C_EN_ALARM,
-		C_GROUP, C_NAME, C_EN_SRV, C_PATH, C_SERVER, NCOLUMNS };
+		C_GROUP, C_NAME, C_EN_SRV, C_PATH, C_SERVER, NCOLUMNS,
+		COLPOPUP };
 
+static Widget call_popup_callback();
 static void create_user_rows(), edit_user_button(), got_user_text(),
 	    draw_row(), delete_callback(), sort_callback(),
 	    cancel_callback(), done_callback(), list_callback(), got_text();
@@ -42,9 +45,13 @@ extern struct passwd *getpwnam();
 #endif
 
 extern Display		*display;	/* everybody uses the same server */
+extern Widget		toplevel;	/* top-level shell for icon name */
 extern GC		gc;		/* everybody uses this context */
+extern GC		chk_gc;		/* checkered gc for light background */
 extern Pixel		color[NCOLS];	/* colors: COL_* */
 extern Pixmap		pixmap[NPICS];	/* common symbols */
+extern Pixmap		col_pixmap[8];	/* colored circles for file list */
+extern Pixmap		chk_pixmap[8];	/* same but checkered (light color) */
 extern Widget		mainwindow;	/* popup menus hang off main window */
 extern struct config	config;		/* global configuration data */
 extern struct plist	*mainlist;	/* list of all schedule entries */
@@ -53,7 +60,7 @@ extern struct user	*user;		/* user list (from file_r.c) */
 extern int		nusers;		/* number of users in user list */
 
 static struct user	*saved_user;	/* saved user[] while editing */
-static int		*saved_nusers;	/* saved nusers while editing */
+static int		saved_nusers;	/* saved nusers while editing */
 static BOOL		have_shell;	/* message popup exists if TRUE */
 static Widget		shell;		/* popup menu shell */
 static Widget		delete;		/* delete button, for desensitizing */
@@ -63,9 +70,10 @@ static int		have_nrows;	/* # of table widget rows allocated */
 static int		xedit, yedit;	/* if editing, column/row */
 static int		ycurr;		/* current row, 0=none, 1=1st user...*/
 static Widget		ulist;		/* user list RowColumn widget */
-static Widget	(*utable)[NCOLUMNS+1];	/* all widgets in user list table */
+static Widget	(*utable)[NCOLUMNS+2];	/* all widgets in user list table */
 					/* [0][*] is title row, */
 					/* [*][NCOLUMNS] is form widget */
+					/* [*][COLPOPUP] is color popup */
 
 /*
  * return TRUE id all files are read from servers. In this case, buttons like
@@ -188,7 +196,9 @@ static BOOL accept_new_list()
 				    !strcmp(ep->user, up->name)) {
 					ep->id   = 0;
 					ep->file = up->file_id;
-					server_entry_op(ep, 'w');
+					if (!server_entry_op(ep, 'w'))
+						create_error_popup(0, 0,
+								"write error");
 				}
 			create_error_popup(mainwindow, 0,
 				"File %s on server %s did not\nexist. "
@@ -276,8 +286,8 @@ void create_user_popup()
 	n = 0;
 	XtSetArg(args[n], XmNdeleteResponse,	XmUNMAP);		n++;
 	XtSetArg(args[n], XmNiconic,		False);			n++;
-	shell = XtAppCreateShell("File List", "plan",
-			applicationShellWidgetClass, display, args, n);
+	shell = XtCreatePopupShell("File List",
+			applicationShellWidgetClass, toplevel, args, n);
 #	ifdef EDITRES
 	XtAddEventHandler(shell, (EventMask)0, TRUE, 
  			_XEditResCheckMessages, NULL);
@@ -300,7 +310,6 @@ void create_user_popup()
 	XtAddCallback(w, XmNactivateCallback, delete_callback, (XtPointer)0);
 	XtAddCallback(w, XmNhelpCallback,     help_callback,   (XtPointer)
 								"user_delete");
-
 	n = 0;
 	XtSetArg(args[n], XmNbottomAttachment,	XmATTACH_FORM);		n++;
 	XtSetArg(args[n], XmNbottomOffset,	8);			n++;
@@ -324,7 +333,6 @@ void create_user_popup()
 	XtAddCallback(w, XmNactivateCallback, done_callback, (XtPointer)0);
 	XtAddCallback(w, XmNhelpCallback,     help_callback, (XtPointer)
 								"user_done");
-
 	n = 0;
 	XtSetArg(args[n], XmNbottomAttachment,	XmATTACH_FORM);		n++;
 	XtSetArg(args[n], XmNbottomOffset,	8);			n++;
@@ -375,8 +383,8 @@ void create_user_popup()
 	XtSetArg(args[n], XmNleftOffset,	8);			n++;
 	XtSetArg(args[n], XmNrightAttachment,	XmATTACH_FORM);		n++;
 	XtSetArg(args[n], XmNrightOffset,	8);			n++;
-	XtSetArg(args[n], XmNwidth,		810);			n++;
-	XtSetArg(args[n], XmNheight,		220);			n++;
+	XtSetArg(args[n], XmNwidth,		785);			n++;
+	XtSetArg(args[n], XmNheight,		250);			n++;
 	XtSetArg(args[n], XmNscrollingPolicy,	XmAUTOMATIC);		n++;
 	scroll = XtCreateWidget("uscroll", xmScrolledWindowWidgetClass,
 			form, args, n);
@@ -425,6 +433,7 @@ static char *cell_help [NCOLUMNS] = { "user_enable", "user_enable",
 
 static void create_user_rows()
 {
+	Widget			w, *urow;
 	int			nrows = nusers+5 - nusers%5;
 	int			x, y, xo;
 	Arg			args[20];
@@ -438,27 +447,28 @@ static void create_user_rows()
 	if (!cell_x[0])
 		for (x=4, n=0; n < NCOLUMNS; x+=cell_xs[n++]) {
 			cell_x[n]  = x;
-			cell_xs[n] = n == C_NAME || n == C_SERVER ? 120 :
-				     n == C_PATH		  ? 220 : 40;
+			cell_xs[n] = n == C_NAME || n == C_SERVER ? 100 :
+				     n == C_PATH		  ? 220 : 45;
 		}
 	if (!have_shell)				/* check # of rows: */
 		have_nrows = 0;
 	if (nrows <= have_nrows)
 		return;
 
-	n = (nrows+1) * (NCOLUMNS+1) * sizeof(Widget *);
+	n = (nrows+1) * (NCOLUMNS+2) * sizeof(Widget *);
 	if (utable && !(utable = (Widget (*)[])realloc(utable, n)) ||
 	   !utable && !(utable = (Widget (*)[])malloc(n)))
 		fatal("no memory");
-
+							/* build list */
 	XtUnmanageChild(ulist);
 	for (y=have_nrows; y <= nrows; y++) {
+	    urow = utable[y];
 	    n = 0;
 	    XtSetArg(args[n], XmNtopAttachment,		XmATTACH_FORM);   n++;
 	    XtSetArg(args[n], XmNheight,		30);		  n++;
 	    XtSetArg(args[n], XmNy,			10 + 30 * y);	  n++;
 	    XtSetArg(args[n], XmNleftAttachment,	XmATTACH_FORM);   n++;
-	    utable[y][NCOLUMNS] = XtCreateWidget("userrowform",
+	    urow[NCOLUMNS] = XtCreateWidget("userrowform",
 	   				 xmFormWidgetClass, ulist, args, n);
 	    for (x=0; x < NCOLUMNS; x++) {
 		xo = n = 0;
@@ -483,6 +493,9 @@ static void create_user_rows()
 				class  = xmPushButtonWidgetClass;
 				callbk = XmNactivateCallback;
 				align  = XmALIGNMENT_CENTER;
+				if (y > 1)
+					XtSetArg(args[n], XmNlabelType,
+							XmPIXMAP); n++;
 				break;
 			  case C_NAME:
 			  case C_PATH:
@@ -503,24 +516,48 @@ static void create_user_rows()
 		XtSetArg(args[n], XmNwidth,		cell_xs[x]-xo);	n++;
 		XtSetArg(args[n], XmNtopAttachment,	XmATTACH_FORM); n++;
 		XtSetArg(args[n], XmNbottomAttachment,	XmATTACH_FORM);	n++;
-		/* needed by LessTif, doesn't work with Metrolink Linux Motif
+#if LESSTIF_VERSION != 0
 		XtSetArg(args[n], XmNleftAttachment,	XmATTACH_SELF); n++;
 		XtSetArg(args[n], XmNrightAttachment,	XmATTACH_SELF); n++;
-		*/
+#endif
 		XtSetArg(args[n], XmNalignment,         align);		n++;
 		XtSetArg(args[n], XmNrecomputeSize,	False);		n++;
 		XtSetArg(args[n], XmNtraversalOn,	True);		n++;
 		XtSetArg(args[n], XmNhighlightThickness,0);		n++;
 		XtSetArg(args[n], XmNshadowThickness,	shadow);	n++;
-		utable[y][x] = XtCreateManagedWidget(name, class,
-				utable[y][NCOLUMNS], args, n);
+		urow[x] = XtCreateManagedWidget(name, class,
+						urow[NCOLUMNS], args, n);
+#ifdef XmCSimpleOptionMenu
+		if (y && x == C_GROUP) {
+			XtSetArg(args[0], XmNbackground, color[COL_CALBACK]);
+			urow[COLPOPUP] = XmCreatePopupMenu(urow[x],
+							"colorpopup", args, 1);
+			for (n=0; n < 16; n++) {
+				XtSetArg(args[0], XmNlabelType, XmPIXMAP);
+				XtSetArg(args[1], XmNbackground,
+					color[COL_CALBACK]);
+				XtSetArg(args[2], XmNlabelPixmap,
+					(n>7 ? chk_pixmap : col_pixmap)[n&7]);
+				w = XtCreateManagedWidget("pcolor",
+					xmPushButtonWidgetClass,
+					urow[COLPOPUP], args, 3);
+				XtAddCallback(w, XmNactivateCallback,
+					(XtCallbackProc)list_callback,
+					(XtPointer)(C_GROUP + y * NCOLUMNS
+							    + (n+1) * 65536));
+			}
+			XtAddEventHandler(urow[x], ButtonPressMask, False,
+					(XtEventHandler)call_popup_callback,
+					(XtPointer)y);
+		} else
+#endif
 		if (y)
-			XtAddCallback(utable[y][x], callbk,
-				list_callback, (XtPointer)(x + y * NCOLUMNS));
-		XtAddCallback(utable[y][x], XmNhelpCallback, help_callback,
+			XtAddCallback(urow[x], callbk, list_callback,
+			   			(XtPointer)(x + y * NCOLUMNS));
+		XtAddCallback(urow[x], XmNhelpCallback, help_callback,
 						(XtPointer)cell_help[x]);
 	    }
-	    XtManageChild(utable[y][NCOLUMNS]);
+	    XtManageChild(urow[NCOLUMNS]);
 	}
 	for (y=have_nrows; y <= nrows; y++)
 		draw_row(y);
@@ -726,56 +763,59 @@ static void got_user_text(x, y, string)
 static void draw_row(y)
 	int			y;
 {
-	Arg			arg;
+	Widget			*urow = utable[y];
+	Arg			args[3];
 	register struct user	*u = &user[y-1];
 	int			col;
 
 	if (y < 1)
 		return;
 	if (y <= nusers) {					/* valid row */
-		XtSetArg(arg, XmNset, !u->suspend_m);
-		XtSetValues (utable[y][C_EN_MONTH], &arg, 1);
-		XtSetArg(arg, XmNset, !u->suspend_w);
-		XtSetValues (utable[y][C_EN_WEEK],  &arg, 1);
-		XtSetArg(arg, XmNset, !u->suspend_d);
-		XtSetValues (utable[y][C_EN_DAY],   &arg, 1);
-		XtSetArg(arg, XmNset, !u->suspend_o);
-		XtSetValues (utable[y][C_EN_YROV],  &arg, 1);
-		XtSetArg(arg, XmNset,  u->alarm);
-		XtSetValues (utable[y][C_EN_ALARM], &arg, 1);
-		XtSetArg(arg, XmNset,  u->fromserver);
-		XtSetValues (utable[y][C_EN_SRV],   &arg, 1);
-		XtSetArg(arg, XmNsensitive, !u->fromserver);
-		XtSetValues(utable[y][C_PATH],      &arg, 1);
-		XtSetArg(arg, XmNsensitive, u->fromserver);
-		XtSetValues(utable[y][C_SERVER],    &arg, 1);
+		XtSetArg(args[0], XmNset, !u->suspend_m);
+		XtSetValues (urow[C_EN_MONTH], args, 1);
+		XtSetArg(args[0], XmNset, !u->suspend_w);
+		XtSetValues (urow[C_EN_WEEK],  args, 1);
+		XtSetArg(args[0], XmNset, !u->suspend_d);
+		XtSetValues (urow[C_EN_DAY],   args, 1);
+		XtSetArg(args[0], XmNset, !u->suspend_o);
+		XtSetValues (urow[C_EN_YROV],  args, 1);
+		XtSetArg(args[0], XmNset,  u->alarm);
+		XtSetValues (urow[C_EN_ALARM], args, 1);
+		XtSetArg(args[0], XmNset,  u->fromserver);
+		XtSetValues (urow[C_EN_SRV],   args, 1);
+		XtSetArg(args[0], XmNsensitive, !u->fromserver);
+		XtSetValues(urow[C_PATH],      args, 1);
+		XtSetArg(args[0], XmNsensitive, u->fromserver);
+		XtSetValues(urow[C_SERVER],    args, 1);
 		if (y == 1) {
-			print_button(utable[y][C_GROUP], "own");
-			col = COL_BACK;
+			print_button(urow[C_GROUP], "own");
 		} else {
-			print_button(utable[y][C_GROUP], "%d", u->color);
-			col = COL_WUSER_0 + u->color;
+#ifdef XmCSimpleOptionMenu
+			XtSetArg(args[0], XmNbackground,  color[COL_CALBACK]);
+			XtSetArg(args[1], XmNlabelPixmap, (u->color > 7 ?
+					chk_pixmap : col_pixmap)[u->color&7]);
+			XtSetValues(urow[C_GROUP], args, 2);
+#endif
 		}
-		XtSetArg(arg, XmNbackground, color[col]);
-		XtSetValues (utable[y][C_GROUP],  &arg, 1);
-		print_button(utable[y][C_NAME],   "%s", u->name ? u->name:"?");
-		print_button(utable[y][C_PATH],   "%s", u->path ? u->path:"?");
-		print_button(utable[y][C_SERVER], "%s", u->server ?
+		print_button(urow[C_NAME],   "%s", u->name ? u->name:"?");
+		print_button(urow[C_PATH],   "%s", u->path ? u->path:"?");
+		print_button(urow[C_SERVER], "%s", u->server ?
 								u->server:"?");
 	} else {
-		XtSetArg(arg, XmNset, 0);			/* blank row */
-		XtSetValues (utable[y][C_EN_MONTH], &arg, 1);
-		XtSetValues (utable[y][C_EN_WEEK],  &arg, 1);
-		XtSetValues (utable[y][C_EN_DAY],   &arg, 1);
-		XtSetValues (utable[y][C_EN_YROV],  &arg, 1);
-		XtSetValues (utable[y][C_EN_ALARM], &arg, 1);
-		XtSetValues (utable[y][C_EN_SRV],   &arg, 1);
-		XtSetArg(arg, XmNbackground, color[COL_BACK]);
-		XtSetValues (utable[y][C_GROUP], &arg, 1);
-		print_button(utable[y][C_GROUP],  " ");
-		print_button(utable[y][C_NAME],   " ");
-		print_button(utable[y][C_PATH],   " ");
-		print_button(utable[y][C_SERVER], " ");
+		XtSetArg(args[0], XmNset, 0);			/* blank row */
+		XtSetValues (urow[C_EN_MONTH], args, 1);
+		XtSetValues (urow[C_EN_WEEK],  args, 1);
+		XtSetValues (urow[C_EN_DAY],   args, 1);
+		XtSetValues (urow[C_EN_YROV],  args, 1);
+		XtSetValues (urow[C_EN_ALARM], args, 1);
+		XtSetValues (urow[C_EN_SRV],   args, 1);
+		XtSetArg(args[0], XmNbackground,  color[COL_BACK]);
+		XtSetArg(args[1], XmNlabelPixmap, pixmap[PIC_BLANK]);
+		XtSetValues (urow[C_GROUP], args, 2);
+		print_button(urow[C_GROUP],  " ");
+		print_button(urow[C_NAME],   " ");
+		print_button(urow[C_PATH],   " ");
+		print_button(urow[C_SERVER], " ");
 	}
 }
 
@@ -869,9 +909,38 @@ static void done_callback(widget, item, data)
 }
 
 
+/*
+ * the color column was pressed. Install a color popup. I couldn't figure out
+ * how to post the menu with the left mouse button, and positioning it - the
+ * XmNmenuPost resource probably doesn't work as expected. Fix later. This is
+ * done only if Motif 1.2 or later is available. (Everybody is using 2.0 now.)
+ */
+
+#ifdef XmCSimpleOptionMenu
+static Widget call_popup_callback(widget, y, event, cont)
+	Widget		widget;
+	XtPointer	y;
+        XEvent		*event;
+	Boolean		*cont;
+{
+	Widget popup = utable[(long)y][COLPOPUP];
+	event->xbutton.x += 19;
+	event->xbutton.y += 30 * user[(long)y-1].color + 4;
+	XmMenuPosition(popup, (XButtonPressedEvent *)event);
+	XtManageChild(popup);
+	if (event->xbutton.button != 3)
+		print_button(info, "Use right mouse button to change color");
+	return(0);
+}
+#endif
+
+
+
 /*
  * one of the buttons in the list was pressed. Remember that the line 0 is
  * the title, and line 1 is reserved for the user's own appointment file.
+ * In the C_GROUP case, increment if the Motif version is older than 1.2,
+ * or accept the result from the color popup otherwise.
  */
 
 /*ARGSUSED*/
@@ -880,8 +949,9 @@ static void list_callback(widget, item, data)
 	int				item;
 	XmToggleButtonCallbackStruct	*data;
 {
-	int				x = item % NCOLUMNS;
-	int				y = item / NCOLUMNS;
+	int				x = (item & 65535) % NCOLUMNS;
+	int				y = (item & 65535) / NCOLUMNS;
+	struct user			*u = &user[y-1];
 	Arg				arg;
 
 	if (y > nusers) {					/* new entry */
@@ -895,45 +965,43 @@ static void list_callback(widget, item, data)
 		ycurr = y;
 		switch(x) {
 		  case C_EN_MONTH:
-			user[y-1].suspend_m = !data->set;
+			u->suspend_m = !data->set;
 			draw_calendar(NULL);
 			break;
 
 		  case C_EN_WEEK:
-			user[y-1].suspend_w = !data->set;
+			u->suspend_w = !data->set;
 			draw_week_calendar(NULL);
 			break;
 
 		  case C_EN_DAY:
-			user[y-1].suspend_d = !data->set;
+			u->suspend_d = !data->set;
 			draw_day_calendar(NULL);
 			break;
 
 		  case C_EN_YROV:
-			user[y-1].suspend_o = !data->set;
+			u->suspend_o = !data->set;
 			draw_yov_calendar(NULL);
 			break;
 
 		  case C_EN_ALARM:
-			user[y-1].alarm = data->set;
+			u->alarm = data->set;
 			break;
 
 		  case C_EN_SRV:
-			user[y-1].fromserver = data->set;
+			u->fromserver = data->set;
 			draw_row(y);
 			break;
 
 		  case C_GROUP:
-			if (y == 1) {
+			if (y > 1) {
+				u->color = item < 65536 ? (u->color + 1) & 15
+							: (item >> 16) - 1;
+				XtSetArg(arg, XmNlabelPixmap, (u->color > 7 ?
+					chk_pixmap : col_pixmap)[u->color&7]);
+				XtSetValues(utable[y][x], &arg, 1);
+			} else
 				print_button(info, "Cannot change own group");
-				return;
-			}
-			user[y-1].color++;
-			user[y-1].color &= 7;
-			XtSetArg(arg, XmNbackground,
-				color[COL_WUSER_0 + user[y-1].color]);
-			XtSetValues(widget, &arg, 1);
-			print_button(widget, "%d", user[y-1].color);
 			break;
 
 		  case C_PATH:
