@@ -2,7 +2,7 @@
  * draws the week view. PostScript printing is also here because it uses
  * the same high-level routines, only the low-level output is diverted.
  *
- *	clicked_week_calendar(x, y)	Press on the week display detected.
+ *	locate_in_week_calendar()	find out where we are in the calendar
  *	draw_week_day(day, month, year)	The day day/month/year has changed,
  *					redraw the week view if it contains
  *					that day.
@@ -49,91 +49,109 @@ extern char *monthname[];
 
 
 /*
- * got a click in the calendar area. Traverse the node tree, first down until
- * the correct line is found, and then horizontally until the node is found.
+ * for a given position on the canvas, identify the entry there and return
+ * a pointer to it and which area it was hit in (one of the M_ constants).
+ * Locate an event if epp!=0, and return exactly where the event was hit.
+ * If none was found, return the time if possible for creating new events.
+ * If epp is 0, we are dragging an event right now; just return a new time.
+ *
+ *   ret	*epp	*time	*trig	xp..ysp	event
+ *
+ *   M_ILLEGAL	0	0	-	-	no recognizable position
+ *   M_OUTSIDE	0	time	-	-	no event but in a valid day
+ *   M_INSIDE	entry	time	trigg	-	in event center, edit it
+ *   M_LEFT	entry	time	trigg	pos/sz	near left edge, move start date
+ *   M_RIGHT	entry	time	trigg	pos/sz	near right edge, move length
  */
 
-void clicked_week_calendar(xc, yc, time)
+MOUSE locate_in_week_calendar(epp, warn, time, trigger, xp,yp,xsp,ysp, xc,yc)
+	struct entry		**epp;		/* returned entry or 0 */
+	BOOL			*warn;		/* is *epp an n-days-ahead w?*/
+	time_t			*time;		/* if *epp=0, clicked where */
+	time_t			*trigger;	/* if *epp=0, entry trigger */
+	int			*xp,  *yp;	/* if M_INSIDE, rubber box */
+	int			*xsp, *ysp;	/* position and size */
 	int			xc, yc;		/* pixel pos clicked */
-	Time			time;		/* current time in ms */
 {
 	register struct config	*c = &config;
-	struct weeknode		*vert, *horz;	/* week node scan pointers */
+	struct weeknode		*vert, *horz;	/* yov node scan pointers */
 	register struct entry	*ep;		/* entry to test */
-	static struct entry	*last_ep;	/* last entry found (dblclk?)*/
-	static Time		last_time;	/* time of last press */
-	int			d;		/* day counter */
+	int			daysz;		/* horz pixels per day */
+	int			d;		/* day strip counter */
 	int			yd;		/* y start coord of day box */
 	int			x, y;		/* upper left of day bk box */
-	int			ys;		/* height of bar line w/ gaps*/
-	time_t			btime;		/* start time of bar */
-	int			b, e;		/* X start end end pos of bar*/
 	int			xend;		/* right margin of chart */
-	int			u;		/* index into user[] array */
+	int			ys;		/* height of bar line w/ gaps*/
+	time_t			bt;		/* start time of bar */
+	int			b, e;		/* X start end end pos of bar*/
 
 	x  = c->week_margin + c->week_daywidth + c->week_gap;
 	yd = 2*c->week_margin + c->week_title + c->week_hour + c->week_gap + 2;
 	ys = c->week_barheight + 2*c->week_bargap;
 	xend = x + c->week_hourwidth * (c->week_maxhour - c->week_minhour);
-
-	for (d=0; d < c->week_ndays; d++) {
-	    for (y=yd, vert=week.tree[d]; vert; vert=vert->down, y+=ys) {
-		if (yc < y || yc >= y+ys)
-			continue;
-		if (xc < x) {
-			create_list_popup(mainlist, curr_week + d*86400,
-							86400, 0, 0, 0, 0);
-			return;
+	if (xc > xend || yc < c->week_margin + c->week_title)
+		return(M_ILLEGAL);
+	if (warn)
+		*warn = FALSE;
+						/* calc time in calendar */
+	*time = 0;
+	for (y=yd, d=0; d < c->week_ndays; d++) {
+		if (yc < y)
+			break;
+		y += week.nlines[d] * ys + c->week_gap;
+		if (yc < y) {
+			*time = curr_week + d * 86400 + c->week_minhour * 3600+
+					(xc - x) * 3600 / c->week_hourwidth;
+			break;
 		}
-		for (horz=vert; horz; horz=horz->next) {
-			u = name_to_user(horz->entry->user);
-			draw_bar(horz, u > 0 ? user[u].color : 0,
-				 x, y + c->week_bargap);
-
-			ep = horz->entry;
-			btime = TOD(ep->time);
+	}
+						/* find entry under mouse */
+	if (epp)
+	    for (d=0; d < c->week_ndays; d++) {
+		for (y=yd, vert=week.tree[d]; vert; vert=vert->down, y+=ys) {
+		    if (yc < y || yc >= y+ys)
+			continue;
+		    if (xc < x)
+			return(M_OUTSIDE);
+		    for (horz=vert; horz; horz=horz->next) {
+			ep = *epp = horz->entry;
+			bt = TOD(ep->time);
+			e = x + XPOS(TOD(bt));
 			if (config.weekwarn)
-				btime -= ep->early_warn > ep->late_warn
-						? ep->early_warn
-						: ep->late_warn;
-			if (btime < c->week_minhour*3600)
-				btime = c->week_minhour*3600;
-			e = b = x + XPOS(TOD(btime));
+				bt -= ep->early_warn > ep->late_warn ?
+					ep->early_warn : ep->late_warn;
+			if (bt < c->week_minhour*3600)
+				bt = c->week_minhour*3600;
+			b = x + XPOS(TOD(bt));
 			if (!ep->notime)
 				e += ep->length * c->week_hourwidth/3600;
 			if (e < b + MINLEN)
 				e = b + MINLEN;
-			if (!horz->textinside)
-				e += horz->textlen;
-			if (xc >= b && xc < e ||
-			    xc >= xend-MINLEN && b >= xend) {
-				if (time-last_time < DBLTIME && ep == last_ep){
-					if (u >= 0 && user[u].readonly)
-						print_button(week.info,
-						 "Cannot edit, no permission");
-					else
-						create_list_popup(mainlist,
-							0, 0, 0, ep, 0, 0);
-					last_ep = 0;
-					return;
-				} else if (horz->entry->user)
-					print_button(week.info, "%s %s: %s",
-						mktimestring(ep->time, FALSE),
-						horz->entry->user,
-						ep->note ? ep->note : "");
-				else
-					print_button(week.info, "%s %s",
-						mktimestring(ep->time, FALSE),
-						ep->note ? ep->note : "");
-				last_ep   = ep;
-				last_time = time;
-				return;
+			if (trigger)
+				*trigger = horz->trigger;
+			if (warn)
+				*warn = horz->days_warn;
+			if (xp) {
+				*xp   = b + 1;
+				*yp   = y + c->week_gap/2 + 1;
+				*xsp  = e - b - 1;
+				*ysp  = c->week_barheight & ~1;
 			}
+			if (xc < b-1)
+				continue;
+			if (xc < b + (e-b)/3)
+				return(M_LEFT);
+			if (xc < b + (e-b)*2/3)
+				return(M_INSIDE);
+			if (xc < e + 1)
+				return(M_RIGHT);
+			if (!horz->textinside && xc < e + horz->textlen)
+				return(M_INSIDE);
+		    }
 		}
+		yd += week.nlines[d] * ys + c->week_gap;
 	    }
-	    yd += week.nlines[d] * ys + c->week_gap;
-	}
-	print_button(week.info, " ");
+	return(*time ? M_OUTSIDE : M_ILLEGAL);
 }
 
 
@@ -170,7 +188,7 @@ void draw_week_day(day, month, year)
  */
 
 void draw_week_calendar(region)
-	Region region;
+	Region	region;
 {
 	if (!g_init_window(&week, region))
 		return;
@@ -315,6 +333,9 @@ static void draw_week_day_background(wday)
 	hp  =    &holiday[tm->tm_yday];
 	for (i=0; i < 2; i++, shp=hp)
 		if (shp->string && yt < y + yts) {
+			g_setcolor(shp->stringcolor ? shp->stringcolor :
+				   shp->daycolor    ? shp->daycolor
+						    : COL_WDAY);
 #ifdef JAPAN
 			partialstr->strptr = strpool;
 			if ((plen = mixedstrlen_in_pixels(shp->string,
@@ -324,9 +345,6 @@ static void draw_week_day_background(wday)
 							c->week_daywidth-2,
 							FONT_WNOTE);
 
-			g_setcolor(shp->stringcolor ? shp->stringcolor :
-				   shp->daycolor    ? shp->daycolor
-						    : COL_WDAY);
 			g_draw_strpack(c->week_margin +
 				c->week_daywidth-2 - plen, yt, partialstr);
 			yt += font[FONT_WDAY]->max_bounds.ascent +
@@ -340,9 +358,6 @@ static void draw_week_day_background(wday)
 #else
 			strncpy(buf, shp->string, sizeof(buf)-1);
 			truncate_string(buf, c->week_daywidth-2, FONT_WDAY);
-			g_setcolor(shp->stringcolor ? shp->stringcolor :
-				   shp->daycolor    ? shp->daycolor
-						    : COL_WDAY);
 			g_drawtext(c->week_margin + c->week_daywidth-2 -
 					strlen_in_pixels(buf, FONT_WDAY),
 				   yt, buf, strlen(buf));
@@ -426,7 +441,7 @@ static void draw_bar(node, color, x, y)
 		b = 0;
 		e = ee = 9999;
 	}
-	if (config.weekwarn && !ep->notime)
+	if (config.weekwarn && !ep->notime && !node->days_warn)
 	    for (i=0; i < 2; i++)
 		if (w[i] < b && w[i] <= w[1] && b+slope > 0) {
 			point[5].x =
@@ -442,7 +457,7 @@ static void draw_bar(node, color, x, y)
 			point[2].y =
 			point[3].y = y + (c->week_barheight&~1)+1;
 			g_setcolor(COL_WWARN);
-			g_drawpoly(point, 5);
+			g_drawpoly(point, 5, FALSE);
 			g_setcolor(COL_WFRAME);
 			g_drawlines(point, 6);
 		}
@@ -464,8 +479,9 @@ static void draw_bar(node, color, x, y)
 	point[3].y = y + (c->week_barheight&~1)+1;
 	if (ep->notime && !config.week_bignotime)
 		point[3].x = point[5].x = point[0].x;
-	g_setcolor(ep->suspended ? COL_WWARN : COL_WUSER_0 + color);
-	g_drawpoly(point, 6);
+	g_setcolor(ep->suspended || node->days_warn ?
+					COL_WWARN : COL_WUSER_0 + (color & 7));
+	g_drawpoly(point, 6, color > 7);
 	g_setcolor(COL_WFRAME);
 	g_drawlines(point, 7);
 
@@ -530,8 +546,8 @@ int truncate_strpack(partialstr, maxpix, ascfont)
 	     partialstr++) {
 		if (plen + partialstr->pixlen > maxpix) {	/* Truncating*/
 			if (partialstr->asciistr == False) {	/* NON ascii */
-				partialstr->length *= (double)(maxpix-plen) /
-						      (double)partialstr->pixlen;
+				partialstr->length *= (double)(maxpix - plen) /
+						    (double)partialstr->pixlen;
 				if (partialstr->length &= ~1)
 					if (++partialstr < upperpartstr)
 						partialstr->strptr = NULL;
@@ -541,7 +557,7 @@ int truncate_strpack(partialstr, maxpix, ascfont)
 				(void)truncate_string(partialstr->strptr,
 						      maxpix-plen, ascfont);
 				if (partialstr->length =
-							strlen(partialstr->strptr))
+						strlen(partialstr->strptr))
 					if (++partialstr < upperpartstr)
 						partialstr->strptr = NULL;
 				else
@@ -568,7 +584,8 @@ void g_draw_strpack(x, y, partialstr)
 			g_drawtext16(x, y, partialstr->strptr,
 				      partialstr->length/2);
 		else
-			g_drawtext(x, y, partialstr->strptr, partialstr->length);
+			g_drawtext(x, y, partialstr->strptr,
+					 partialstr->length);
 		x += partialstr->pixlen;
 	}
 }
